/*
* glutes_shape.c
*
* Copyright (c) 2005  Joachim Pouderoux   All Rights Reserved.
* Copyright (c) 2003  David Blythe   All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sublicense,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included
* in all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
* THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif


#include "../Inc/GLES/glutes.h"
#include "glutes_internal.h"

#ifndef GLES_CL

#include <stdlib.h>
#include <string.h>
#include <math.h>

#pragma warning(disable: 4305)

#define PI_ 3.14159265358979323846

void FGAPIENTRY
glutSolidTorus(GLfloat ir, GLfloat or, GLint sides, GLint rings)
{
	GLint i, j, k, triangles;
	GLfloat s, t, x, y, z, twopi, nx, ny, nz;
	GLfloat sin_s, cos_s, cos_t, sin_t, twopi_s, twopi_t;
	GLfloat twopi_sides, twopi_rings;
	static GLfloat* v, *n;
	static GLfloat parms[4];
	GLfloat* p, *q;

	if (v) 
	{
		if (parms[0] != ir || parms[1] != or || parms[2] != sides || parms[3] != rings) 
		{
			free(v);
			free(n);
			n = v = 0;

			glVertexPointer(3, GL_FLOAT, 0, 0);
			glNormalPointer(GL_FLOAT, 0, 0);
		}
	}

	if (!v) 
	{
		parms[0] = ir; 
		parms[1] = or; 
		parms[2] = (GLfloat)sides; 
		parms[3] = (GLfloat)rings;

		p = v = (GLfloat*)malloc(sides*(rings+1)*2*3*sizeof *v);
		q = n = (GLfloat*)malloc(sides*(rings+1)*2*3*sizeof *n);

		twopi = 2.0f * (GLfloat)PI_;
		twopi_sides = twopi/sides;
		twopi_rings = twopi/rings;

		for (i = 0; i < sides; i++) 
		{
			for (j = 0; j <= rings; j++) 
			{
				for (k = 1; k >= 0; k--) 
				{
					s = (i + k) % sides + 0.5f;
					t = (GLfloat)( j % rings);

					twopi_s= s*twopi_sides;
					twopi_t = t*twopi_rings;

					cos_s = (GLfloat)cos(twopi_s);
					sin_s = (GLfloat)sin(twopi_s);

					cos_t = (GLfloat)cos(twopi_t);
					sin_t = (GLfloat)sin(twopi_t);

					x = (or+ir*(GLfloat)cos_s)*(GLfloat)cos_t;
					y = (or+ir*(GLfloat)cos_s)*(GLfloat)sin_t;
					z = ir * (GLfloat)sin_s;

					*p++ = x;
					*p++ = y;
					*p++ = z;

					nx = (GLfloat)cos_s*(GLfloat)cos_t;
					ny = (GLfloat)cos_s*(GLfloat)sin_t;
					nz = (GLfloat)sin_s;

					*q++ = nx;
					*q++ = ny;
					*q++ = nz;
				}
			}
		}
	}

	glVertexPointer(3, GL_FLOAT, 0, v);
	glNormalPointer(GL_FLOAT, 0, n);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	triangles = (rings + 1) * 2;

	for(i = 0; i < sides; i++)
		glDrawArrays(GL_TRIANGLE_STRIP, triangles * i, triangles);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}


//-------------------------------------------------------------------------------------------------------------
static GLfloat boxvec[6][3] =
{
	{-1.0, 0.0, 0.0},
	{0.0, 1.0, 0.0},
	{1.0, 0.0, 0.0},
	{0.0, -1.0, 0.0},
	{0.0, 0.0, 1.0},
	{0.0, 0.0, -1.0}
};

static GLushort boxndex [12][3] = 
{
	{0, 1, 2},
	{0, 2, 3},
	{3, 2, 6},
	{3, 6, 7},
	{6, 4, 7},
	{6, 5, 4},
	{4, 5, 1},
	{4, 1, 0},
	{2, 1, 5},
	{2, 5, 6},
	{3, 7, 4},
	{3, 4, 0}
};

static GLushort wireboxndex[6][4] = 
{
	{0, 1, 2, 3},
	{3, 2, 6, 7},
	{7, 6, 5, 4},
	{4, 5, 1, 0},
	{5, 6, 2, 1},
	{7, 4, 0, 3}
};


void FGAPIENTRY		   //x						 y						 z
glutSolidBox(GLfloat Width, GLfloat Depth, GLfloat Height)
{
	int i;
	GLfloat v[8][3];

	v[0][0] = v[1][0] = v[2][0] = v[3][0] = - Width/ 2.0f;
	v[4][0] = v[5][0] = v[6][0] = v[7][0] = Width / 2.0f;
	v[0][1] = v[1][1] = v[4][1] = v[5][1] = -Depth / 2.0f;
	v[2][1] = v[3][1] = v[6][1] = v[7][1] = Depth / 2.0f;
	v[0][2] = v[3][2] = v[4][2] = v[7][2] = -Height / 2.0f;
	v[1][2] = v[2][2] = v[5][2] = v[6][2] = Height / 2.0f;

	glVertexPointer(3, GL_FLOAT, 0, v);
	glEnableClientState (GL_VERTEX_ARRAY);

	for (i = 0; i < 6; i++)
	{
		glNormal3f(boxvec[i][0], boxvec[i][1], boxvec[i][2]);
		glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, boxndex[i*2]);
	}

	glDisableClientState (GL_VERTEX_ARRAY);
}


void FGAPIENTRY      //x						 y						 z
glutWireBox(GLfloat Width, GLfloat Depth, GLfloat Height)
{
	GLfloat v[8][3];
	int i;

	v[0][0] = v[1][0] = v[2][0] = v[3][0] = - Width/ 2.0f;
	v[4][0] = v[5][0] = v[6][0] = v[7][0] = Width / 2.0f;
	v[0][1] = v[1][1] = v[4][1] = v[5][1] = -Depth / 2.0f;
	v[2][1] = v[3][1] = v[6][1] = v[7][1] = Depth / 2.0f;
	v[0][2] = v[3][2] = v[4][2] = v[7][2] = -Height / 2.0f;
	v[1][2] = v[2][2] = v[5][2] = v[6][2] = Height / 2.0f;

	glVertexPointer(3, GL_FLOAT, 0, v);
	glEnableClientState (GL_VERTEX_ARRAY);

	for ( i = 0; i < 6; i++)
	{
		glNormal3f(boxvec[i][0], boxvec[i][1], boxvec[i][2]);
		glDrawElements(GL_LINE_LOOP, 4, GL_UNSIGNED_SHORT, wireboxndex[i]);
	}
	glDisableClientState (GL_VERTEX_ARRAY);
}

//-------------------------------------------------------------------------------------------------------------
void PlotSpherePoints(GLfloat radius, GLint stacks, GLint slices, GLfloat* v, GLfloat* n)
{

	GLint i, j; 
	GLfloat slicestep, stackstep;

	stackstep = ((GLfloat)PI_) / stacks;
	slicestep = 2.0f * ((GLfloat)PI_) / slices;

	for (i = 0; i < stacks; ++i)		
	{
		GLfloat a = i * stackstep;
		GLfloat b = a + stackstep;

		GLfloat s0 =  (GLfloat)sin(a);
		GLfloat s1 =  (GLfloat)sin(b);

		GLfloat c0 =  (GLfloat)cos(a);
		GLfloat c1 =  (GLfloat)cos(b);

		for (j = 0; j <= slices; ++j)		
		{
			GLfloat c = j * slicestep;
			GLfloat x = (GLfloat)cos(c);
			GLfloat y = (GLfloat)sin(c);

			*n = x * s0;
			*v = *n * radius;

			n++;
			v++;

			*n = y * s0;
			*v = *n * radius;

			n++;
			v++;

			*n = c0;
			*v = *n * radius;

			n++;
			v++;

			*n = x * s1;
			*v = *n * radius;

			n++;
			v++;

			*n = y * s1;
			*v = *n * radius;

			n++;
			v++;

			*n = c1;
			*v = *n * radius;

			n++;
			v++;

		}
	}
}


void FGAPIENTRY
glutSolidSphere(GLfloat radius, GLint slices, GLint stacks) 
{
	GLint i, triangles; 
	static GLfloat* v, *n;
	static GLfloat parms[3];

	if (v) 
	{
		if (parms[0] != radius || parms[1] != slices || parms[2] != stacks) 
		{
			free(v); 
			free(n);

			n = v = 0;

			glVertexPointer(3, GL_FLOAT, 0, 0);
			glNormalPointer(GL_FLOAT, 0, 0);
		}
	}

	if (!v) 
	{
		parms[0] = radius; 
		parms[1] = (GLfloat)slices; 
		parms[2] = (GLfloat)stacks;

		v = (GLfloat*)malloc(stacks*(slices+1)*2*3*sizeof *v);
		n = (GLfloat*)malloc(stacks*(slices+1)*2*3*sizeof *n);

		PlotSpherePoints(radius, stacks, slices, v, n);

	}

	glVertexPointer(3, GL_FLOAT, 0, v);
	glNormalPointer(GL_FLOAT, 0, n);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	triangles = (slices + 1) * 2;

	for(i = 0; i < stacks; i++)
		glDrawArrays(GL_TRIANGLE_STRIP, i * triangles, triangles);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}


void FGAPIENTRY
glutWireSphere(GLfloat radius, GLint slices, GLint stacks) 
{
	GLint i, j, f; 
	static GLfloat* v, *n;
	static GLfloat parms[3];

	if (v) 
	{
		if (parms[0] != radius || parms[1] != slices || parms[2] != stacks) 
		{
			free(v); 
			free(n);

			n = v = 0;

			glVertexPointer(3, GL_FLOAT, 0, 0);
			glNormalPointer(GL_FLOAT, 0, 0);
		}
	}

	if (!v) 
	{
		parms[0] = radius; 
		parms[1] = (GLfloat)slices; 
		parms[2] = (GLfloat)stacks;

		v = (GLfloat*)malloc(stacks*(slices+1)*2*3*sizeof *v);
		n = (GLfloat*)malloc(stacks*(slices+1)*2*3*sizeof *n);

		PlotSpherePoints(radius, stacks, slices, v, n);

	}

	glVertexPointer(3, GL_FLOAT, 0, v);
	glNormalPointer(GL_FLOAT, 0, n);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	for(i = 0; i < stacks; ++i)
	{
		f = i * (slices + 1);

		for (j = 0; j <= slices; ++j)
			glDrawArrays(GL_LINE_LOOP, (f + j)*2, 3);
	}

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}

void FGAPIENTRY
glutSolidCone(GLfloat base, GLfloat height, GLint slices, GLint stacks) 
{
	GLint i, j;
	GLfloat twopi, nx, ny, nz;
	static GLfloat* v, *n;
	static GLfloat parms[4];
	GLfloat* p, *q;

	if (v) 
	{
		if (parms[0] != base || parms[1] != height || parms[2] != slices || parms[3] != stacks) 
		{
			free(v); 
			free(n);

			n = v = 0;

			glVertexPointer(3, GL_FLOAT, 0, 0);
			glNormalPointer(GL_FLOAT, 0, 0);
		}
	}

	if ((!v) && (height != 0.0f))
	{
		float phi = (float)atan(base/height);
		float cphi = (GLfloat)cos(phi);
		float sphi= (GLfloat)sin(phi);

		parms[0] = base; 
		parms[1] = height; 
		parms[2] = (GLfloat)slices;
		parms[3] = (GLfloat)stacks;

		p = v = (float*)malloc(stacks*(slices+1)*2*3*sizeof *v);
		q = n = (float*)malloc(stacks*(slices+1)*2*3*sizeof *n);

		twopi = 2.0f * ((float)PI_);

		for (i = 0; i < stacks; i++) 
		{
			GLfloat r = base*(1.0f - (GLfloat)i /stacks);
			GLfloat r1 = base*(1.0f - (GLfloat)(i+1.0)/stacks);
			GLfloat z = height*i /stacks;
			GLfloat z1 = height*(1.0f+i) /stacks;

			for (j = 0; j <= slices; j++) 
			{
				GLfloat theta = j == slices ? 0.f : (GLfloat) j /slices*twopi;
				GLfloat ctheta = (GLfloat)cos(theta);
				GLfloat stheta = (GLfloat)sin(theta);

				nx = ctheta;
				ny = stheta;
				nz = sphi;

				*p++ = r1*nx;
				*p++ = r1*ny;
				*p++ = z1;

				*q++ = nx*cphi;
				*q++ = ny*cphi;
				*q++ = nz;

				*p++ = r*nx;
				*p++ = r*ny;
				*p++ = z;


				*q++ = nx*cphi;
				*q++ = ny*cphi;
				*q++ = nz;

			}
		}
	}

	glVertexPointer(3, GL_FLOAT, 0, v);
	glNormalPointer(GL_FLOAT, 0, n);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	for(i = 0; i < stacks; i++)
		glDrawArrays(GL_TRIANGLE_STRIP, i*(slices+1)*2, (slices+1)*2);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);
}

void FGAPIENTRY
glutSolidCube(GLfloat size) 
{
	static GLfloat v[108];	   // 108 =  6*18

	static const GLfloat cubev[108] = 
	{
		-1., -1., 1.,	/* front */
		1., -1., 1.,
		-1.,  1., 1.,

		1., -1., 1.,
		1.,  1., 1.,
		-1.,  1., 1.,

		-1.,  1., -1.,	/* back */
		1., -1., -1.,
		-1., -1., -1.,

		-1.,  1., -1.,
		1.,  1., -1.,
		1., -1., -1.,

		-1., -1., -1.,	/* left */
		-1., -1.,  1.,
		-1.,  1., -1.,

		-1., -1.,  1.,
		-1.,  1.,  1.,
		-1.,  1., -1.,

		1., -1.,  1.,	/* right */
		1., -1., -1.,
		1.,  1.,  1.,

		1., -1., -1.,
		1.,  1., -1.,
		1.,  1.,  1.,

		-1.,  1.,  1.,	/* top */
		1.,  1.,  1.,
		-1.,  1., -1.,

		1.,  1.,  1.,
		1.,  1., -1.,
		-1.,  1., -1.,

		-1., -1., -1.,	/* bottom */
		1., -1., -1.,
		-1., -1.,  1.,

		1., -1., -1.,
		1., -1.,  1.,
		-1., -1.,  1.,
	};

	static const GLfloat cuben[108] = 
	{
		0., 0., 1.,	/* front */
		0., 0., 1.,
		0., 0., 1.,

		0., 0., 1.,
		0., 0., 1.,
		0., 0., 1.,

		0., 0., -1.,	/* back */
		0., 0., -1.,
		0., 0., -1.,

		0., 0., -1.,
		0., 0., -1.,
		0., 0., -1.,

		-1., 0., 0.,	/* left */
		-1., 0., 0.,
		-1., 0., 0.,

		-1., 0., 0.,
		-1., 0., 0.,
		-1., 0., 0.,

		1., 0., 0.,	/* right */
		1., 0., 0.,
		1., 0., 0.,

		1., 0., 0.,
		1., 0., 0.,
		1., 0., 0.,

		0., 1., 0.,	/* top */
		0., 1., 0.,
		0., 1., 0.,

		0., 1., 0.,
		0., 1., 0.,
		0., 1., 0.,

		0., -1., 0.,	/* bottom */
		0., -1., 0.,
		0., -1., 0.,

		0., -1., 0.,
		0., -1., 0.,
		0., -1., 0.,
	};

	int i;
	size /= 2;

	for(i = 0; i < 108; i++) 
		v[i] = cubev[i] * size;

	glVertexPointer(3, GL_FLOAT, 0, v);
	glNormalPointer(GL_FLOAT, 0, cuben);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	glDrawArrays(GL_TRIANGLES, 0, 36);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}

void FGAPIENTRY
glutWireCube(GLfloat size) 
{
	static GLfloat v[72];

	static const GLfloat cubev[72] = 	  // 72 = 3*6*4
	{
		-1., -1., 1.,	/* front */
		1., -1., 1.,
		1.,  1., 1.,
		-1.,  1., 1.,

		-1.,  1., -1.,	/* back */
		1.,  1., -1.,
		1., -1., -1.,
		-1., -1., -1.,

		-1., -1., -1.,	/* left */
		-1., -1.,  1.,
		-1.,  1.,  1.,
		-1.,  1., -1.,

		1., -1.,  1.,	/* right */
		1., -1., -1.,
		1.,  1., -1.,
		1.,  1.,  1.,

		-1.,  1.,  1.,	/* top */
		1.,  1.,  1.,
		1.,  1., -1.,
		-1.,  1., -1.,

		-1., -1., -1.,	/* bottom */
		1., -1., -1.,
		1., -1.,  1.,
		-1., -1.,  1.,
	};

	static const GLfloat cuben[72] = 
	{
		0., 0., 1.,	/* front */
		0., 0., 1.,
		0., 0., 1.,
		0., 0., 1.,

		0., 0., -1.,	/* back */
		0., 0., -1.,
		0., 0., -1.,
		0., 0., -1.,

		-1., 0., 0.,	/* left */
		-1., 0., 0.,
		-1., 0., 0.,
		-1., 0., 0.,

		1., 0., 0.,	/* right */
		1., 0., 0.,
		1., 0., 0.,
		1., 0., 0.,

		0., 1., 0.,	/* top */
		0., 1., 0.,
		0., 1., 0.,
		0., 1., 0.,

		0., -1., 0.,	/* bottom */
		0., -1., 0.,
		0., -1., 0.,
		0., -1., 0.,
	};

	int i;
	size /= 2;

	for(i = 0; i < 72; i++) 
		v[i] = cubev[i] * size;

	glVertexPointer(3, GL_FLOAT, 0, v);
	glNormalPointer(GL_FLOAT, 0, cuben);

	glEnableClientState (GL_VERTEX_ARRAY);
	glEnableClientState (GL_NORMAL_ARRAY);

	for(i = 0; i < 6; i++)
		glDrawArrays(GL_LINE_LOOP, 4*i, 4);

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_NORMAL_ARRAY);

}

float teapot_vertex[1598 * 3] = {
	8.844996, 0.317460, 15.707068,
	8.730990, 0.317460, 16.022612,
	8.724312, 0.317460, 16.232975,
	8.802066, 0.317460, 16.338156,
	8.941353, 0.317460, 16.338156,
	9.119279, 0.317460, 16.232975,
	9.312946, 0.317460, 16.022612,
	9.499457, 0.317460, 15.707068,
	8.606719, -1.805666, 15.707068,
	8.495678, -1.779248, 16.022612,
	8.489173, -1.777701, 16.232975,
	8.564903, -1.795718, 16.338156,
	8.700570, -1.827994, 16.338156,
	8.873868, -1.869223, 16.232975,
	9.062499, -1.914099, 16.022612,
	9.244161, -1.957318, 15.707068,
	7.926079, -3.742871, 15.707068,
	7.823506, -3.692349, 16.022612,
	7.817499, -3.689390, 16.232975,
	7.887453, -3.723846, 16.338156,
	8.012773, -3.785571, 16.338156,
	8.172854, -3.864419, 16.232975,
	8.347097, -3.950242, 16.022612,
	8.514904, -4.032895, 15.707068,
	6.854365, -5.442868, 15.707068,
	6.765128, -5.371193, 16.022612,
	6.759901, -5.366995, 16.232975,
	6.820761, -5.415877, 16.338156,
	6.929787, -5.503446, 16.338156,
	7.069057, -5.615305, 16.232975,
	7.220648, -5.737062, 16.022612,
	7.366639, -5.854320, 15.707068,
	5.442865, -6.854367, 15.707068,
	5.371192, -6.765130, 16.022612,
	5.366993, -6.759903, 16.232975,
	5.415876, -6.820763, 16.338156,
	5.503445, -6.929789, 16.338156,
	5.615304, -7.069058, 16.232975,
	5.737061, -7.220649, 16.022612,
	5.854318, -7.366639, 15.707068,
	3.742870, -7.926080, 15.707068,
	3.692348, -7.823508, 16.022612,
	3.689389, -7.817499, 16.232975,
	3.723845, -7.887455, 16.338156,
	3.785570, -8.012773, 16.338156,
	3.864418, -8.172854, 16.232975,
	3.950241, -8.347097, 16.022612,
	4.032893, -8.514904, 15.707068,
	1.805665, -8.606719, 15.707068,
	1.779248, -8.495678, 16.022612,
	1.777700, -8.489173, 16.232975,
	1.795717, -8.564903, 16.338156,
	1.827993, -8.700571, 16.338156,
	1.869222, -8.873868, 16.232975,
	1.914098, -9.062499, 16.022612,
	1.957317, -9.244161, 15.707068,
	-0.317460, -8.844996, 15.707068,
	-0.317460, -8.730990, 16.022612,
	-0.317460, -8.724312, 16.232975,
	-0.317460, -8.802066, 16.338156,
	-0.317460, -8.941353, 16.338156,
	-0.317460, -9.119279, 16.232975,
	-0.317460, -9.312946, 16.022612,
	-0.317460, -9.499457, 15.707068,
	-2.626048, -8.606719, 15.707068,
	-2.530961, -8.495678, 16.022612,
	-2.480209, -8.489173, 16.232975,
	-2.465243, -8.564903, 16.338156,
	-2.477513, -8.700570, 16.338156,
	-2.508468, -8.873868, 16.232975,
	-2.549560, -9.062499, 16.022612,
	-2.592237, -9.244161, 15.707068,
	-4.635378, -7.926080, 15.707068,
	-4.489481, -7.823508, 16.022612,
	-4.418183, -7.817499, 16.232975,
	-4.406829, -7.887455, 16.338156,
	-4.440768, -8.012773, 16.338156,
	-4.505347, -8.172854, 16.232975,
	-4.585913, -8.347097, 16.022612,
	-4.667814, -8.514904, 15.707068,
	-6.325071, -6.854366, 15.707068,
	-6.161837, -6.765129, 16.022612,
	-6.092032, -6.759902, 16.232975,
	-6.096938, -6.820762, 16.338156,
	-6.157832, -6.929788, 16.338156,
	-6.255993, -7.069058, 16.232975,
	-6.372703, -7.220649, 16.022612,
	-6.489240, -7.366639, 15.707068,
	-7.674749, -5.442866, 15.707068,
	-7.516843, -5.371193, 16.022612,
	-7.462411, -5.366994, 16.232975,
	-7.490289, -5.415877, 16.338156,
	-7.579309, -5.503446, 16.338156,
	-7.708303, -5.615305, 16.232975,
	-7.856110, -5.737061, 16.022612,
	-8.001558, -5.854319, 15.707068,
	-8.664035, -3.742871, 15.707068,
	-8.523313, -3.692349, 16.022612,
	-8.489968, -3.689390, 16.232975,
	-8.541600, -3.723846, 16.338156,
	-8.655803, -3.785571, 16.338156,
	-8.810177, -3.864419, 16.232975,
	-8.982318, -3.950242, 16.022612,
	-9.149824, -4.032894, 15.707068,
	-9.272550, -1.805666, 15.707068,
	-9.150064, -1.779248, 16.022612,
	-9.135359, -1.777701, 16.232975,
	-9.205592, -1.795718, 16.338156,
	-9.337923, -1.827994, 16.338156,
	-9.509509, -1.869222, 16.232975,
	-9.697510, -1.914099, 16.022612,
	-9.879081, -1.957317, 15.707068,
	-9.479917, 0.317460, 15.707068,
	-9.365911, 0.317460, 16.022612,
	-9.359232, 0.317460, 16.232975,
	-9.436986, 0.317460, 16.338156,
	-9.576272, 0.317460, 16.338156,
	-9.754199, 0.317460, 16.232975,
	-9.947866, 0.317460, 16.022612,
	-10.134377, 0.317460, 15.707068,
	-9.241639, 2.440585, 15.707068,
	-9.130598, 2.414168, 16.022612,
	-9.124094, 2.412620, 16.232975,
	-9.199824, 2.430637, 16.338156,
	-9.335490, 2.462913, 16.338156,
	-9.508788, 2.504142, 16.232975,
	-9.697418, 2.549018, 16.022612,
	-9.879081, 2.592237, 15.707068,
	-8.561000, 4.377790, 15.707068,
	-8.458427, 4.327269, 16.022612,
	-8.452419, 4.324309, 16.232975,
	-8.522374, 4.358765, 16.338156,
	-8.647693, 4.420491, 16.338156,
	-8.807775, 4.499338, 16.232975,
	-8.982018, 4.585161, 16.022612,
	-9.149824, 4.667814, 15.707068,
	-7.489285, 6.077787, 15.707068,
	-7.400049, 6.006112, 16.022612,
	-7.394822, 6.001914, 16.232975,
	-7.455682, 6.050797, 16.338156,
	-7.564708, 6.138365, 16.338156,
	-7.703978, 6.250225, 16.232975,
	-7.855568, 6.371981, 16.022612,
	-8.001558, 6.489239, 15.707068,
	-6.077787, 7.489285, 15.707068,
	-6.006113, 7.400049, 16.022612,
	-6.001914, 7.394822, 16.232975,
	-6.050797, 7.455682, 16.338156,
	-6.138366, 7.564708, 16.338156,
	-6.250225, 7.703977, 16.232975,
	-6.371982, 7.855568, 16.022612,
	-6.489239, 8.001558, 15.707068,
	-4.377790, 8.561000, 15.707068,
	-4.327269, 8.458427, 16.022612,
	-4.324310, 8.452419, 16.232975,
	-4.358766, 8.522374, 16.338156,
	-4.420491, 8.647693, 16.338156,
	-4.499339, 8.807775, 16.232975,
	-4.585162, 8.982018, 16.022612,
	-4.667814, 9.149824, 15.707068,
	-2.440586, 9.241639, 15.707068,
	-2.414168, 9.130598, 16.022612,
	-2.412621, 9.124094, 16.232975,
	-2.430638, 9.199824, 16.338156,
	-2.462914, 9.335491, 16.338156,
	-2.504142, 9.508788, 16.232975,
	-2.549019, 9.697418, 16.022612,
	-2.592237, 9.879081, 15.707068,
	-0.317460, 9.479917, 15.707068,
	-0.317460, 9.365911, 16.022612,
	-0.317460, 9.359232, 16.232975,
	-0.317460, 9.436986, 16.338156,
	-0.317460, 9.576272, 16.338156,
	-0.317460, 9.754199, 16.232975,
	-0.317460, 9.947866, 16.022612,
	-0.317460, 10.134377, 15.707068,
	1.805665, 9.241639, 15.707068,
	1.779248, 9.130598, 16.022612,
	1.777700, 9.124094, 16.232975,
	1.795717, 9.199824, 16.338156,
	1.827993, 9.335490, 16.338156,
	1.869222, 9.508788, 16.232975,
	1.914098, 9.697418, 16.022612,
	1.957317, 9.879081, 15.707068,
	3.742871, 8.561000, 15.707068,
	3.692349, 8.458427, 16.022612,
	3.689389, 8.452419, 16.232975,
	3.723845, 8.522374, 16.338156,
	3.785571, 8.647693, 16.338156,
	3.864418, 8.807775, 16.232975,
	3.950241, 8.982018, 16.022612,
	4.032893, 9.149824, 15.707068,
	5.442866, 7.489285, 15.707068,
	5.371192, 7.400048, 16.022612,
	5.366993, 7.394821, 16.232975,
	5.415876, 7.455681, 16.338156,
	5.503445, 7.564707, 16.338156,
	5.615305, 7.703977, 16.232975,
	5.737061, 7.855568, 16.022612,
	5.854319, 8.001558, 15.707068,
	6.854366, 6.077785, 15.707068,
	6.765129, 6.006112, 16.022612,
	6.759902, 6.001914, 16.232975,
	6.820762, 6.050796, 16.338156,
	6.929788, 6.138365, 16.338156,
	7.069057, 6.250224, 16.232975,
	7.220649, 6.371981, 16.022612,
	7.366639, 6.489238, 15.707068,
	7.926079, 4.377790, 15.707068,
	7.823506, 4.327268, 16.022612,
	7.817499, 4.324309, 16.232975,
	7.887453, 4.358765, 16.338156,
	8.012773, 4.420490, 16.338156,
	8.172854, 4.499338, 16.232975,
	8.347097, 4.585161, 16.022612,
	8.514904, 4.667813, 15.707068,
	8.606719, 2.440585, 15.707068,
	8.495678, 2.414168, 16.022612,
	8.489173, 2.412620, 16.232975,
	8.564903, 2.430637, 16.338156,
	8.700571, 2.462913, 16.338156,
	8.873868, 2.504142, 16.232975,
	9.062499, 2.549018, 16.022612,
	9.244161, 2.592237, 15.707068,
	10.195895, 0.317460, 14.235963,
	10.863714, 0.317460, 12.773441,
	11.474290, 0.317460, 11.328093,
	11.999002, 0.317460, 9.908504,
	12.409234, 0.317460, 8.523259,
	12.676361, 0.317460, 7.180947,
	12.771763, 0.317460, 5.890151,
	9.922488, -2.118696, 14.235963,
	10.572938, -2.273443, 12.773441,
	11.167635, -2.414926, 11.328093,
	11.678704, -2.536512, 9.908504,
	12.078266, -2.631571, 8.523259,
	12.338447, -2.693470, 7.180947,
	12.431368, -2.715577, 5.890151,
	9.141496, -4.341520, 14.235963,
	9.742337, -4.637463, 12.773441,
	10.291677, -4.908039, 11.328093,
	10.763767, -5.140565, 9.908504,
	11.132854, -5.322358, 8.523259,
	11.373191, -5.440735, 7.180946,
	11.459025, -5.483013, 5.890150,
	7.911769, -6.292162, 14.235963,
	8.434498, -6.712011, 12.773441,
	8.912420, -7.095873, 11.328093,
	9.323134, -7.425754, 9.908504,
	9.644238, -7.683661, 8.523259,
	9.853330, -7.851600, 7.180946,
	9.928004, -7.911579, 5.890150,
	6.292161, -7.911770, 14.235963,
	6.712010, -8.434498, 12.773441,
	7.095871, -8.912420, 11.328093,
	7.425753, -9.323135, 9.908504,
	7.683660, -9.644239, 8.523259,
	7.851599, -9.853331, 7.180946,
	7.911578, -9.928006, 5.890150,
	4.341519, -9.141496, 14.235963,
	4.637462, -9.742337, 12.773441,
	4.908037, -10.291677, 11.328093,
	5.140564, -10.763767, 9.908503,
	5.322357, -11.132854, 8.523258,
	5.440734, -11.373191, 7.180946,
	5.483012, -11.459025, 5.890150,
	2.118695, -9.922488, 14.235963,
	2.273443, -10.572938, 12.773441,
	2.414925, -11.167635, 11.328092,
	2.536512, -11.678704, 9.908503,
	2.631571, -12.078266, 8.523258,
	2.693469, -12.338447, 7.180946,
	2.715576, -12.431368, 5.890150,
	-0.317460, -10.195895, 14.235963,
	-0.317460, -10.863714, 12.773441,
	-0.317460, -11.474290, 11.328092,
	-0.317460, -11.999002, 9.908503,
	-0.317460, -12.409234, 8.523258,
	-0.317460, -12.676361, 7.180946,
	-0.317460, -12.771763, 5.890150,
	-2.753617, -9.922488, 14.235963,
	-2.908363, -10.572938, 12.773441,
	-3.049846, -11.167635, 11.328092,
	-3.171432, -11.678704, 9.908503,
	-3.266491, -12.078266, 8.523258,
	-3.328390, -12.338447, 7.180946,
	-3.350497, -12.431368, 5.890150,
	-4.976440, -9.141496, 14.235963,
	-5.272383, -9.742337, 12.773441,
	-5.542959, -10.291677, 11.328093,
	-5.775485, -10.763767, 9.908503,
	-5.957279, -11.132854, 8.523258,
	-6.075655, -11.373191, 7.180946,
	-6.117933, -11.459025, 5.890150,
	-6.927083, -7.911770, 14.235963,
	-7.346931, -8.434498, 12.773441,
	-7.730793, -8.912420, 11.328093,
	-8.060675, -9.323134, 9.908504,
	-8.318581, -9.644238, 8.523259,
	-8.486521, -9.853330, 7.180946,
	-8.546499, -9.928004, 5.890150,
	-8.546690, -6.292161, 14.235963,
	-9.069418, -6.712010, 12.773441,
	-9.547340, -7.095871, 11.328093,
	-9.958055, -7.425754, 9.908504,
	-10.279160, -7.683661, 8.523259,
	-10.488251, -7.851600, 7.180946,
	-10.562926, -7.911578, 5.890150,
	-9.776416, -4.341520, 14.235963,
	-10.377257, -4.637463, 12.773441,
	-10.926597, -4.908038, 11.328093,
	-11.398687, -5.140564, 9.908504,
	-11.767775, -5.322358, 8.523259,
	-12.008111, -5.440735, 7.180946,
	-12.093946, -5.483012, 5.890150,
	-10.557408, -2.118696, 14.235963,
	-11.207857, -2.273443, 12.773441,
	-11.802555, -2.414926, 11.328093,
	-12.313624, -2.536512, 9.908504,
	-12.713186, -2.631571, 8.523259,
	-12.973367, -2.693470, 7.180947,
	-13.066288, -2.715576, 5.890151,
	-10.830815, 0.317460, 14.235963,
	-11.498634, 0.317460, 12.773441,
	-12.109210, 0.317460, 11.328093,
	-12.633923, 0.317460, 9.908504,
	-13.044154, 0.317460, 8.523259,
	-13.311281, 0.317460, 7.180947,
	-13.406683, 0.317460, 5.890151,
	-10.557408, 2.753616, 14.235963,
	-11.207857, 2.908363, 12.773441,
	-11.802555, 3.049845, 11.328093,
	-12.313624, 3.171432, 9.908504,
	-12.713186, 3.266491, 8.523259,
	-12.973367, 3.328390, 7.180947,
	-13.066288, 3.350497, 5.890151,
	-9.776416, 4.976439, 14.235963,
	-10.377257, 5.272382, 12.773441,
	-10.926597, 5.542958, 11.328093,
	-11.398687, 5.775485, 9.908504,
	-11.767775, 5.957278, 8.523259,
	-12.008111, 6.075655, 7.180947,
	-12.093946, 6.117932, 5.890151,
	-8.546690, 6.927082, 14.235963,
	-9.069418, 7.346931, 12.773441,
	-9.547340, 7.730793, 11.328093,
	-9.958055, 8.060674, 9.908504,
	-10.279159, 8.318581, 8.523259,
	-10.488250, 8.486521, 7.180947,
	-10.562924, 8.546499, 5.890151,
	-6.927082, 8.546690, 14.235963,
	-7.346931, 9.069418, 12.773441,
	-7.730792, 9.547340, 11.328093,
	-8.060674, 9.958055, 9.908504,
	-8.318581, 10.279160, 8.523259,
	-8.486520, 10.488251, 7.180947,
	-8.546498, 10.562926, 5.890151,
	-4.976440, 9.776416, 14.235963,
	-5.272383, 10.377257, 12.773441,
	-5.542958, 10.926597, 11.328093,
	-5.775485, 11.398687, 9.908504,
	-5.957278, 11.767775, 8.523260,
	-6.075655, 12.008111, 7.180947,
	-6.117932, 12.093946, 5.890151,
	-2.753616, 10.557408, 14.235963,
	-2.908363, 11.207857, 12.773442,
	-3.049846, 11.802555, 11.328094,
	-3.171432, 12.313624, 9.908504,
	-3.266491, 12.713186, 8.523260,
	-3.328390, 12.973367, 7.180947,
	-3.350497, 13.066288, 5.890151,
	-0.317460, 10.830815, 14.235963,
	-0.317460, 11.498634, 12.773442,
	-0.317460, 12.109210, 11.328094,
	-0.317460, 12.633923, 9.908504,
	-0.317460, 13.044154, 8.523260,
	-0.317460, 13.311281, 7.180947,
	-0.317460, 13.406683, 5.890151,
	2.118696, 10.557408, 14.235963,
	2.273443, 11.207857, 12.773442,
	2.414925, 11.802555, 11.328094,
	2.536512, 12.313624, 9.908504,
	2.631571, 12.713186, 8.523260,
	2.693470, 12.973367, 7.180947,
	2.715576, 13.066288, 5.890151,
	4.341519, 9.776416, 14.235963,
	4.637462, 10.377257, 12.773441,
	4.908038, 10.926597, 11.328093,
	5.140564, 11.398687, 9.908504,
	5.322358, 11.767775, 8.523260,
	5.440734, 12.008111, 7.180947,
	5.483012, 12.093946, 5.890151,
	6.292161, 8.546690, 14.235963,
	6.712010, 9.069418, 12.773441,
	7.095872, 9.547340, 11.328093,
	7.425754, 9.958055, 9.908504,
	7.683660, 10.279159, 8.523259,
	7.851600, 10.488250, 7.180947,
	7.911578, 10.562924, 5.890151,
	7.911769, 6.927080, 14.235963,
	8.434498, 7.346930, 12.773441,
	8.912420, 7.730792, 11.328093,
	9.323135, 8.060674, 9.908504,
	9.644239, 8.318581, 8.523259,
	9.853331, 8.486520, 7.180947,
	9.928006, 8.546498, 5.890151,
	9.141496, 4.976439, 14.235963,
	9.742337, 5.272382, 12.773441,
	10.291677, 5.542958, 11.328093,
	10.763767, 5.775485, 9.908504,
	11.132854, 5.957277, 8.523259,
	11.373191, 6.075655, 7.180947,
	11.459025, 6.117932, 5.890151,
	9.922488, 2.753615, 14.235963,
	10.572938, 2.908363, 12.773441,
	11.167635, 3.049845, 11.328093,
	11.678704, 3.171432, 9.908504,
	12.078266, 3.266491, 8.523259,
	12.338447, 3.328390, 7.180947,
	12.431368, 3.350496, 5.890151,
	12.590498, 0.317460, 4.716700,
	12.123027, 0.317460, 3.714973,
	11.483829, 0.317460, 2.876385,
	10.787392, 0.317460, 2.192349,
	10.148195, 0.317460, 1.654279,
	9.680723, 0.317460, 1.253589,
	9.499457, 0.317460, 0.981692,
	12.254818, -2.673574, 4.716700,
	11.799502, -2.565251, 3.714973,
	11.176927, -2.417136, 2.876385,
	10.498601, -2.255757, 2.192349,
	9.876025, -2.107643, 1.654279,
	9.420712, -1.999320, 1.253589,
	9.244161, -1.957317, 0.981692,
	11.295939, -5.402685, 4.716699,
	10.875351, -5.195525, 3.714973,
	10.300260, -4.912266, 2.876385,
	9.673669, -4.603641, 2.192349,
	9.098577, -4.320381, 1.654279,
	8.677989, -4.113222, 1.253589,
	8.514904, -4.032894, 0.981692,
	9.786122, -7.797620, 4.716699,
	9.420213, -7.503726, 3.714973,
	8.919888, -7.101870, 2.876385,
	8.374757, -6.664027, 2.192349,
	7.874431, -6.262173, 1.654279,
	7.508522, -5.968278, 1.253589,
	7.366639, -5.854320, 0.981692,
	7.797618, -9.786122, 4.716699,
	7.503724, -9.420213, 3.714973,
	7.101868, -8.919888, 2.876385,
	6.664026, -8.374757, 2.192349,
	6.262171, -7.874432, 1.654279,
	5.968277, -7.508523, 1.253589,
	5.854318, -7.366639, 0.981691,
	5.402685, -11.295939, 4.716699,
	5.195524, -10.875351, 3.714973,
	4.912265, -10.300260, 2.876385,
	4.603640, -9.673669, 2.192349,
	4.320380, -9.098577, 1.654279,
	4.113221, -8.677989, 1.253589,
	4.032893, -8.514904, 0.981691,
	2.673573, -12.254818, 4.716699,
	2.565251, -11.799502, 3.714973,
	2.417135, -11.176927, 2.876385,
	2.255757, -10.498601, 2.192349,
	2.107642, -9.876025, 1.654279,
	1.999320, -9.420712, 1.253588,
	1.957317, -9.244161, 0.981691,
	-0.317460, -12.590498, 4.716699,
	-0.317460, -12.123027, 3.714973,
	-0.317460, -11.483829, 2.876385,
	-0.317460, -10.787392, 2.192349,
	-0.317460, -10.148195, 1.654279,
	-0.317460, -9.680723, 1.253588,
	-0.317460, -9.499457, 0.981691,
	-3.308494, -12.254818, 4.716699,
	-3.200171, -11.799502, 3.714973,
	-3.052057, -11.176927, 2.876385,
	-2.890678, -10.498601, 2.192349,
	-2.742563, -9.876025, 1.654279,
	-2.634240, -9.420712, 1.253588,
	-2.592237, -9.244161, 0.981691,
	-6.037606, -11.295939, 4.716699,
	-5.830445, -10.875351, 3.714973,
	-5.547187, -10.300260, 2.876385,
	-5.238561, -9.673669, 2.192349,
	-4.955302, -9.098577, 1.654279,
	-4.748142, -8.677989, 1.253589,
	-4.667814, -8.514904, 0.981691,
	-8.432540, -9.786122, 4.716699,
	-8.138646, -9.420213, 3.714973,
	-7.736790, -8.919888, 2.876385,
	-7.298948, -8.374757, 2.192349,
	-6.897093, -7.874432, 1.654279,
	-6.603199, -7.508523, 1.253589,
	-6.489240, -7.366639, 0.981691,
	-10.421042, -7.797618, 4.716699,
	-10.055133, -7.503725, 3.714973,
	-9.554809, -7.101869, 2.876385,
	-9.009677, -6.664027, 2.192349,
	-8.509352, -6.262171, 1.654279,
	-8.143443, -5.968277, 1.253589,
	-8.001558, -5.854319, 0.981692,
	-11.930860, -5.402685, 4.716699,
	-11.510270, -5.195525, 3.714973,
	-10.935181, -4.912266, 2.876385,
	-10.308589, -4.603640, 2.192349,
	-9.733498, -4.320381, 1.654279,
	-9.312909, -4.113221, 1.253589,
	-9.149824, -4.032893, 0.981692,
	-12.889738, -2.673573, 4.716700,
	-12.434422, -2.565251, 3.714973,
	-11.811847, -2.417135, 2.876385,
	-11.133521, -2.255757, 2.192349,
	-10.510946, -2.107642, 1.654279,
	-10.055632, -1.999320, 1.253589,
	-9.879081, -1.957317, 0.981692,
	-13.225418, 0.317460, 4.716700,
	-12.757947, 0.317460, 3.714973,
	-12.118750, 0.317460, 2.876385,
	-11.422312, 0.317460, 2.192349,
	-10.783115, 0.317460, 1.654279,
	-10.315643, 0.317460, 1.253589,
	-10.134377, 0.317460, 0.981692,
	-12.889738, 3.308494, 4.716700,
	-12.434422, 3.200171, 3.714973,
	-11.811847, 3.052057, 2.876385,
	-11.133521, 2.890678, 2.192349,
	-10.510946, 2.742563, 1.654279,
	-10.055632, 2.634240, 1.253589,
	-9.879081, 2.592237, 0.981692,
	-11.930860, 6.037605, 4.716700,
	-11.510270, 5.830445, 3.714973,
	-10.935181, 5.547186, 2.876385,
	-10.308589, 5.238561, 2.192350,
	-9.733498, 4.955301, 1.654280,
	-9.312909, 4.748142, 1.253589,
	-9.149824, 4.667814, 0.981692,
	-10.421042, 8.432540, 4.716700,
	-10.055133, 8.138646, 3.714973,
	-9.554808, 7.736790, 2.876385,
	-9.009677, 7.298947, 2.192350,
	-8.509352, 6.897093, 1.654280,
	-8.143443, 6.603199, 1.253589,
	-8.001558, 6.489240, 0.981692,
	-8.432539, 10.421042, 4.716700,
	-8.138645, 10.055133, 3.714974,
	-7.736790, 9.554809, 2.876386,
	-7.298947, 9.009677, 2.192350,
	-6.897092, 8.509352, 1.654280,
	-6.603199, 8.143443, 1.253589,
	-6.489239, 8.001558, 0.981692,
	-6.037606, 11.930860, 4.716700,
	-5.830445, 11.510270, 3.714974,
	-5.547186, 10.935181, 2.876386,
	-5.238561, 10.308589, 2.192350,
	-4.955301, 9.733498, 1.654280,
	-4.748142, 9.312909, 1.253589,
	-4.667814, 9.149824, 0.981692,
	-3.308494, 12.889738, 4.716700,
	-3.200171, 12.434422, 3.714974,
	-3.052056, 11.811847, 2.876386,
	-2.890677, 11.133521, 2.192350,
	-2.742563, 10.510946, 1.654280,
	-2.634240, 10.055632, 1.253589,
	-2.592237, 9.879081, 0.981692,
	-0.317460, 13.225418, 4.716700,
	-0.317460, 12.757947, 3.714974,
	-0.317460, 12.118750, 2.876386,
	-0.317460, 11.422312, 2.192350,
	-0.317460, 10.783115, 1.654280,
	-0.317460, 10.315643, 1.253589,
	-0.317460, 10.134377, 0.981692,
	2.673574, 12.889738, 4.716700,
	2.565251, 12.434422, 3.714974,
	2.417136, 11.811847, 2.876386,
	2.255757, 11.133521, 2.192350,
	2.107643, 10.510946, 1.654280,
	1.999320, 10.055632, 1.253589,
	1.957317, 9.879081, 0.981692,
	5.402685, 11.930860, 4.716700,
	5.195525, 11.510270, 3.714974,
	4.912266, 10.935181, 2.876386,
	4.603640, 10.308589, 2.192350,
	4.320381, 9.733498, 1.654280,
	4.113221, 9.312909, 1.253589,
	4.032893, 9.149824, 0.981692,
	7.797619, 10.421042, 4.716700,
	7.503726, 10.055133, 3.714974,
	7.101870, 9.554808, 2.876386,
	6.664027, 9.009677, 2.192350,
	6.262172, 8.509352, 1.654280,
	5.968277, 8.143443, 1.253589,
	5.854319, 8.001558, 0.981692,
	9.786122, 8.432539, 4.716700,
	9.420213, 8.138645, 3.714973,
	8.919888, 7.736789, 2.876385,
	8.374757, 7.298947, 2.192350,
	7.874431, 6.897091, 1.654280,
	7.508522, 6.603198, 1.253589,
	7.366639, 6.489239, 0.981692,
	11.295939, 6.037605, 4.716700,
	10.875351, 5.830445, 3.714973,
	10.300260, 5.547185, 2.876385,
	9.673669, 5.238560, 2.192350,
	9.098577, 4.955301, 1.654280,
	8.677989, 4.748141, 1.253589,
	8.514904, 4.667814, 0.981692,
	12.254818, 3.308494, 4.716700,
	11.799502, 3.200171, 3.714973,
	11.176927, 3.052056, 2.876385,
	10.498601, 2.890677, 2.192349,
	9.876025, 2.742563, 1.654279,
	9.420712, 2.634240, 1.253589,
	9.244161, 2.592237, 0.981692,
	9.445079, 0.317460, 0.772760,
	9.184629, 0.317460, 0.572415,
	8.572145, 0.317460, 0.389242,
	7.461658, 0.317460, 0.231828,
	5.707207, 0.317460, 0.108759,
	3.162823, 0.317460, 0.028621,
	-0.317460, 0.317460, 0.000000,
	9.191195, -1.944716, 0.772760,
	8.937519, -1.884365, 0.572415,
	8.340963, -1.742440, 0.389242,
	7.259357, -1.485118, 0.231828,
	5.550530, -1.078577, 0.108759,
	3.072315, -0.488991, 0.028621,
	8.465978, -4.008796, 0.772760,
	8.231649, -3.893378, 0.572415,
	7.680592, -3.621957, 0.389242,
	6.681479, -3.129847, 0.231828,
	5.102984, -2.352364, 0.108759,
	2.813780, -1.224823, 0.028621,
	7.324073, -5.820132, 0.772760,
	7.120210, -5.656390, 0.572415,
	6.640793, -5.271328, 0.389242,
	5.771572, -4.573180, 0.231828,
	4.398291, -3.470177, 0.108759,
	2.406698, -1.870552, 0.028621,
	5.820130, -7.324074, 0.772760,
	5.656389, -7.120210, 0.572415,
	5.271328, -6.640793, 0.389242,
	4.573179, -5.771572, 0.231828,
	3.470176, -4.398291, 0.108759,
	1.870552, -2.406698, 0.028621,
	4.008795, -8.465978, 0.772760,
	3.893378, -8.231649, 0.572415,
	3.621956, -7.680592, 0.389242,
	3.129846, -6.681479, 0.231828,
	2.352363, -5.102984, 0.108759,
	1.224823, -2.813780, 0.028621,
	1.944716, -9.191195, 0.772760,
	1.884365, -8.937519, 0.572415,
	1.742440, -8.340963, 0.389242,
	1.485118, -7.259357, 0.231828,
	1.078576, -5.550530, 0.108759,
	0.488991, -3.072315, 0.028621,
	-0.317460, -9.445079, 0.772760,
	-0.317460, -9.184629, 0.572415,
	-0.317460, -8.572145, 0.389242,
	-0.317460, -7.461658, 0.231828,
	-0.317460, -5.707207, 0.108759,
	-0.317460, -3.162823, 0.028621,
	-2.579637, -9.191195, 0.772760,
	-2.519286, -8.937519, 0.572415,
	-2.377361, -8.340963, 0.389242,
	-2.120039, -7.259357, 0.231828,
	-1.713497, -5.550530, 0.108759,
	-1.123912, -3.072315, 0.028621,
	-4.643717, -8.465978, 0.772760,
	-4.528299, -8.231649, 0.572415,
	-4.256877, -7.680592, 0.389242,
	-3.764767, -6.681479, 0.231828,
	-2.987284, -5.102984, 0.108759,
	-1.859743, -2.813780, 0.028621,
	-6.455052, -7.324074, 0.772760,
	-6.291310, -7.120210, 0.572415,
	-5.906250, -6.640793, 0.389242,
	-5.208100, -5.771572, 0.231828,
	-4.105097, -4.398291, 0.108759,
	-2.505473, -2.406698, 0.028621,
	-7.958995, -5.820130, 0.772760,
	-7.755131, -5.656389, 0.572415,
	-7.275714, -5.271328, 0.389242,
	-6.406493, -4.573179, 0.231828,
	-5.033212, -3.470176, 0.108759,
	-3.041619, -1.870552, 0.028621,
	-9.100898, -4.008796, 0.772760,
	-8.866569, -3.893378, 0.572415,
	-8.315513, -3.621956, 0.389242,
	-7.316400, -3.129846, 0.231828,
	-5.737905, -2.352363, 0.108759,
	-3.448700, -1.224823, 0.028621,
	-9.826116, -1.944716, 0.772760,
	-9.572439, -1.884365, 0.572415,
	-8.975883, -1.742440, 0.389242,
	-7.894279, -1.485118, 0.231828,
	-6.185452, -1.078576, 0.108759,
	-3.707236, -0.488991, 0.028621,
	-10.079999, 0.317460, 0.772760,
	-9.819550, 0.317460, 0.572415,
	-9.207066, 0.317460, 0.389242,
	-8.096580, 0.317460, 0.231828,
	-6.342128, 0.317460, 0.108759,
	-3.797744, 0.317460, 0.028621,
	-9.826116, 2.579637, 0.772760,
	-9.572439, 2.519286, 0.572415,
	-8.975883, 2.377361, 0.389242,
	-7.894279, 2.120039, 0.231828,
	-6.185452, 1.713497, 0.108759,
	-3.707236, 1.123912, 0.028621,
	-9.100898, 4.643716, 0.772761,
	-8.866569, 4.528299, 0.572415,
	-8.315513, 4.256877, 0.389242,
	-7.316400, 3.764767, 0.231828,
	-5.737905, 2.987284, 0.108759,
	-3.448700, 1.859743, 0.028621,
	-7.958995, 6.455052, 0.772761,
	-7.755131, 6.291310, 0.572415,
	-7.275714, 5.906250, 0.389242,
	-6.406493, 5.208100, 0.231828,
	-5.033212, 4.105097, 0.108759,
	-3.041619, 2.505473, 0.028621,
	-6.455051, 7.958994, 0.772761,
	-6.291310, 7.755131, 0.572415,
	-5.906249, 7.275714, 0.389243,
	-5.208100, 6.406493, 0.231828,
	-4.105097, 5.033212, 0.108759,
	-2.505473, 3.041619, 0.028621,
	-4.643716, 9.100898, 0.772761,
	-4.528299, 8.866569, 0.572415,
	-4.256877, 8.315513, 0.389243,
	-3.764767, 7.316400, 0.231828,
	-2.987284, 5.737905, 0.108759,
	-1.859743, 3.448700, 0.028621,
	-2.579637, 9.826116, 0.772761,
	-2.519285, 9.572439, 0.572415,
	-2.377360, 8.975883, 0.389243,
	-2.120039, 7.894279, 0.231828,
	-1.713497, 6.185452, 0.108759,
	-1.123912, 3.707236, 0.028621,
	-0.317460, 10.079999, 0.772761,
	-0.317460, 9.819550, 0.572415,
	-0.317460, 9.207066, 0.389243,
	-0.317460, 8.096580, 0.231828,
	-0.317460, 6.342128, 0.108759,
	-0.317460, 3.797744, 0.028621,
	1.944716, 9.826116, 0.772761,
	1.884365, 9.572439, 0.572415,
	1.742440, 8.975883, 0.389243,
	1.485118, 7.894279, 0.231828,
	1.078577, 6.185452, 0.108759,
	0.488991, 3.707236, 0.028621,
	4.008796, 9.100898, 0.772761,
	3.893378, 8.866569, 0.572415,
	3.621957, 8.315513, 0.389243,
	3.129847, 7.316400, 0.231828,
	2.352364, 5.737905, 0.108759,
	1.224823, 3.448700, 0.028621,
	5.820131, 7.958994, 0.772761,
	5.656390, 7.755131, 0.572415,
	5.271328, 7.275714, 0.389243,
	4.573180, 6.406493, 0.231828,
	3.470177, 5.033212, 0.108759,
	1.870552, 3.041619, 0.028621,
	7.324073, 6.455050, 0.772761,
	7.120210, 6.291310, 0.572415,
	6.640793, 5.906249, 0.389242,
	5.771572, 5.208100, 0.231828,
	4.398291, 4.105097, 0.108759,
	2.406698, 2.505473, 0.028621,
	8.465978, 4.643716, 0.772761,
	8.231649, 4.528299, 0.572415,
	7.680592, 4.256877, 0.389242,
	6.681479, 3.764767, 0.231828,
	5.102984, 2.987284, 0.108759,
	2.813780, 1.859743, 0.028621,
	9.191195, 2.579637, 0.772760,
	8.937519, 2.519285, 0.572415,
	8.340963, 2.377360, 0.389242,
	7.259357, 2.120039, 0.231828,
	5.550530, 1.713497, 0.108759,
	3.072315, 1.123912, 0.028621,
	-10.788839, 0.317460, 13.252840,
	-12.630108, 0.317460, 13.248547,
	-14.219512, 0.317460, 13.218494,
	-15.545609, 0.317460, 13.136926,
	-16.596945, 0.317460, 12.978080,
	-17.362074, 0.317460, 12.716200,
	-17.829546, 0.317460, 12.325526,
	-17.987913, 0.317460, 11.780301,
	-10.752586, -0.403783, 13.334408,
	-12.636132, -0.403783, 13.329877,
	-14.259572, -0.403783, 13.298161,
	-15.612089, -0.403783, 13.212074,
	-16.682873, -0.403783, 13.044430,
	-17.461107, -0.403783, 12.768043,
	-17.935978, -0.403783, 12.355729,
	-18.096672, -0.403783, 11.780301,
	-10.659092, -0.884612, 13.544771,
	-12.651669, -0.884612, 13.539626,
	-14.362879, -0.884612, 13.503617,
	-15.783542, -0.884612, 13.405877,
	-16.904480, -0.884612, 13.215541,
	-17.716515, -0.884612, 12.901743,
	-18.210466, -0.884612, 12.433617,
	-18.377155, -0.884612, 11.780301,
	-10.531253, -1.125026, 13.832410,
	-12.672913, -1.125026, 13.826427,
	-14.504135, -1.125026, 13.784547,
	-16.017977, -1.125026, 13.670873,
	-17.207493, -1.125026, 13.449508,
	-18.065742, -1.125026, 13.084557,
	-18.585785, -1.125026, 12.540118,
	-18.760674, -1.125026, 11.780301,
	-10.391965, -1.125026, 14.145807,
	-12.696059, -1.125026, 14.138909,
	-14.658042, -1.125026, 14.090634,
	-16.273405, -1.125026, 13.959600,
	-17.537642, -1.125026, 13.704431,
	-18.446245, -1.125026, 13.283743,
	-18.994713, -1.125026, 12.656158,
	-19.178535, -1.125026, 11.780301,
	-10.264126, -0.884612, 14.433445,
	-12.717305, -0.884612, 14.425711,
	-14.799299, -0.884612, 14.371563,
	-16.507837, -0.884612, 14.224596,
	-17.840654, -0.884612, 13.938397,
	-18.795475, -0.884612, 13.466556,
	-19.370029, -0.884612, 12.762660,
	-19.562056, -0.884612, 11.780301,
	-10.170631, -0.403783, 14.643807,
	-12.732841, -0.403783, 14.635460,
	-14.902605, -0.403783, 14.577019,
	-16.679293, -0.403783, 14.418400,
	-18.062262, -0.403783, 14.109509,
	-19.050880, -0.403783, 13.600255,
	-19.644516, -0.403783, 12.840549,
	-19.842539, -0.403783, 11.780301,
	-10.134377, 0.317460, 14.725376,
	-12.738866, 0.317460, 14.716790,
	-14.942665, 0.317460, 14.656687,
	-16.745773, 0.317460, 14.493548,
	-18.148190, 0.317460, 14.175858,
	-19.149916, 0.317460, 13.652097,
	-19.750952, 0.317460, 12.870751,
	-19.951296, 0.317460, 11.780301,
	-10.170631, 1.038703, 14.643807,
	-12.732841, 1.038703, 14.635460,
	-14.902605, 1.038703, 14.577019,
	-16.679293, 1.038703, 14.418400,
	-18.062262, 1.038703, 14.109509,
	-19.050880, 1.038703, 13.600255,
	-19.644516, 1.038703, 12.840549,
	-19.842539, 1.038703, 11.780301,
	-10.264126, 1.519531, 14.433445,
	-12.717305, 1.519531, 14.425711,
	-14.799299, 1.519531, 14.371563,
	-16.507837, 1.519531, 14.224596,
	-17.840654, 1.519531, 13.938397,
	-18.795475, 1.519531, 13.466556,
	-19.370029, 1.519531, 12.762660,
	-19.562056, 1.519531, 11.780301,
	-10.391965, 1.759946, 14.145807,
	-12.696059, 1.759946, 14.138909,
	-14.658042, 1.759946, 14.090634,
	-16.273405, 1.759946, 13.959600,
	-17.537642, 1.759946, 13.704431,
	-18.446245, 1.759946, 13.283743,
	-18.994713, 1.759946, 12.656158,
	-19.178535, 1.759946, 11.780301,
	-10.531253, 1.759946, 13.832410,
	-12.672913, 1.759946, 13.826427,
	-14.504135, 1.759946, 13.784547,
	-16.017977, 1.759946, 13.670873,
	-17.207493, 1.759946, 13.449508,
	-18.065742, 1.759946, 13.084557,
	-18.585785, 1.759946, 12.540118,
	-18.760674, 1.759946, 11.780301,
	-10.659092, 1.519531, 13.544771,
	-12.651669, 1.519531, 13.539626,
	-14.362879, 1.519531, 13.503617,
	-15.783542, 1.519531, 13.405877,
	-16.904480, 1.519531, 13.215541,
	-17.716515, 1.519531, 12.901743,
	-18.210466, 1.519531, 12.433617,
	-18.377155, 1.519531, 11.780301,
	-10.752586, 1.038703, 13.334408,
	-12.636132, 1.038703, 13.329877,
	-14.259572, 1.038703, 13.298161,
	-15.612089, 1.038703, 13.212074,
	-16.682873, 1.038703, 13.044430,
	-17.461107, 1.038703, 12.768043,
	-17.935978, 1.038703, 12.355729,
	-18.096672, 1.038703, 11.780301,
	-17.905867, 0.317460, 11.067645,
	-17.652096, 0.317460, 10.226195,
	-17.215153, 0.317460, 9.307468,
	-16.583590, 0.317460, 8.362984,
	-15.745955, 0.317460, 7.444257,
	-14.690801, 0.317460, 6.602807,
	-13.406683, 0.317460, 5.890151,
	-18.011351, -0.403783, 11.038075,
	-17.747961, -0.403783, 10.176096,
	-17.295374, -0.403783, 9.243259,
	-16.642458, -0.403783, 8.288469,
	-15.778084, -0.403783, 7.360627,
	-14.691118, -0.403783, 6.508634,
	-13.370431, -0.403783, 5.781392,
	-18.283382, -0.884612, 10.961822,
	-17.995188, -0.884612, 10.046893,
	-17.502262, -0.884612, 9.077666,
	-16.794285, -0.884612, 8.096302,
	-15.860947, -0.884612, 7.144948,
	-14.691936, -0.884612, 6.265767,
	-13.276937, -0.884612, 5.500908,
	-18.655348, -1.125026, 10.857557,
	-18.333239, -1.125026, 9.870228,
	-17.785149, -1.125026, 8.851247,
	-17.001884, -1.125026, 7.833541,
	-15.974251, -1.125026, 6.850043,
	-14.693055, -1.125026, 5.933682,
	-13.149097, -1.125026, 5.117390,
	-19.060621, -1.125026, 10.743955,
	-18.701557, -1.125026, 9.677743,
	-18.093367, -1.125026, 8.604549,
	-17.228075, -1.125026, 7.547250,
	-16.097702, -1.125026, 6.528727,
	-14.694273, -1.125026, 5.571859,
	-13.009809, -1.125026, 4.699528,
	-19.432587, -0.884612, 10.639689,
	-19.039604, -0.884612, 9.501079,
	-18.376253, -0.884612, 8.378128,
	-17.435675, -0.884612, 7.284491,
	-16.211006, -0.884611, 6.233820,
	-14.695390, -0.884611, 5.239775,
	-12.881970, -0.884611, 4.316009,
	-19.704617, -0.403783, 10.563436,
	-19.286835, -0.403783, 9.371878,
	-18.583141, -0.403783, 8.212537,
	-17.587502, -0.403783, 7.092322,
	-16.293871, -0.403783, 6.018143,
	-14.696208, -0.403783, 4.996907,
	-12.788476, -0.403783, 4.035526,
	-19.810102, 0.317460, 10.533868,
	-19.382698, 0.317460, 9.321778,
	-18.663363, 0.317460, 8.148327,
	-17.646372, 0.317460, 7.017808,
	-16.326002, 0.317460, 5.934513,
	-14.696526, 0.317460, 4.902735,
	-12.752222, 0.317460, 3.926767,
	-19.704617, 1.038703, 10.563436,
	-19.286835, 1.038703, 9.371878,
	-18.583141, 1.038703, 8.212537,
	-17.587502, 1.038703, 7.092322,
	-16.293871, 1.038703, 6.018143,
	-14.696208, 1.038703, 4.996907,
	-12.788476, 1.038703, 4.035526,
	-19.432587, 1.519531, 10.639689,
	-19.039604, 1.519532, 9.501079,
	-18.376253, 1.519532, 8.378128,
	-17.435675, 1.519532, 7.284491,
	-16.211006, 1.519532, 6.233820,
	-14.695390, 1.519532, 5.239776,
	-12.881970, 1.519532, 4.316009,
	-19.060621, 1.759946, 10.743955,
	-18.701557, 1.759946, 9.677743,
	-18.093367, 1.759946, 8.604549,
	-17.228075, 1.759946, 7.547250,
	-16.097702, 1.759946, 6.528727,
	-14.694273, 1.759946, 5.571859,
	-13.009809, 1.759946, 4.699528,
	-18.655348, 1.759946, 10.857557,
	-18.333239, 1.759946, 9.870228,
	-17.785149, 1.759946, 8.851247,
	-17.001884, 1.759946, 7.833541,
	-15.974251, 1.759946, 6.850043,
	-14.693055, 1.759946, 5.933682,
	-13.149097, 1.759946, 5.117390,
	-18.283382, 1.519531, 10.961822,
	-17.995188, 1.519531, 10.046893,
	-17.502262, 1.519531, 9.077668,
	-16.794285, 1.519531, 8.096302,
	-15.860947, 1.519532, 7.144950,
	-14.691936, 1.519532, 6.265767,
	-13.276937, 1.519532, 5.500908,
	-18.011351, 1.038703, 11.038075,
	-17.747961, 1.038703, 10.176096,
	-17.295374, 1.038703, 9.243259,
	-16.642458, 1.038703, 8.288469,
	-15.778084, 1.038703, 7.360627,
	-14.691118, 1.038703, 6.508634,
	-13.370431, 1.038703, 5.781392,
	10.808380, 0.317460, 9.326072,
	12.888156, 0.317460, 9.576504,
	14.223789, 0.317460, 10.247660,
	15.032802, 0.317460, 11.219335,
	15.532713, 0.317460, 12.371320,
	15.941034, 0.317460, 13.583408,
	16.475286, 0.317460, 14.735394,
	17.352991, 0.317460, 15.707068,
	10.808380, -1.269275, 9.026985,
	12.947766, -1.214673, 9.318240,
	14.314474, -1.073859, 10.038783,
	15.137439, -0.881317, 11.063648,
	15.645594, -0.671533, 12.267872,
	16.067867, -0.478992, 13.526493,
	16.633196, -0.338178, 14.714546,
	17.570511, -0.283576, 15.707068,
	10.808380, -2.327097, 8.255656,
	13.101500, -2.236095, 8.652195,
	14.548347, -2.001405, 9.500099,
	15.407291, -1.680502, 10.662140,
	15.936707, -1.330862, 12.001086,
	16.394960, -1.009959, 13.379709,
	17.040426, -0.775269, 14.660779,
	18.131475, -0.684267, 15.707068,
	10.808380, -2.856009, 7.200981,
	13.311708, -2.746806, 7.741479,
	14.868133, -2.465178, 8.763533,
	15.776275, -2.080095, 10.113138,
	16.334761, -1.660526, 11.636296,
	16.842213, -1.275443, 13.179006,
	17.597256, -0.993815, 14.587263,
	18.898512, -0.884612, 15.707068,
	10.808380, -2.856009, 6.051857,
	13.540741, -2.746806, 6.749207,
	15.216555, -2.465178, 7.961003,
	16.178301, -2.080095, 9.514974,
	16.768459, -1.660526, 11.238839,
	17.329515, -1.275443, 12.960328,
	18.203949, -0.993815, 14.507162,
	19.734236, -0.884612, 15.707068,
	10.808380, -2.327097, 4.997183,
	13.750950, -2.236095, 5.838491,
	15.536338, -2.001405, 7.224437,
	16.547283, -1.680502, 8.965973,
	17.166515, -1.330862, 10.874050,
	17.776768, -1.009959, 12.759624,
	18.760777, -0.775269, 14.433646,
	20.501274, -0.684267, 15.707068,
	10.808380, -1.269274, 4.225854,
	13.904683, -1.214673, 5.172446,
	15.770210, -1.073859, 6.685753,
	16.817135, -0.881317, 8.564465,
	17.457628, -0.671533, 10.607265,
	18.103863, -0.478991, 12.612840,
	19.168011, -0.338177, 14.379879,
	21.062241, -0.283576, 15.707068,
	10.808380, 0.317460, 3.926767,
	13.964294, 0.317460, 4.914183,
	15.860895, 0.317460, 6.476876,
	16.921772, 0.317460, 8.408777,
	17.570507, 0.317460, 10.503817,
	18.230694, 0.317460, 12.555924,
	19.325914, 0.317460, 14.359031,
	21.279758, 0.317460, 15.707068,
	10.808380, 1.904195, 4.225854,
	13.904683, 1.849593, 5.172446,
	15.770210, 1.708779, 6.685753,
	16.817135, 1.516237, 8.564465,
	17.457628, 1.306453, 10.607265,
	18.103863, 1.113911, 12.612840,
	19.168009, 0.973097, 14.379879,
	21.062239, 0.918495, 15.707068,
	10.808380, 2.962018, 4.997184,
	13.750950, 2.871015, 5.838492,
	15.536338, 2.636325, 7.224437,
	16.547283, 2.315422, 8.965973,
	17.166513, 1.965781, 10.874050,
	17.776768, 1.644879, 12.759624,
	18.760777, 1.410188, 14.433646,
	20.501274, 1.319186, 15.707068,
	10.808380, 3.490929, 6.051858,
	13.540741, 3.381726, 6.749207,
	15.216555, 3.100098, 7.961004,
	16.178301, 2.715014, 9.514974,
	16.768459, 2.295446, 11.238839,
	17.329515, 1.910362, 12.960328,
	18.203947, 1.628734, 14.507162,
	19.734236, 1.519531, 15.707068,
	10.808380, 3.490929, 7.200981,
	13.311708, 3.381726, 7.741479,
	14.868133, 3.100098, 8.763533,
	15.776275, 2.715014, 10.113138,
	16.334761, 2.295446, 11.636296,
	16.842213, 1.910362, 13.179006,
	17.597254, 1.628734, 14.587263,
	18.898512, 1.519531, 15.707068,
	10.808380, 2.962018, 8.255656,
	13.101500, 2.871015, 8.652195,
	14.548347, 2.636325, 9.500100,
	15.407291, 2.315422, 10.662140,
	15.936706, 1.965781, 12.001086,
	16.394960, 1.644879, 13.379709,
	17.040426, 1.410188, 14.660779,
	18.131475, 1.319186, 15.707068,
	10.808380, 1.904194, 9.026985,
	12.947766, 1.849593, 9.318240,
	14.314474, 1.708779, 10.038783,
	15.137439, 1.516237, 11.063648,
	15.645594, 1.306453, 12.267872,
	16.067865, 1.113911, 13.526493,
	16.633194, 0.973097, 14.714546,
	17.570507, 0.918495, 15.707068,
	17.629658, 0.317460, 15.887380,
	17.883429, 0.317460, 16.007586,
	18.091408, 0.317460, 16.067690,
	18.230694, 0.317460, 16.067690,
	18.278393, 0.317460, 16.007586,
	18.211613, 0.317460, 15.887380,
	18.007452, 0.317460, 15.707068,
	17.860283, -0.270259, 15.890232,
	18.115904, -0.235914, 16.012938,
	18.316536, -0.188952, 16.074823,
	18.441341, -0.137786, 16.075539,
	18.469490, -0.090824, 16.014721,
	18.380142, -0.056479, 15.892016,
	18.152464, -0.043162, 15.707068,
	18.455048, -0.662071, 15.897593,
	18.715439, -0.604829, 16.026737,
	18.897125, -0.526561, 16.093224,
	18.984591, -0.441282, 16.095776,
	18.962313, -0.363013, 16.033119,
	18.814770, -0.305772, 15.903975,
	18.526442, -0.283576, 15.707068,
	19.268301, -0.857977, 15.907656,
	19.535213, -0.789287, 16.045605,
	19.690996, -0.695365, 16.118382,
	19.727402, -0.593031, 16.123451,
	19.636171, -0.499108, 16.058277,
	19.409056, -0.430418, 15.920328,
	19.037800, -0.403783, 15.707068,
	20.154381, -0.857977, 15.918620,
	20.428402, -0.789287, 16.066162,
	20.555962, -0.695365, 16.145794,
	20.536732, -0.593031, 16.153603,
	20.370375, -0.499108, 16.085688,
	20.056561, -0.430418, 15.938146,
	19.594950, -0.403783, 15.707068,
	20.967634, -0.662071, 15.928683,
	21.248175, -0.604829, 16.085032,
	21.349833, -0.526561, 16.170952,
	21.279541, -0.441282, 16.181278,
	21.044233, -0.363013, 16.110847,
	20.650845, -0.305772, 15.954498,
	20.106306, -0.283576, 15.707068,
	21.562401, -0.270259, 15.936043,
	21.847712, -0.235914, 16.098831,
	21.930426, -0.188952, 16.189348,
	21.822788, -0.137786, 16.201515,
	21.537056, -0.090824, 16.129246,
	21.085472, -0.056479, 15.966458,
	20.480286, -0.043162, 15.707068,
	21.793024, 0.317460, 15.938898,
	22.080185, 0.317460, 16.104181,
	22.155554, 0.317460, 16.196486,
	22.033438, 0.317460, 16.209364,
	21.728149, 0.317460, 16.136379,
	21.254000, 0.317460, 15.971096,
	20.625296, 0.317460, 15.707068,
	21.562399, 0.905178, 15.936043,
	21.847712, 0.870833, 16.098831,
	21.930426, 0.823872, 16.189348,
	21.822788, 0.772705, 16.201515,
	21.537056, 0.725743, 16.129246,
	21.085472, 0.691399, 15.966458,
	20.480286, 0.678081, 15.707068,
	20.967634, 1.296990, 15.928683,
	21.248175, 1.239749, 16.085032,
	21.349833, 1.161480, 16.170952,
	21.279541, 1.076202, 16.181278,
	21.044233, 0.997933, 16.110847,
	20.650845, 0.940691, 15.954498,
	20.106306, 0.918495, 15.707068,
	20.154381, 1.492896, 15.918620,
	20.428398, 1.424206, 16.066162,
	20.555962, 1.330284, 16.145794,
	20.536732, 1.227950, 16.153603,
	20.370375, 1.134027, 16.085688,
	20.056561, 1.065337, 15.938146,
	19.594950, 1.038703, 15.707068,
	19.268301, 1.492896, 15.907656,
	19.535213, 1.424206, 16.045605,
	19.690996, 1.330284, 16.118382,
	19.727402, 1.227950, 16.123451,
	19.636169, 1.134027, 16.058277,
	19.409054, 1.065337, 15.920328,
	19.037800, 1.038703, 15.707068,
	18.455048, 1.296990, 15.897593,
	18.715439, 1.239749, 16.026737,
	18.897125, 1.161480, 16.093224,
	18.984591, 1.076202, 16.095776,
	18.962313, 0.997933, 16.033119,
	18.814770, 0.940691, 15.903975,
	18.526442, 0.918495, 15.707068,
	17.860283, 0.905178, 15.890232,
	18.115904, 0.870833, 16.012938,
	18.316534, 0.823872, 16.074823,
	18.441341, 0.772705, 16.075539,
	18.469490, 0.725743, 16.014721,
	18.380142, 0.691399, 15.892016,
	18.152464, 0.678081, 15.707068,
	-0.317460, 0.317459, 20.615528,
	1.334911, 0.317459, 20.503906,
	2.002729, 0.317459, 20.203388,
	1.983648, 0.317459, 19.765491,
	1.575325, 0.317459, 19.241732,
	1.075416, 0.317459, 18.683626,
	0.781577, 0.317460, 18.142694,
	0.991462, 0.317460, 17.670452,
	1.292156, -0.066726, 20.503906,
	1.942690, -0.221977, 20.203388,
	1.924094, -0.217484, 19.765491,
	1.526318, -0.122435, 19.241732,
	1.039313, -0.006019, 18.683626,
	0.753031, 0.062574, 18.142694,
	0.957422, 0.014156, 17.670452,
	1.169913, -0.416589, 20.503906,
	1.771035, -0.713232, 20.203388,
	1.753828, -0.704677, 19.765491,
	1.386216, -0.523128, 19.241732,
	0.936123, -0.300793, 18.683626,
	0.671473, -0.169878, 18.142694,
	0.860188, -0.262588, 17.670452,
	0.977215, -0.723097, 20.503906,
	1.500448, -1.143619, 20.203388,
	1.485441, -1.131527, 19.765491,
	1.165397, -0.874243, 19.241732,
	0.773521, -0.559187, 18.683626,
	0.543017, -0.373780, 18.142694,
	0.707086, -0.505444, 17.670452,
	0.723096, -0.977216, 20.503906,
	1.143618, -1.500449, 20.203388,
	1.131526, -1.485442, 19.765491,
	0.874242, -1.165398, 19.241732,
	0.559186, -0.773522, 18.683626,
	0.373779, -0.543017, 18.142694,
	0.505444, -0.707087, 17.670452,
	0.416588, -1.169914, 20.503906,
	0.713232, -1.771036, 20.203388,
	0.704676, -1.753829, 19.765491,
	0.523127, -1.386217, 19.241732,
	0.300792, -0.936124, 18.683626,
	0.169877, -0.671473, 18.142694,
	0.262587, -0.860189, 17.670452,
	0.066725, -1.292157, 20.503906,
	0.221976, -1.942692, 20.203388,
	0.217483, -1.924095, 19.765491,
	0.122434, -1.526319, 19.241732,
	0.006018, -1.039314, 18.683626,
	-0.062575, -0.753032, 18.142694,
	-0.014157, -0.957423, 17.670452,
	-0.317460, -1.334912, 20.503906,
	-0.317460, -2.002730, 20.203388,
	-0.317460, -1.983649, 19.765491,
	-0.317460, -1.575326, 19.241732,
	-0.317460, -1.075417, 18.683626,
	-0.317460, -0.781577, 18.142694,
	-0.317460, -0.991463, 17.670452,
	-0.701646, -1.292157, 20.503906,
	-0.856897, -1.942691, 20.203388,
	-0.852404, -1.924095, 19.765491,
	-0.757354, -1.526319, 19.241732,
	-0.640939, -1.039314, 18.683626,
	-0.572346, -0.753032, 18.142694,
	-0.620764, -0.957423, 17.670452,
	-1.051509, -1.169914, 20.503906,
	-1.348152, -1.771036, 20.203388,
	-1.339597, -1.753829, 19.765491,
	-1.158048, -1.386217, 19.241732,
	-0.935713, -0.936124, 18.683626,
	-0.804798, -0.671473, 18.142694,
	-0.897508, -0.860189, 17.670452,
	-1.358017, -0.977216, 20.503906,
	-1.778539, -1.500449, 20.203388,
	-1.766447, -1.485441, 19.765491,
	-1.509162, -1.165398, 19.241732,
	-1.194106, -0.773522, 18.683626,
	-1.008700, -0.543017, 18.142694,
	-1.140364, -0.707087, 17.670452,
	-1.612136, -0.723097, 20.503906,
	-2.135369, -1.143619, 20.203388,
	-2.120362, -1.131527, 19.765491,
	-1.800318, -0.874242, 19.241732,
	-1.408442, -0.559187, 18.683626,
	-1.177937, -0.373780, 18.142694,
	-1.342007, -0.505444, 17.670452,
	-1.804834, -0.416589, 20.503906,
	-2.405955, -0.713232, 20.203388,
	-2.388748, -0.704677, 19.765491,
	-2.021136, -0.523128, 19.241732,
	-1.571043, -0.300793, 18.683626,
	-1.306393, -0.169878, 18.142694,
	-1.495109, -0.262588, 17.670452,
	-1.927077, -0.066726, 20.503906,
	-2.577611, -0.221977, 20.203388,
	-2.559015, -0.217484, 19.765491,
	-2.161239, -0.122435, 19.241732,
	-1.674234, -0.006019, 18.683626,
	-1.387952, 0.062574, 18.142694,
	-1.592343, 0.014156, 17.670452,
	-1.969832, 0.317459, 20.503906,
	-2.637649, 0.317459, 20.203388,
	-2.618569, 0.317459, 19.765491,
	-2.210246, 0.317459, 19.241732,
	-1.710337, 0.317459, 18.683626,
	-1.416497, 0.317460, 18.142694,
	-1.626383, 0.317460, 17.670452,
	-1.927077, 0.701645, 20.503906,
	-2.577611, 0.856896, 20.203388,
	-2.559015, 0.852403, 19.765491,
	-2.161239, 0.757354, 19.241732,
	-1.674234, 0.640938, 18.683626,
	-1.387952, 0.572345, 18.142694,
	-1.592343, 0.620763, 17.670452,
	-1.804834, 1.051508, 20.503906,
	-2.405955, 1.348151, 20.203388,
	-2.388748, 1.339596, 19.765491,
	-2.021136, 1.158047, 19.241732,
	-1.571043, 0.935712, 18.683626,
	-1.306393, 0.804797, 18.142694,
	-1.495109, 0.897507, 17.670452,
	-1.612136, 1.358016, 20.503906,
	-2.135369, 1.778538, 20.203388,
	-2.120361, 1.766446, 19.765491,
	-1.800318, 1.509161, 19.241732,
	-1.408442, 1.194106, 18.683626,
	-1.177937, 1.008699, 18.142694,
	-1.342007, 1.140363, 17.670452,
	-1.358017, 1.612135, 20.503906,
	-1.778538, 2.135368, 20.203388,
	-1.766447, 2.120361, 19.765491,
	-1.509162, 1.800317, 19.241732,
	-1.194106, 1.408441, 18.683626,
	-1.008700, 1.177937, 18.142694,
	-1.140364, 1.342006, 17.670452,
	-1.051509, 1.804833, 20.503906,
	-1.348152, 2.405954, 20.203388,
	-1.339597, 2.388747, 19.765491,
	-1.158048, 2.021136, 19.241732,
	-0.935713, 1.571043, 18.683626,
	-0.804797, 1.306393, 18.142694,
	-0.897507, 1.495108, 17.670452,
	-0.701646, 1.927076, 20.503906,
	-0.856897, 2.577610, 20.203388,
	-0.852404, 2.559014, 19.765491,
	-0.757354, 2.161238, 19.241732,
	-0.640939, 1.674233, 18.683626,
	-0.572345, 1.387951, 18.142694,
	-0.620764, 1.592342, 17.670452,
	-0.317460, 1.969831, 20.503906,
	-0.317460, 2.637648, 20.203388,
	-0.317460, 2.618568, 19.765491,
	-0.317460, 2.210245, 19.241732,
	-0.317460, 1.710336, 18.683626,
	-0.317460, 1.416496, 18.142694,
	-0.317460, 1.626382, 17.670452,
	0.066725, 1.927076, 20.503906,
	0.221976, 2.577610, 20.203388,
	0.217483, 2.559014, 19.765491,
	0.122434, 2.161238, 19.241732,
	0.006018, 1.674233, 18.683626,
	-0.062575, 1.387951, 18.142694,
	-0.014157, 1.592342, 17.670452,
	0.416588, 1.804833, 20.503906,
	0.713232, 2.405954, 20.203388,
	0.704676, 2.388747, 19.765491,
	0.523128, 2.021136, 19.241732,
	0.300792, 1.571043, 18.683626,
	0.169877, 1.306393, 18.142694,
	0.262587, 1.495108, 17.670452,
	0.723096, 1.612135, 20.503906,
	1.143618, 2.135368, 20.203388,
	1.131526, 2.120360, 19.765491,
	0.874242, 1.800317, 19.241732,
	0.559186, 1.408441, 18.683626,
	0.373779, 1.177937, 18.142694,
	0.505444, 1.342006, 17.670452,
	0.977216, 1.358016, 20.503906,
	1.500448, 1.778538, 20.203388,
	1.485441, 1.766446, 19.765491,
	1.165397, 1.509161, 19.241732,
	0.773522, 1.194105, 18.683626,
	0.543017, 1.008699, 18.142694,
	0.707086, 1.140363, 17.670452,
	1.169913, 1.051508, 20.503906,
	1.771035, 1.348151, 20.203388,
	1.753828, 1.339596, 19.765491,
	1.386216, 1.158047, 19.241732,
	0.936123, 0.935712, 18.683626,
	0.671473, 0.804797, 18.142694,
	0.860188, 0.897507, 17.670452,
	1.292156, 0.701645, 20.503906,
	1.942691, 0.856896, 20.203388,
	1.924094, 0.852403, 19.765491,
	1.526318, 0.757354, 19.241732,
	1.039314, 0.640938, 18.683626,
	0.753031, 0.572345, 18.142694,
	0.957422, 0.620763, 17.670452,
	1.802383, 0.317460, 17.304108,
	2.991099, 0.317460, 17.023624,
	4.374435, 0.317460, 16.794657,
	5.769219, 0.317460, 16.582863,
	6.992279, 0.317460, 16.353899,
	7.860442, 0.317460, 16.073414,
	8.190536, 0.317460, 15.707068,
	1.747255, -0.173751, 17.304108,
	2.905057, -0.449200, 17.023624,
	4.252418, -0.769747, 16.794657,
	5.610929, -1.092947, 16.582863,
	6.802183, -1.376354, 16.353899,
	7.647768, -1.577525, 16.073414,
	7.969276, -1.654014, 15.707068,
	1.589781, -0.621946, 17.304108,
	2.659278, -1.148724, 17.023624,
	3.903878, -1.761748, 16.794657,
	5.158777, -2.379845, 16.582863,
	6.259174, -2.921842, 16.353899,
	7.040267, -3.306567, 16.073414,
	7.337255, -3.452847, 15.707068,
	1.341827, -1.015261, 17.304108,
	2.272283, -1.762592, 17.023624,
	3.355077, -2.632279, 16.794657,
	4.446830, -3.509163, 16.582863,
	5.404169, -4.278087, 16.353899,
	6.083715, -4.823891, 16.073414,
	6.342092, -5.031415, 15.707068,
	1.015260, -1.341828, 17.304108,
	1.762591, -2.272284, 17.023624,
	2.632278, -3.355077, 16.794657,
	3.509162, -4.446831, 16.582863,
	4.278086, -5.404170, 16.353899,
	4.823889, -6.083715, 16.073414,
	5.031414, -6.342093, 15.707068,
	0.621946, -1.589782, 17.304108,
	1.148723, -2.659279, 17.023624,
	1.761747, -3.903879, 16.794657,
	2.379844, -5.158778, 16.582863,
	2.921841, -6.259176, 16.353899,
	3.306566, -7.040268, 16.073414,
	3.452847, -7.337256, 15.707068,
	0.173750, -1.747256, 17.304108,
	0.449199, -2.905058, 17.023624,
	0.769746, -4.252419, 16.794657,
	1.092946, -5.610931, 16.582863,
	1.376353, -6.802185, 16.353899,
	1.577524, -7.647770, 16.073414,
	1.654013, -7.969279, 15.707068,
	-0.317460, -1.802384, 17.304108,
	-0.317460, -2.991099, 17.023624,
	-0.317460, -4.374436, 16.794657,
	-0.317460, -5.769220, 16.582863,
	-0.317460, -6.992280, 16.353899,
	-0.317460, -7.860442, 16.073414,
	-0.317460, -8.190536, 15.707068,
	-0.808671, -1.747256, 17.304108,
	-1.084120, -2.905057, 17.023624,
	-1.404667, -4.252419, 16.794657,
	-1.727867, -5.610931, 16.582863,
	-2.011274, -6.802185, 16.353899,
	-2.212445, -7.647769, 16.073414,
	-2.288934, -7.969278, 15.707068,
	-1.256866, -1.589782, 17.304108,
	-1.783644, -2.659279, 17.023624,
	-2.396668, -3.903879, 16.794657,
	-3.014765, -5.158778, 16.582863,
	-3.556762, -6.259176, 16.353899,
	-3.941488, -7.040268, 16.073414,
	-4.087768, -7.337256, 15.707068,
	-1.650181, -1.341828, 17.304108,
	-2.397512, -2.272284, 17.023624,
	-3.267199, -3.355077, 16.794657,
	-4.144083, -4.446831, 16.582863,
	-4.913007, -5.404170, 16.353899,
	-5.458811, -6.083715, 16.073414,
	-5.666336, -6.342093, 15.707068,
	-1.976748, -1.015261, 17.304108,
	-2.907203, -1.762592, 17.023624,
	-3.989997, -2.632279, 16.794657,
	-5.081751, -3.509163, 16.582863,
	-6.039090, -4.278087, 16.353899,
	-6.718636, -4.823890, 16.073414,
	-6.977013, -5.031415, 15.707068,
	-2.224702, -0.621946, 17.304108,
	-3.294199, -1.148724, 17.023624,
	-4.538799, -1.761748, 16.794657,
	-5.793698, -2.379845, 16.582863,
	-6.894096, -2.921842, 16.353899,
	-7.675188, -3.306567, 16.073414,
	-7.972176, -3.452847, 15.707068,
	-2.382176, -0.173751, 17.304108,
	-3.539977, -0.449200, 17.023624,
	-4.887340, -0.769747, 16.794657,
	-6.245851, -1.092947, 16.582863,
	-7.437105, -1.376354, 16.353899,
	-8.282690, -1.577525, 16.073414,
	-8.604198, -1.654014, 15.707068,
	-2.437304, 0.317460, 17.304108,
	-3.626019, 0.317460, 17.023624,
	-5.009356, 0.317460, 16.794657,
	-6.404140, 0.317460, 16.582863,
	-7.627201, 0.317460, 16.353899,
	-8.495362, 0.317460, 16.073414,
	-8.825456, 0.317460, 15.707068,
	-2.382176, 0.808670, 17.304108,
	-3.539977, 1.084119, 17.023624,
	-4.887339, 1.404666, 16.794657,
	-6.245851, 1.727866, 16.582863,
	-7.437105, 2.011273, 16.353899,
	-8.282689, 2.212444, 16.073414,
	-8.604198, 2.288933, 15.707068,
	-2.224702, 1.256866, 17.304108,
	-3.294199, 1.783643, 17.023624,
	-4.538799, 2.396667, 16.794657,
	-5.793698, 3.014764, 16.582863,
	-6.894096, 3.556762, 16.353899,
	-7.675188, 3.941487, 16.073414,
	-7.972176, 4.087767, 15.707068,
	-1.976748, 1.650180, 17.304108,
	-2.907203, 2.397511, 17.023624,
	-3.989997, 3.267198, 16.794657,
	-5.081751, 4.144083, 16.582863,
	-6.039090, 4.913006, 16.353899,
	-6.718636, 5.458810, 16.073414,
	-6.977013, 5.666335, 15.707068,
	-1.650181, 1.976747, 17.304108,
	-2.397512, 2.907202, 17.023624,
	-3.267199, 3.989996, 16.794657,
	-4.144083, 5.081750, 16.582863,
	-4.913007, 6.039089, 16.353899,
	-5.458810, 6.718635, 16.073414,
	-5.666335, 6.977012, 15.707068,
	-1.256866, 2.224701, 17.304108,
	-1.783644, 3.294198, 17.023624,
	-2.396667, 4.538798, 16.794657,
	-3.014765, 5.793697, 16.582863,
	-3.556762, 6.894095, 16.353899,
	-3.941487, 7.675188, 16.073414,
	-4.087767, 7.972175, 15.707068,
	-0.808671, 2.382175, 17.304108,
	-1.084120, 3.539977, 17.023624,
	-1.404667, 4.887339, 16.794657,
	-1.727866, 6.245851, 16.582863,
	-2.011274, 7.437104, 16.353899,
	-2.212445, 8.282689, 16.073414,
	-2.288934, 8.604198, 15.707068,
	-0.317460, 2.437303, 17.304108,
	-0.317460, 3.626018, 17.023624,
	-0.317460, 5.009355, 16.794657,
	-0.317460, 6.404139, 16.582863,
	-0.317460, 7.627199, 16.353899,
	-0.317460, 8.495362, 16.073414,
	-0.317460, 8.825456, 15.707068,
	0.173750, 2.382175, 17.304108,
	0.449199, 3.539977, 17.023624,
	0.769746, 4.887338, 16.794657,
	1.092946, 6.245850, 16.582863,
	1.376354, 7.437104, 16.353899,
	1.577524, 8.282689, 16.073414,
	1.654013, 8.604198, 15.707068,
	0.621946, 2.224701, 17.304108,
	1.148723, 3.294198, 17.023624,
	1.761747, 4.538798, 16.794657,
	2.379844, 5.793697, 16.582863,
	2.921841, 6.894095, 16.353899,
	3.306567, 7.675188, 16.073414,
	3.452847, 7.972175, 15.707068,
	1.015260, 1.976747, 17.304108,
	1.762591, 2.907202, 17.023624,
	2.632278, 3.989996, 16.794657,
	3.509163, 5.081750, 16.582863,
	4.278086, 6.039089, 16.353899,
	4.823889, 6.718635, 16.073414,
	5.031415, 6.977012, 15.707068,
	1.341827, 1.650180, 17.304108,
	2.272283, 2.397511, 17.023624,
	3.355077, 3.267198, 16.794657,
	4.446830, 4.144083, 16.582863,
	5.404169, 4.913006, 16.353899,
	6.083715, 5.458809, 16.073414,
	6.342092, 5.666334, 15.707068,
	1.589781, 1.256866, 17.304108,
	2.659278, 1.783643, 17.023624,
	3.903878, 2.396667, 16.794657,
	5.158777, 3.014764, 16.582863,
	6.259174, 3.556761, 16.353899,
	7.040267, 3.941486, 16.073414,
	7.337255, 4.087767, 15.707068,
	1.747255, 0.808670, 17.304108,
	2.905057, 1.084119, 17.023624,
	4.252419, 1.404666, 16.794657,
	5.610930, 1.727866, 16.582863,
	6.802183, 2.011273, 16.353899,
	7.647769, 2.212444, 16.073414,
	7.969278, 2.288933, 15.707068
};

unsigned short teapot_faces[3136 * 3] = {
	9, 8, 0,
	0, 1, 9,
	10, 9, 1,
	1, 2, 10,
	11, 10, 2,
	2, 3, 11,
	12, 11, 3,
	3, 4, 12,
	13, 12, 4,
	4, 5, 13,
	14, 13, 5,
	5, 6, 14,
	15, 14, 6,
	6, 7, 15,
	17, 16, 8,
	8, 9, 17,
	18, 17, 9,
	9, 10, 18,
	19, 18, 10,
	10, 11, 19,
	20, 19, 11,
	11, 12, 20,
	21, 20, 12,
	12, 13, 21,
	22, 21, 13,
	13, 14, 22,
	23, 22, 14,
	14, 15, 23,
	25, 24, 16,
	16, 17, 25,
	26, 25, 17,
	17, 18, 26,
	27, 26, 18,
	18, 19, 27,
	28, 27, 19,
	19, 20, 28,
	29, 28, 20,
	20, 21, 29,
	30, 29, 21,
	21, 22, 30,
	31, 30, 22,
	22, 23, 31,
	33, 32, 24,
	24, 25, 33,
	34, 33, 25,
	25, 26, 34,
	35, 34, 26,
	26, 27, 35,
	36, 35, 27,
	27, 28, 36,
	37, 36, 28,
	28, 29, 37,
	38, 37, 29,
	29, 30, 38,
	39, 38, 30,
	30, 31, 39,
	41, 40, 32,
	32, 33, 41,
	42, 41, 33,
	33, 34, 42,
	43, 42, 34,
	34, 35, 43,
	44, 43, 35,
	35, 36, 44,
	45, 44, 36,
	36, 37, 45,
	46, 45, 37,
	37, 38, 46,
	47, 46, 38,
	38, 39, 47,
	49, 48, 40,
	40, 41, 49,
	50, 49, 41,
	41, 42, 50,
	51, 50, 42,
	42, 43, 51,
	52, 51, 43,
	43, 44, 52,
	53, 52, 44,
	44, 45, 53,
	54, 53, 45,
	45, 46, 54,
	55, 54, 46,
	46, 47, 55,
	57, 56, 48,
	48, 49, 57,
	58, 57, 49,
	49, 50, 58,
	59, 58, 50,
	50, 51, 59,
	60, 59, 51,
	51, 52, 60,
	61, 60, 52,
	52, 53, 61,
	62, 61, 53,
	53, 54, 62,
	63, 62, 54,
	54, 55, 63,
	65, 64, 56,
	56, 57, 65,
	66, 65, 57,
	57, 58, 66,
	67, 66, 58,
	58, 59, 67,
	68, 67, 59,
	59, 60, 68,
	69, 68, 60,
	60, 61, 69,
	70, 69, 61,
	61, 62, 70,
	71, 70, 62,
	62, 63, 71,
	73, 72, 64,
	64, 65, 73,
	74, 73, 65,
	65, 66, 74,
	75, 74, 66,
	66, 67, 75,
	76, 75, 67,
	67, 68, 76,
	77, 76, 68,
	68, 69, 77,
	78, 77, 69,
	69, 70, 78,
	79, 78, 70,
	70, 71, 79,
	81, 80, 72,
	72, 73, 81,
	82, 81, 73,
	73, 74, 82,
	83, 82, 74,
	74, 75, 83,
	84, 83, 75,
	75, 76, 84,
	85, 84, 76,
	76, 77, 85,
	86, 85, 77,
	77, 78, 86,
	87, 86, 78,
	78, 79, 87,
	89, 88, 80,
	80, 81, 89,
	90, 89, 81,
	81, 82, 90,
	91, 90, 82,
	82, 83, 91,
	92, 91, 83,
	83, 84, 92,
	93, 92, 84,
	84, 85, 93,
	94, 93, 85,
	85, 86, 94,
	95, 94, 86,
	86, 87, 95,
	97, 96, 88,
	88, 89, 97,
	98, 97, 89,
	89, 90, 98,
	99, 98, 90,
	90, 91, 99,
	100, 99, 91,
	91, 92, 100,
	101, 100, 92,
	92, 93, 101,
	102, 101, 93,
	93, 94, 102,
	103, 102, 94,
	94, 95, 103,
	105, 104, 96,
	96, 97, 105,
	106, 105, 97,
	97, 98, 106,
	107, 106, 98,
	98, 99, 107,
	108, 107, 99,
	99, 100, 108,
	109, 108, 100,
	100, 101, 109,
	110, 109, 101,
	101, 102, 110,
	111, 110, 102,
	102, 103, 111,
	113, 112, 104,
	104, 105, 113,
	114, 113, 105,
	105, 106, 114,
	115, 114, 106,
	106, 107, 115,
	116, 115, 107,
	107, 108, 116,
	117, 116, 108,
	108, 109, 117,
	118, 117, 109,
	109, 110, 118,
	119, 118, 110,
	110, 111, 119,
	121, 120, 112,
	112, 113, 121,
	122, 121, 113,
	113, 114, 122,
	123, 122, 114,
	114, 115, 123,
	124, 123, 115,
	115, 116, 124,
	125, 124, 116,
	116, 117, 125,
	126, 125, 117,
	117, 118, 126,
	127, 126, 118,
	118, 119, 127,
	129, 128, 120,
	120, 121, 129,
	130, 129, 121,
	121, 122, 130,
	131, 130, 122,
	122, 123, 131,
	132, 131, 123,
	123, 124, 132,
	133, 132, 124,
	124, 125, 133,
	134, 133, 125,
	125, 126, 134,
	135, 134, 126,
	126, 127, 135,
	137, 136, 128,
	128, 129, 137,
	138, 137, 129,
	129, 130, 138,
	139, 138, 130,
	130, 131, 139,
	140, 139, 131,
	131, 132, 140,
	141, 140, 132,
	132, 133, 141,
	142, 141, 133,
	133, 134, 142,
	143, 142, 134,
	134, 135, 143,
	145, 144, 136,
	136, 137, 145,
	146, 145, 137,
	137, 138, 146,
	147, 146, 138,
	138, 139, 147,
	148, 147, 139,
	139, 140, 148,
	149, 148, 140,
	140, 141, 149,
	150, 149, 141,
	141, 142, 150,
	151, 150, 142,
	142, 143, 151,
	153, 152, 144,
	144, 145, 153,
	154, 153, 145,
	145, 146, 154,
	155, 154, 146,
	146, 147, 155,
	156, 155, 147,
	147, 148, 156,
	157, 156, 148,
	148, 149, 157,
	158, 157, 149,
	149, 150, 158,
	159, 158, 150,
	150, 151, 159,
	161, 160, 152,
	152, 153, 161,
	162, 161, 153,
	153, 154, 162,
	163, 162, 154,
	154, 155, 163,
	164, 163, 155,
	155, 156, 164,
	165, 164, 156,
	156, 157, 165,
	166, 165, 157,
	157, 158, 166,
	167, 166, 158,
	158, 159, 167,
	169, 168, 160,
	160, 161, 169,
	170, 169, 161,
	161, 162, 170,
	171, 170, 162,
	162, 163, 171,
	172, 171, 163,
	163, 164, 172,
	173, 172, 164,
	164, 165, 173,
	174, 173, 165,
	165, 166, 174,
	175, 174, 166,
	166, 167, 175,
	177, 176, 168,
	168, 169, 177,
	178, 177, 169,
	169, 170, 178,
	179, 178, 170,
	170, 171, 179,
	180, 179, 171,
	171, 172, 180,
	181, 180, 172,
	172, 173, 181,
	182, 181, 173,
	173, 174, 182,
	183, 182, 174,
	174, 175, 183,
	185, 184, 176,
	176, 177, 185,
	186, 185, 177,
	177, 178, 186,
	187, 186, 178,
	178, 179, 187,
	188, 187, 179,
	179, 180, 188,
	189, 188, 180,
	180, 181, 189,
	190, 189, 181,
	181, 182, 190,
	191, 190, 182,
	182, 183, 191,
	193, 192, 184,
	184, 185, 193,
	194, 193, 185,
	185, 186, 194,
	195, 194, 186,
	186, 187, 195,
	196, 195, 187,
	187, 188, 196,
	197, 196, 188,
	188, 189, 197,
	198, 197, 189,
	189, 190, 198,
	199, 198, 190,
	190, 191, 199,
	201, 200, 192,
	192, 193, 201,
	202, 201, 193,
	193, 194, 202,
	203, 202, 194,
	194, 195, 203,
	204, 203, 195,
	195, 196, 204,
	205, 204, 196,
	196, 197, 205,
	206, 205, 197,
	197, 198, 206,
	207, 206, 198,
	198, 199, 207,
	209, 208, 200,
	200, 201, 209,
	210, 209, 201,
	201, 202, 210,
	211, 210, 202,
	202, 203, 211,
	212, 211, 203,
	203, 204, 212,
	213, 212, 204,
	204, 205, 213,
	214, 213, 205,
	205, 206, 214,
	215, 214, 206,
	206, 207, 215,
	217, 216, 208,
	208, 209, 217,
	218, 217, 209,
	209, 210, 218,
	219, 218, 210,
	210, 211, 219,
	220, 219, 211,
	211, 212, 220,
	221, 220, 212,
	212, 213, 221,
	222, 221, 213,
	213, 214, 222,
	223, 222, 214,
	214, 215, 223,
	1, 0, 216,
	216, 217, 1,
	2, 1, 217,
	217, 218, 2,
	3, 2, 218,
	218, 219, 3,
	4, 3, 219,
	219, 220, 4,
	5, 4, 220,
	220, 221, 5,
	6, 5, 221,
	221, 222, 6,
	7, 6, 222,
	222, 223, 7,
	231, 15, 7,
	7, 224, 231,
	232, 231, 224,
	224, 225, 232,
	233, 232, 225,
	225, 226, 233,
	234, 233, 226,
	226, 227, 234,
	235, 234, 227,
	227, 228, 235,
	236, 235, 228,
	228, 229, 236,
	237, 236, 229,
	229, 230, 237,
	238, 23, 15,
	15, 231, 238,
	239, 238, 231,
	231, 232, 239,
	240, 239, 232,
	232, 233, 240,
	241, 240, 233,
	233, 234, 241,
	242, 241, 234,
	234, 235, 242,
	243, 242, 235,
	235, 236, 243,
	244, 243, 236,
	236, 237, 244,
	245, 31, 23,
	23, 238, 245,
	246, 245, 238,
	238, 239, 246,
	247, 246, 239,
	239, 240, 247,
	248, 247, 240,
	240, 241, 248,
	249, 248, 241,
	241, 242, 249,
	250, 249, 242,
	242, 243, 250,
	251, 250, 243,
	243, 244, 251,
	252, 39, 31,
	31, 245, 252,
	253, 252, 245,
	245, 246, 253,
	254, 253, 246,
	246, 247, 254,
	255, 254, 247,
	247, 248, 255,
	256, 255, 248,
	248, 249, 256,
	257, 256, 249,
	249, 250, 257,
	258, 257, 250,
	250, 251, 258,
	259, 47, 39,
	39, 252, 259,
	260, 259, 252,
	252, 253, 260,
	261, 260, 253,
	253, 254, 261,
	262, 261, 254,
	254, 255, 262,
	263, 262, 255,
	255, 256, 263,
	264, 263, 256,
	256, 257, 264,
	265, 264, 257,
	257, 258, 265,
	266, 55, 47,
	47, 259, 266,
	267, 266, 259,
	259, 260, 267,
	268, 267, 260,
	260, 261, 268,
	269, 268, 261,
	261, 262, 269,
	270, 269, 262,
	262, 263, 270,
	271, 270, 263,
	263, 264, 271,
	272, 271, 264,
	264, 265, 272,
	273, 63, 55,
	55, 266, 273,
	274, 273, 266,
	266, 267, 274,
	275, 274, 267,
	267, 268, 275,
	276, 275, 268,
	268, 269, 276,
	277, 276, 269,
	269, 270, 277,
	278, 277, 270,
	270, 271, 278,
	279, 278, 271,
	271, 272, 279,
	280, 71, 63,
	63, 273, 280,
	281, 280, 273,
	273, 274, 281,
	282, 281, 274,
	274, 275, 282,
	283, 282, 275,
	275, 276, 283,
	284, 283, 276,
	276, 277, 284,
	285, 284, 277,
	277, 278, 285,
	286, 285, 278,
	278, 279, 286,
	287, 79, 71,
	71, 280, 287,
	288, 287, 280,
	280, 281, 288,
	289, 288, 281,
	281, 282, 289,
	290, 289, 282,
	282, 283, 290,
	291, 290, 283,
	283, 284, 291,
	292, 291, 284,
	284, 285, 292,
	293, 292, 285,
	285, 286, 293,
	294, 87, 79,
	79, 287, 294,
	295, 294, 287,
	287, 288, 295,
	296, 295, 288,
	288, 289, 296,
	297, 296, 289,
	289, 290, 297,
	298, 297, 290,
	290, 291, 298,
	299, 298, 291,
	291, 292, 299,
	300, 299, 292,
	292, 293, 300,
	301, 95, 87,
	87, 294, 301,
	302, 301, 294,
	294, 295, 302,
	303, 302, 295,
	295, 296, 303,
	304, 303, 296,
	296, 297, 304,
	305, 304, 297,
	297, 298, 305,
	306, 305, 298,
	298, 299, 306,
	307, 306, 299,
	299, 300, 307,
	308, 103, 95,
	95, 301, 308,
	309, 308, 301,
	301, 302, 309,
	310, 309, 302,
	302, 303, 310,
	311, 310, 303,
	303, 304, 311,
	312, 311, 304,
	304, 305, 312,
	313, 312, 305,
	305, 306, 313,
	314, 313, 306,
	306, 307, 314,
	315, 111, 103,
	103, 308, 315,
	316, 315, 308,
	308, 309, 316,
	317, 316, 309,
	309, 310, 317,
	318, 317, 310,
	310, 311, 318,
	319, 318, 311,
	311, 312, 319,
	320, 319, 312,
	312, 313, 320,
	321, 320, 313,
	313, 314, 321,
	322, 119, 111,
	111, 315, 322,
	323, 322, 315,
	315, 316, 323,
	324, 323, 316,
	316, 317, 324,
	325, 324, 317,
	317, 318, 325,
	326, 325, 318,
	318, 319, 326,
	327, 326, 319,
	319, 320, 327,
	328, 327, 320,
	320, 321, 328,
	329, 127, 119,
	119, 322, 329,
	330, 329, 322,
	322, 323, 330,
	331, 330, 323,
	323, 324, 331,
	332, 331, 324,
	324, 325, 332,
	333, 332, 325,
	325, 326, 333,
	334, 333, 326,
	326, 327, 334,
	335, 334, 327,
	327, 328, 335,
	336, 135, 127,
	127, 329, 336,
	337, 336, 329,
	329, 330, 337,
	338, 337, 330,
	330, 331, 338,
	339, 338, 331,
	331, 332, 339,
	340, 339, 332,
	332, 333, 340,
	341, 340, 333,
	333, 334, 341,
	342, 341, 334,
	334, 335, 342,
	343, 143, 135,
	135, 336, 343,
	344, 343, 336,
	336, 337, 344,
	345, 344, 337,
	337, 338, 345,
	346, 345, 338,
	338, 339, 346,
	347, 346, 339,
	339, 340, 347,
	348, 347, 340,
	340, 341, 348,
	349, 348, 341,
	341, 342, 349,
	350, 151, 143,
	143, 343, 350,
	351, 350, 343,
	343, 344, 351,
	352, 351, 344,
	344, 345, 352,
	353, 352, 345,
	345, 346, 353,
	354, 353, 346,
	346, 347, 354,
	355, 354, 347,
	347, 348, 355,
	356, 355, 348,
	348, 349, 356,
	357, 159, 151,
	151, 350, 357,
	358, 357, 350,
	350, 351, 358,
	359, 358, 351,
	351, 352, 359,
	360, 359, 352,
	352, 353, 360,
	361, 360, 353,
	353, 354, 361,
	362, 361, 354,
	354, 355, 362,
	363, 362, 355,
	355, 356, 363,
	364, 167, 159,
	159, 357, 364,
	365, 364, 357,
	357, 358, 365,
	366, 365, 358,
	358, 359, 366,
	367, 366, 359,
	359, 360, 367,
	368, 367, 360,
	360, 361, 368,
	369, 368, 361,
	361, 362, 369,
	370, 369, 362,
	362, 363, 370,
	371, 175, 167,
	167, 364, 371,
	372, 371, 364,
	364, 365, 372,
	373, 372, 365,
	365, 366, 373,
	374, 373, 366,
	366, 367, 374,
	375, 374, 367,
	367, 368, 375,
	376, 375, 368,
	368, 369, 376,
	377, 376, 369,
	369, 370, 377,
	378, 183, 175,
	175, 371, 378,
	379, 378, 371,
	371, 372, 379,
	380, 379, 372,
	372, 373, 380,
	381, 380, 373,
	373, 374, 381,
	382, 381, 374,
	374, 375, 382,
	383, 382, 375,
	375, 376, 383,
	384, 383, 376,
	376, 377, 384,
	385, 191, 183,
	183, 378, 385,
	386, 385, 378,
	378, 379, 386,
	387, 386, 379,
	379, 380, 387,
	388, 387, 380,
	380, 381, 388,
	389, 388, 381,
	381, 382, 389,
	390, 389, 382,
	382, 383, 390,
	391, 390, 383,
	383, 384, 391,
	392, 199, 191,
	191, 385, 392,
	393, 392, 385,
	385, 386, 393,
	394, 393, 386,
	386, 387, 394,
	395, 394, 387,
	387, 388, 395,
	396, 395, 388,
	388, 389, 396,
	397, 396, 389,
	389, 390, 397,
	398, 397, 390,
	390, 391, 398,
	399, 207, 199,
	199, 392, 399,
	400, 399, 392,
	392, 393, 400,
	401, 400, 393,
	393, 394, 401,
	402, 401, 394,
	394, 395, 402,
	403, 402, 395,
	395, 396, 403,
	404, 403, 396,
	396, 397, 404,
	405, 404, 397,
	397, 398, 405,
	406, 215, 207,
	207, 399, 406,
	407, 406, 399,
	399, 400, 407,
	408, 407, 400,
	400, 401, 408,
	409, 408, 401,
	401, 402, 409,
	410, 409, 402,
	402, 403, 410,
	411, 410, 403,
	403, 404, 411,
	412, 411, 404,
	404, 405, 412,
	413, 223, 215,
	215, 406, 413,
	414, 413, 406,
	406, 407, 414,
	415, 414, 407,
	407, 408, 415,
	416, 415, 408,
	408, 409, 416,
	417, 416, 409,
	409, 410, 417,
	418, 417, 410,
	410, 411, 418,
	419, 418, 411,
	411, 412, 419,
	224, 7, 223,
	223, 413, 224,
	225, 224, 413,
	413, 414, 225,
	226, 225, 414,
	414, 415, 226,
	227, 226, 415,
	415, 416, 227,
	228, 227, 416,
	416, 417, 228,
	229, 228, 417,
	417, 418, 229,
	230, 229, 418,
	418, 419, 230,
	427, 237, 230,
	230, 420, 427,
	428, 427, 420,
	420, 421, 428,
	429, 428, 421,
	421, 422, 429,
	430, 429, 422,
	422, 423, 430,
	431, 430, 423,
	423, 424, 431,
	432, 431, 424,
	424, 425, 432,
	433, 432, 425,
	425, 426, 433,
	434, 244, 237,
	237, 427, 434,
	435, 434, 427,
	427, 428, 435,
	436, 435, 428,
	428, 429, 436,
	437, 436, 429,
	429, 430, 437,
	438, 437, 430,
	430, 431, 438,
	439, 438, 431,
	431, 432, 439,
	440, 439, 432,
	432, 433, 440,
	441, 251, 244,
	244, 434, 441,
	442, 441, 434,
	434, 435, 442,
	443, 442, 435,
	435, 436, 443,
	444, 443, 436,
	436, 437, 444,
	445, 444, 437,
	437, 438, 445,
	446, 445, 438,
	438, 439, 446,
	447, 446, 439,
	439, 440, 447,
	448, 258, 251,
	251, 441, 448,
	449, 448, 441,
	441, 442, 449,
	450, 449, 442,
	442, 443, 450,
	451, 450, 443,
	443, 444, 451,
	452, 451, 444,
	444, 445, 452,
	453, 452, 445,
	445, 446, 453,
	454, 453, 446,
	446, 447, 454,
	455, 265, 258,
	258, 448, 455,
	456, 455, 448,
	448, 449, 456,
	457, 456, 449,
	449, 450, 457,
	458, 457, 450,
	450, 451, 458,
	459, 458, 451,
	451, 452, 459,
	460, 459, 452,
	452, 453, 460,
	461, 460, 453,
	453, 454, 461,
	462, 272, 265,
	265, 455, 462,
	463, 462, 455,
	455, 456, 463,
	464, 463, 456,
	456, 457, 464,
	465, 464, 457,
	457, 458, 465,
	466, 465, 458,
	458, 459, 466,
	467, 466, 459,
	459, 460, 467,
	468, 467, 460,
	460, 461, 468,
	469, 279, 272,
	272, 462, 469,
	470, 469, 462,
	462, 463, 470,
	471, 470, 463,
	463, 464, 471,
	472, 471, 464,
	464, 465, 472,
	473, 472, 465,
	465, 466, 473,
	474, 473, 466,
	466, 467, 474,
	475, 474, 467,
	467, 468, 475,
	476, 286, 279,
	279, 469, 476,
	477, 476, 469,
	469, 470, 477,
	478, 477, 470,
	470, 471, 478,
	479, 478, 471,
	471, 472, 479,
	480, 479, 472,
	472, 473, 480,
	481, 480, 473,
	473, 474, 481,
	482, 481, 474,
	474, 475, 482,
	483, 293, 286,
	286, 476, 483,
	484, 483, 476,
	476, 477, 484,
	485, 484, 477,
	477, 478, 485,
	486, 485, 478,
	478, 479, 486,
	487, 486, 479,
	479, 480, 487,
	488, 487, 480,
	480, 481, 488,
	489, 488, 481,
	481, 482, 489,
	490, 300, 293,
	293, 483, 490,
	491, 490, 483,
	483, 484, 491,
	492, 491, 484,
	484, 485, 492,
	493, 492, 485,
	485, 486, 493,
	494, 493, 486,
	486, 487, 494,
	495, 494, 487,
	487, 488, 495,
	496, 495, 488,
	488, 489, 496,
	497, 307, 300,
	300, 490, 497,
	498, 497, 490,
	490, 491, 498,
	499, 498, 491,
	491, 492, 499,
	500, 499, 492,
	492, 493, 500,
	501, 500, 493,
	493, 494, 501,
	502, 501, 494,
	494, 495, 502,
	503, 502, 495,
	495, 496, 503,
	504, 314, 307,
	307, 497, 504,
	505, 504, 497,
	497, 498, 505,
	506, 505, 498,
	498, 499, 506,
	507, 506, 499,
	499, 500, 507,
	508, 507, 500,
	500, 501, 508,
	509, 508, 501,
	501, 502, 509,
	510, 509, 502,
	502, 503, 510,
	511, 321, 314,
	314, 504, 511,
	512, 511, 504,
	504, 505, 512,
	513, 512, 505,
	505, 506, 513,
	514, 513, 506,
	506, 507, 514,
	515, 514, 507,
	507, 508, 515,
	516, 515, 508,
	508, 509, 516,
	517, 516, 509,
	509, 510, 517,
	518, 328, 321,
	321, 511, 518,
	519, 518, 511,
	511, 512, 519,
	520, 519, 512,
	512, 513, 520,
	521, 520, 513,
	513, 514, 521,
	522, 521, 514,
	514, 515, 522,
	523, 522, 515,
	515, 516, 523,
	524, 523, 516,
	516, 517, 524,
	525, 335, 328,
	328, 518, 525,
	526, 525, 518,
	518, 519, 526,
	527, 526, 519,
	519, 520, 527,
	528, 527, 520,
	520, 521, 528,
	529, 528, 521,
	521, 522, 529,
	530, 529, 522,
	522, 523, 530,
	531, 530, 523,
	523, 524, 531,
	532, 342, 335,
	335, 525, 532,
	533, 532, 525,
	525, 526, 533,
	534, 533, 526,
	526, 527, 534,
	535, 534, 527,
	527, 528, 535,
	536, 535, 528,
	528, 529, 536,
	537, 536, 529,
	529, 530, 537,
	538, 537, 530,
	530, 531, 538,
	539, 349, 342,
	342, 532, 539,
	540, 539, 532,
	532, 533, 540,
	541, 540, 533,
	533, 534, 541,
	542, 541, 534,
	534, 535, 542,
	543, 542, 535,
	535, 536, 543,
	544, 543, 536,
	536, 537, 544,
	545, 544, 537,
	537, 538, 545,
	546, 356, 349,
	349, 539, 546,
	547, 546, 539,
	539, 540, 547,
	548, 547, 540,
	540, 541, 548,
	549, 548, 541,
	541, 542, 549,
	550, 549, 542,
	542, 543, 550,
	551, 550, 543,
	543, 544, 551,
	552, 551, 544,
	544, 545, 552,
	553, 363, 356,
	356, 546, 553,
	554, 553, 546,
	546, 547, 554,
	555, 554, 547,
	547, 548, 555,
	556, 555, 548,
	548, 549, 556,
	557, 556, 549,
	549, 550, 557,
	558, 557, 550,
	550, 551, 558,
	559, 558, 551,
	551, 552, 559,
	560, 370, 363,
	363, 553, 560,
	561, 560, 553,
	553, 554, 561,
	562, 561, 554,
	554, 555, 562,
	563, 562, 555,
	555, 556, 563,
	564, 563, 556,
	556, 557, 564,
	565, 564, 557,
	557, 558, 565,
	566, 565, 558,
	558, 559, 566,
	567, 377, 370,
	370, 560, 567,
	568, 567, 560,
	560, 561, 568,
	569, 568, 561,
	561, 562, 569,
	570, 569, 562,
	562, 563, 570,
	571, 570, 563,
	563, 564, 571,
	572, 571, 564,
	564, 565, 572,
	573, 572, 565,
	565, 566, 573,
	574, 384, 377,
	377, 567, 574,
	575, 574, 567,
	567, 568, 575,
	576, 575, 568,
	568, 569, 576,
	577, 576, 569,
	569, 570, 577,
	578, 577, 570,
	570, 571, 578,
	579, 578, 571,
	571, 572, 579,
	580, 579, 572,
	572, 573, 580,
	581, 391, 384,
	384, 574, 581,
	582, 581, 574,
	574, 575, 582,
	583, 582, 575,
	575, 576, 583,
	584, 583, 576,
	576, 577, 584,
	585, 584, 577,
	577, 578, 585,
	586, 585, 578,
	578, 579, 586,
	587, 586, 579,
	579, 580, 587,
	588, 398, 391,
	391, 581, 588,
	589, 588, 581,
	581, 582, 589,
	590, 589, 582,
	582, 583, 590,
	591, 590, 583,
	583, 584, 591,
	592, 591, 584,
	584, 585, 592,
	593, 592, 585,
	585, 586, 593,
	594, 593, 586,
	586, 587, 594,
	595, 405, 398,
	398, 588, 595,
	596, 595, 588,
	588, 589, 596,
	597, 596, 589,
	589, 590, 597,
	598, 597, 590,
	590, 591, 598,
	599, 598, 591,
	591, 592, 599,
	600, 599, 592,
	592, 593, 600,
	601, 600, 593,
	593, 594, 601,
	602, 412, 405,
	405, 595, 602,
	603, 602, 595,
	595, 596, 603,
	604, 603, 596,
	596, 597, 604,
	605, 604, 597,
	597, 598, 605,
	606, 605, 598,
	598, 599, 606,
	607, 606, 599,
	599, 600, 607,
	608, 607, 600,
	600, 601, 608,
	609, 419, 412,
	412, 602, 609,
	610, 609, 602,
	602, 603, 610,
	611, 610, 603,
	603, 604, 611,
	612, 611, 604,
	604, 605, 612,
	613, 612, 605,
	605, 606, 613,
	614, 613, 606,
	606, 607, 614,
	615, 614, 607,
	607, 608, 615,
	420, 230, 419,
	419, 609, 420,
	421, 420, 609,
	609, 610, 421,
	422, 421, 610,
	610, 611, 422,
	423, 422, 611,
	611, 612, 423,
	424, 423, 612,
	612, 613, 424,
	425, 424, 613,
	613, 614, 425,
	426, 425, 614,
	614, 615, 426,
	623, 433, 426,
	426, 616, 623,
	624, 623, 616,
	616, 617, 624,
	625, 624, 617,
	617, 618, 625,
	626, 625, 618,
	618, 619, 626,
	627, 626, 619,
	619, 620, 627,
	628, 627, 620,
	620, 621, 628,
	622, 628, 621,
	621, 622, 622,
	629, 440, 433,
	433, 623, 629,
	630, 629, 623,
	623, 624, 630,
	631, 630, 624,
	624, 625, 631,
	632, 631, 625,
	625, 626, 632,
	633, 632, 626,
	626, 627, 633,
	634, 633, 627,
	627, 628, 634,
	622, 634, 628,
	628, 622, 622,
	635, 447, 440,
	440, 629, 635,
	636, 635, 629,
	629, 630, 636,
	637, 636, 630,
	630, 631, 637,
	638, 637, 631,
	631, 632, 638,
	639, 638, 632,
	632, 633, 639,
	640, 639, 633,
	633, 634, 640,
	622, 640, 634,
	634, 622, 622,
	641, 454, 447,
	447, 635, 641,
	642, 641, 635,
	635, 636, 642,
	643, 642, 636,
	636, 637, 643,
	644, 643, 637,
	637, 638, 644,
	645, 644, 638,
	638, 639, 645,
	646, 645, 639,
	639, 640, 646,
	622, 646, 640,
	640, 622, 622,
	647, 461, 454,
	454, 641, 647,
	648, 647, 641,
	641, 642, 648,
	649, 648, 642,
	642, 643, 649,
	650, 649, 643,
	643, 644, 650,
	651, 650, 644,
	644, 645, 651,
	652, 651, 645,
	645, 646, 652,
	622, 652, 646,
	646, 622, 622,
	653, 468, 461,
	461, 647, 653,
	654, 653, 647,
	647, 648, 654,
	655, 654, 648,
	648, 649, 655,
	656, 655, 649,
	649, 650, 656,
	657, 656, 650,
	650, 651, 657,
	658, 657, 651,
	651, 652, 658,
	622, 658, 652,
	652, 622, 622,
	659, 475, 468,
	468, 653, 659,
	660, 659, 653,
	653, 654, 660,
	661, 660, 654,
	654, 655, 661,
	662, 661, 655,
	655, 656, 662,
	663, 662, 656,
	656, 657, 663,
	664, 663, 657,
	657, 658, 664,
	622, 664, 658,
	658, 622, 622,
	665, 482, 475,
	475, 659, 665,
	666, 665, 659,
	659, 660, 666,
	667, 666, 660,
	660, 661, 667,
	668, 667, 661,
	661, 662, 668,
	669, 668, 662,
	662, 663, 669,
	670, 669, 663,
	663, 664, 670,
	622, 670, 664,
	664, 622, 622,
	671, 489, 482,
	482, 665, 671,
	672, 671, 665,
	665, 666, 672,
	673, 672, 666,
	666, 667, 673,
	674, 673, 667,
	667, 668, 674,
	675, 674, 668,
	668, 669, 675,
	676, 675, 669,
	669, 670, 676,
	622, 676, 670,
	670, 622, 622,
	677, 496, 489,
	489, 671, 677,
	678, 677, 671,
	671, 672, 678,
	679, 678, 672,
	672, 673, 679,
	680, 679, 673,
	673, 674, 680,
	681, 680, 674,
	674, 675, 681,
	682, 681, 675,
	675, 676, 682,
	622, 682, 676,
	676, 622, 622,
	683, 503, 496,
	496, 677, 683,
	684, 683, 677,
	677, 678, 684,
	685, 684, 678,
	678, 679, 685,
	686, 685, 679,
	679, 680, 686,
	687, 686, 680,
	680, 681, 687,
	688, 687, 681,
	681, 682, 688,
	622, 688, 682,
	682, 622, 622,
	689, 510, 503,
	503, 683, 689,
	690, 689, 683,
	683, 684, 690,
	691, 690, 684,
	684, 685, 691,
	692, 691, 685,
	685, 686, 692,
	693, 692, 686,
	686, 687, 693,
	694, 693, 687,
	687, 688, 694,
	622, 694, 688,
	688, 622, 622,
	695, 517, 510,
	510, 689, 695,
	696, 695, 689,
	689, 690, 696,
	697, 696, 690,
	690, 691, 697,
	698, 697, 691,
	691, 692, 698,
	699, 698, 692,
	692, 693, 699,
	700, 699, 693,
	693, 694, 700,
	622, 700, 694,
	694, 622, 622,
	701, 524, 517,
	517, 695, 701,
	702, 701, 695,
	695, 696, 702,
	703, 702, 696,
	696, 697, 703,
	704, 703, 697,
	697, 698, 704,
	705, 704, 698,
	698, 699, 705,
	706, 705, 699,
	699, 700, 706,
	622, 706, 700,
	700, 622, 622,
	707, 531, 524,
	524, 701, 707,
	708, 707, 701,
	701, 702, 708,
	709, 708, 702,
	702, 703, 709,
	710, 709, 703,
	703, 704, 710,
	711, 710, 704,
	704, 705, 711,
	712, 711, 705,
	705, 706, 712,
	622, 712, 706,
	706, 622, 622,
	713, 538, 531,
	531, 707, 713,
	714, 713, 707,
	707, 708, 714,
	715, 714, 708,
	708, 709, 715,
	716, 715, 709,
	709, 710, 716,
	717, 716, 710,
	710, 711, 717,
	718, 717, 711,
	711, 712, 718,
	622, 718, 712,
	712, 622, 622,
	719, 545, 538,
	538, 713, 719,
	720, 719, 713,
	713, 714, 720,
	721, 720, 714,
	714, 715, 721,
	722, 721, 715,
	715, 716, 722,
	723, 722, 716,
	716, 717, 723,
	724, 723, 717,
	717, 718, 724,
	622, 724, 718,
	718, 622, 622,
	725, 552, 545,
	545, 719, 725,
	726, 725, 719,
	719, 720, 726,
	727, 726, 720,
	720, 721, 727,
	728, 727, 721,
	721, 722, 728,
	729, 728, 722,
	722, 723, 729,
	730, 729, 723,
	723, 724, 730,
	622, 730, 724,
	724, 622, 622,
	731, 559, 552,
	552, 725, 731,
	732, 731, 725,
	725, 726, 732,
	733, 732, 726,
	726, 727, 733,
	734, 733, 727,
	727, 728, 734,
	735, 734, 728,
	728, 729, 735,
	736, 735, 729,
	729, 730, 736,
	622, 736, 730,
	730, 622, 622,
	737, 566, 559,
	559, 731, 737,
	738, 737, 731,
	731, 732, 738,
	739, 738, 732,
	732, 733, 739,
	740, 739, 733,
	733, 734, 740,
	741, 740, 734,
	734, 735, 741,
	742, 741, 735,
	735, 736, 742,
	622, 742, 736,
	736, 622, 622,
	743, 573, 566,
	566, 737, 743,
	744, 743, 737,
	737, 738, 744,
	745, 744, 738,
	738, 739, 745,
	746, 745, 739,
	739, 740, 746,
	747, 746, 740,
	740, 741, 747,
	748, 747, 741,
	741, 742, 748,
	622, 748, 742,
	742, 622, 622,
	749, 580, 573,
	573, 743, 749,
	750, 749, 743,
	743, 744, 750,
	751, 750, 744,
	744, 745, 751,
	752, 751, 745,
	745, 746, 752,
	753, 752, 746,
	746, 747, 753,
	754, 753, 747,
	747, 748, 754,
	622, 754, 748,
	748, 622, 622,
	755, 587, 580,
	580, 749, 755,
	756, 755, 749,
	749, 750, 756,
	757, 756, 750,
	750, 751, 757,
	758, 757, 751,
	751, 752, 758,
	759, 758, 752,
	752, 753, 759,
	760, 759, 753,
	753, 754, 760,
	622, 760, 754,
	754, 622, 622,
	761, 594, 587,
	587, 755, 761,
	762, 761, 755,
	755, 756, 762,
	763, 762, 756,
	756, 757, 763,
	764, 763, 757,
	757, 758, 764,
	765, 764, 758,
	758, 759, 765,
	766, 765, 759,
	759, 760, 766,
	622, 766, 760,
	760, 622, 622,
	767, 601, 594,
	594, 761, 767,
	768, 767, 761,
	761, 762, 768,
	769, 768, 762,
	762, 763, 769,
	770, 769, 763,
	763, 764, 770,
	771, 770, 764,
	764, 765, 771,
	772, 771, 765,
	765, 766, 772,
	622, 772, 766,
	766, 622, 622,
	773, 608, 601,
	601, 767, 773,
	774, 773, 767,
	767, 768, 774,
	775, 774, 768,
	768, 769, 775,
	776, 775, 769,
	769, 770, 776,
	777, 776, 770,
	770, 771, 777,
	778, 777, 771,
	771, 772, 778,
	622, 778, 772,
	772, 622, 622,
	779, 615, 608,
	608, 773, 779,
	780, 779, 773,
	773, 774, 780,
	781, 780, 774,
	774, 775, 781,
	782, 781, 775,
	775, 776, 782,
	783, 782, 776,
	776, 777, 783,
	784, 783, 777,
	777, 778, 784,
	622, 784, 778,
	778, 622, 622,
	616, 426, 615,
	615, 779, 616,
	617, 616, 779,
	779, 780, 617,
	618, 617, 780,
	780, 781, 618,
	619, 618, 781,
	781, 782, 619,
	620, 619, 782,
	782, 783, 620,
	621, 620, 783,
	783, 784, 621,
	622, 621, 784,
	784, 622, 622,
	794, 793, 785,
	785, 786, 794,
	795, 794, 786,
	786, 787, 795,
	796, 795, 787,
	787, 788, 796,
	797, 796, 788,
	788, 789, 797,
	798, 797, 789,
	789, 790, 798,
	799, 798, 790,
	790, 791, 799,
	800, 799, 791,
	791, 792, 800,
	802, 801, 793,
	793, 794, 802,
	803, 802, 794,
	794, 795, 803,
	804, 803, 795,
	795, 796, 804,
	805, 804, 796,
	796, 797, 805,
	806, 805, 797,
	797, 798, 806,
	807, 806, 798,
	798, 799, 807,
	808, 807, 799,
	799, 800, 808,
	810, 809, 801,
	801, 802, 810,
	811, 810, 802,
	802, 803, 811,
	812, 811, 803,
	803, 804, 812,
	813, 812, 804,
	804, 805, 813,
	814, 813, 805,
	805, 806, 814,
	815, 814, 806,
	806, 807, 815,
	816, 815, 807,
	807, 808, 816,
	818, 817, 809,
	809, 810, 818,
	819, 818, 810,
	810, 811, 819,
	820, 819, 811,
	811, 812, 820,
	821, 820, 812,
	812, 813, 821,
	822, 821, 813,
	813, 814, 822,
	823, 822, 814,
	814, 815, 823,
	824, 823, 815,
	815, 816, 824,
	826, 825, 817,
	817, 818, 826,
	827, 826, 818,
	818, 819, 827,
	828, 827, 819,
	819, 820, 828,
	829, 828, 820,
	820, 821, 829,
	830, 829, 821,
	821, 822, 830,
	831, 830, 822,
	822, 823, 831,
	832, 831, 823,
	823, 824, 832,
	834, 833, 825,
	825, 826, 834,
	835, 834, 826,
	826, 827, 835,
	836, 835, 827,
	827, 828, 836,
	837, 836, 828,
	828, 829, 837,
	838, 837, 829,
	829, 830, 838,
	839, 838, 830,
	830, 831, 839,
	840, 839, 831,
	831, 832, 840,
	842, 841, 833,
	833, 834, 842,
	843, 842, 834,
	834, 835, 843,
	844, 843, 835,
	835, 836, 844,
	845, 844, 836,
	836, 837, 845,
	846, 845, 837,
	837, 838, 846,
	847, 846, 838,
	838, 839, 847,
	848, 847, 839,
	839, 840, 848,
	850, 849, 841,
	841, 842, 850,
	851, 850, 842,
	842, 843, 851,
	852, 851, 843,
	843, 844, 852,
	853, 852, 844,
	844, 845, 853,
	854, 853, 845,
	845, 846, 854,
	855, 854, 846,
	846, 847, 855,
	856, 855, 847,
	847, 848, 856,
	858, 857, 849,
	849, 850, 858,
	859, 858, 850,
	850, 851, 859,
	860, 859, 851,
	851, 852, 860,
	861, 860, 852,
	852, 853, 861,
	862, 861, 853,
	853, 854, 862,
	863, 862, 854,
	854, 855, 863,
	864, 863, 855,
	855, 856, 864,
	866, 865, 857,
	857, 858, 866,
	867, 866, 858,
	858, 859, 867,
	868, 867, 859,
	859, 860, 868,
	869, 868, 860,
	860, 861, 869,
	870, 869, 861,
	861, 862, 870,
	871, 870, 862,
	862, 863, 871,
	872, 871, 863,
	863, 864, 872,
	874, 873, 865,
	865, 866, 874,
	875, 874, 866,
	866, 867, 875,
	876, 875, 867,
	867, 868, 876,
	877, 876, 868,
	868, 869, 877,
	878, 877, 869,
	869, 870, 878,
	879, 878, 870,
	870, 871, 879,
	880, 879, 871,
	871, 872, 880,
	882, 881, 873,
	873, 874, 882,
	883, 882, 874,
	874, 875, 883,
	884, 883, 875,
	875, 876, 884,
	885, 884, 876,
	876, 877, 885,
	886, 885, 877,
	877, 878, 886,
	887, 886, 878,
	878, 879, 887,
	888, 887, 879,
	879, 880, 888,
	890, 889, 881,
	881, 882, 890,
	891, 890, 882,
	882, 883, 891,
	892, 891, 883,
	883, 884, 892,
	893, 892, 884,
	884, 885, 893,
	894, 893, 885,
	885, 886, 894,
	895, 894, 886,
	886, 887, 895,
	896, 895, 887,
	887, 888, 896,
	786, 785, 889,
	889, 890, 786,
	787, 786, 890,
	890, 891, 787,
	788, 787, 891,
	891, 892, 788,
	789, 788, 892,
	892, 893, 789,
	790, 789, 893,
	893, 894, 790,
	791, 790, 894,
	894, 895, 791,
	792, 791, 895,
	895, 896, 792,
	904, 800, 792,
	792, 897, 904,
	905, 904, 897,
	897, 898, 905,
	906, 905, 898,
	898, 899, 906,
	907, 906, 899,
	899, 900, 907,
	908, 907, 900,
	900, 901, 908,
	909, 908, 901,
	901, 902, 909,
	910, 909, 902,
	902, 903, 910,
	911, 808, 800,
	800, 904, 911,
	912, 911, 904,
	904, 905, 912,
	913, 912, 905,
	905, 906, 913,
	914, 913, 906,
	906, 907, 914,
	915, 914, 907,
	907, 908, 915,
	916, 915, 908,
	908, 909, 916,
	917, 916, 909,
	909, 910, 917,
	918, 816, 808,
	808, 911, 918,
	919, 918, 911,
	911, 912, 919,
	920, 919, 912,
	912, 913, 920,
	921, 920, 913,
	913, 914, 921,
	922, 921, 914,
	914, 915, 922,
	923, 922, 915,
	915, 916, 923,
	924, 923, 916,
	916, 917, 924,
	925, 824, 816,
	816, 918, 925,
	926, 925, 918,
	918, 919, 926,
	927, 926, 919,
	919, 920, 927,
	928, 927, 920,
	920, 921, 928,
	929, 928, 921,
	921, 922, 929,
	930, 929, 922,
	922, 923, 930,
	931, 930, 923,
	923, 924, 931,
	932, 832, 824,
	824, 925, 932,
	933, 932, 925,
	925, 926, 933,
	934, 933, 926,
	926, 927, 934,
	935, 934, 927,
	927, 928, 935,
	936, 935, 928,
	928, 929, 936,
	937, 936, 929,
	929, 930, 937,
	938, 937, 930,
	930, 931, 938,
	939, 840, 832,
	832, 932, 939,
	940, 939, 932,
	932, 933, 940,
	941, 940, 933,
	933, 934, 941,
	942, 941, 934,
	934, 935, 942,
	943, 942, 935,
	935, 936, 943,
	944, 943, 936,
	936, 937, 944,
	945, 944, 937,
	937, 938, 945,
	946, 848, 840,
	840, 939, 946,
	947, 946, 939,
	939, 940, 947,
	948, 947, 940,
	940, 941, 948,
	949, 948, 941,
	941, 942, 949,
	950, 949, 942,
	942, 943, 950,
	951, 950, 943,
	943, 944, 951,
	952, 951, 944,
	944, 945, 952,
	953, 856, 848,
	848, 946, 953,
	954, 953, 946,
	946, 947, 954,
	955, 954, 947,
	947, 948, 955,
	956, 955, 948,
	948, 949, 956,
	957, 956, 949,
	949, 950, 957,
	958, 957, 950,
	950, 951, 958,
	959, 958, 951,
	951, 952, 959,
	960, 864, 856,
	856, 953, 960,
	961, 960, 953,
	953, 954, 961,
	962, 961, 954,
	954, 955, 962,
	963, 962, 955,
	955, 956, 963,
	964, 963, 956,
	956, 957, 964,
	965, 964, 957,
	957, 958, 965,
	966, 965, 958,
	958, 959, 966,
	967, 872, 864,
	864, 960, 967,
	968, 967, 960,
	960, 961, 968,
	969, 968, 961,
	961, 962, 969,
	970, 969, 962,
	962, 963, 970,
	971, 970, 963,
	963, 964, 971,
	972, 971, 964,
	964, 965, 972,
	973, 972, 965,
	965, 966, 973,
	974, 880, 872,
	872, 967, 974,
	975, 974, 967,
	967, 968, 975,
	976, 975, 968,
	968, 969, 976,
	977, 976, 969,
	969, 970, 977,
	978, 977, 970,
	970, 971, 978,
	979, 978, 971,
	971, 972, 979,
	980, 979, 972,
	972, 973, 980,
	981, 888, 880,
	880, 974, 981,
	982, 981, 974,
	974, 975, 982,
	983, 982, 975,
	975, 976, 983,
	984, 983, 976,
	976, 977, 984,
	985, 984, 977,
	977, 978, 985,
	986, 985, 978,
	978, 979, 986,
	987, 986, 979,
	979, 980, 987,
	988, 896, 888,
	888, 981, 988,
	989, 988, 981,
	981, 982, 989,
	990, 989, 982,
	982, 983, 990,
	991, 990, 983,
	983, 984, 991,
	992, 991, 984,
	984, 985, 992,
	993, 992, 985,
	985, 986, 993,
	994, 993, 986,
	986, 987, 994,
	897, 792, 896,
	896, 988, 897,
	898, 897, 988,
	988, 989, 898,
	899, 898, 989,
	989, 990, 899,
	900, 899, 990,
	990, 991, 900,
	901, 900, 991,
	991, 992, 901,
	902, 901, 992,
	992, 993, 902,
	903, 902, 993,
	993, 994, 903,
	1004, 1003, 995,
	995, 996, 1004,
	1005, 1004, 996,
	996, 997, 1005,
	1006, 1005, 997,
	997, 998, 1006,
	1007, 1006, 998,
	998, 999, 1007,
	1008, 1007, 999,
	999, 1000, 1008,
	1009, 1008, 1000,
	1000, 1001, 1009,
	1010, 1009, 1001,
	1001, 1002, 1010,
	1012, 1011, 1003,
	1003, 1004, 1012,
	1013, 1012, 1004,
	1004, 1005, 1013,
	1014, 1013, 1005,
	1005, 1006, 1014,
	1015, 1014, 1006,
	1006, 1007, 1015,
	1016, 1015, 1007,
	1007, 1008, 1016,
	1017, 1016, 1008,
	1008, 1009, 1017,
	1018, 1017, 1009,
	1009, 1010, 1018,
	1020, 1019, 1011,
	1011, 1012, 1020,
	1021, 1020, 1012,
	1012, 1013, 1021,
	1022, 1021, 1013,
	1013, 1014, 1022,
	1023, 1022, 1014,
	1014, 1015, 1023,
	1024, 1023, 1015,
	1015, 1016, 1024,
	1025, 1024, 1016,
	1016, 1017, 1025,
	1026, 1025, 1017,
	1017, 1018, 1026,
	1028, 1027, 1019,
	1019, 1020, 1028,
	1029, 1028, 1020,
	1020, 1021, 1029,
	1030, 1029, 1021,
	1021, 1022, 1030,
	1031, 1030, 1022,
	1022, 1023, 1031,
	1032, 1031, 1023,
	1023, 1024, 1032,
	1033, 1032, 1024,
	1024, 1025, 1033,
	1034, 1033, 1025,
	1025, 1026, 1034,
	1036, 1035, 1027,
	1027, 1028, 1036,
	1037, 1036, 1028,
	1028, 1029, 1037,
	1038, 1037, 1029,
	1029, 1030, 1038,
	1039, 1038, 1030,
	1030, 1031, 1039,
	1040, 1039, 1031,
	1031, 1032, 1040,
	1041, 1040, 1032,
	1032, 1033, 1041,
	1042, 1041, 1033,
	1033, 1034, 1042,
	1044, 1043, 1035,
	1035, 1036, 1044,
	1045, 1044, 1036,
	1036, 1037, 1045,
	1046, 1045, 1037,
	1037, 1038, 1046,
	1047, 1046, 1038,
	1038, 1039, 1047,
	1048, 1047, 1039,
	1039, 1040, 1048,
	1049, 1048, 1040,
	1040, 1041, 1049,
	1050, 1049, 1041,
	1041, 1042, 1050,
	1052, 1051, 1043,
	1043, 1044, 1052,
	1053, 1052, 1044,
	1044, 1045, 1053,
	1054, 1053, 1045,
	1045, 1046, 1054,
	1055, 1054, 1046,
	1046, 1047, 1055,
	1056, 1055, 1047,
	1047, 1048, 1056,
	1057, 1056, 1048,
	1048, 1049, 1057,
	1058, 1057, 1049,
	1049, 1050, 1058,
	1060, 1059, 1051,
	1051, 1052, 1060,
	1061, 1060, 1052,
	1052, 1053, 1061,
	1062, 1061, 1053,
	1053, 1054, 1062,
	1063, 1062, 1054,
	1054, 1055, 1063,
	1064, 1063, 1055,
	1055, 1056, 1064,
	1065, 1064, 1056,
	1056, 1057, 1065,
	1066, 1065, 1057,
	1057, 1058, 1066,
	1068, 1067, 1059,
	1059, 1060, 1068,
	1069, 1068, 1060,
	1060, 1061, 1069,
	1070, 1069, 1061,
	1061, 1062, 1070,
	1071, 1070, 1062,
	1062, 1063, 1071,
	1072, 1071, 1063,
	1063, 1064, 1072,
	1073, 1072, 1064,
	1064, 1065, 1073,
	1074, 1073, 1065,
	1065, 1066, 1074,
	1076, 1075, 1067,
	1067, 1068, 1076,
	1077, 1076, 1068,
	1068, 1069, 1077,
	1078, 1077, 1069,
	1069, 1070, 1078,
	1079, 1078, 1070,
	1070, 1071, 1079,
	1080, 1079, 1071,
	1071, 1072, 1080,
	1081, 1080, 1072,
	1072, 1073, 1081,
	1082, 1081, 1073,
	1073, 1074, 1082,
	1084, 1083, 1075,
	1075, 1076, 1084,
	1085, 1084, 1076,
	1076, 1077, 1085,
	1086, 1085, 1077,
	1077, 1078, 1086,
	1087, 1086, 1078,
	1078, 1079, 1087,
	1088, 1087, 1079,
	1079, 1080, 1088,
	1089, 1088, 1080,
	1080, 1081, 1089,
	1090, 1089, 1081,
	1081, 1082, 1090,
	1092, 1091, 1083,
	1083, 1084, 1092,
	1093, 1092, 1084,
	1084, 1085, 1093,
	1094, 1093, 1085,
	1085, 1086, 1094,
	1095, 1094, 1086,
	1086, 1087, 1095,
	1096, 1095, 1087,
	1087, 1088, 1096,
	1097, 1096, 1088,
	1088, 1089, 1097,
	1098, 1097, 1089,
	1089, 1090, 1098,
	1100, 1099, 1091,
	1091, 1092, 1100,
	1101, 1100, 1092,
	1092, 1093, 1101,
	1102, 1101, 1093,
	1093, 1094, 1102,
	1103, 1102, 1094,
	1094, 1095, 1103,
	1104, 1103, 1095,
	1095, 1096, 1104,
	1105, 1104, 1096,
	1096, 1097, 1105,
	1106, 1105, 1097,
	1097, 1098, 1106,
	996, 995, 1099,
	1099, 1100, 996,
	997, 996, 1100,
	1100, 1101, 997,
	998, 997, 1101,
	1101, 1102, 998,
	999, 998, 1102,
	1102, 1103, 999,
	1000, 999, 1103,
	1103, 1104, 1000,
	1001, 1000, 1104,
	1104, 1105, 1001,
	1002, 1001, 1105,
	1105, 1106, 1002,
	1114, 1010, 1002,
	1002, 1107, 1114,
	1115, 1114, 1107,
	1107, 1108, 1115,
	1116, 1115, 1108,
	1108, 1109, 1116,
	1117, 1116, 1109,
	1109, 1110, 1117,
	1118, 1117, 1110,
	1110, 1111, 1118,
	1119, 1118, 1111,
	1111, 1112, 1119,
	1120, 1119, 1112,
	1112, 1113, 1120,
	1121, 1018, 1010,
	1010, 1114, 1121,
	1122, 1121, 1114,
	1114, 1115, 1122,
	1123, 1122, 1115,
	1115, 1116, 1123,
	1124, 1123, 1116,
	1116, 1117, 1124,
	1125, 1124, 1117,
	1117, 1118, 1125,
	1126, 1125, 1118,
	1118, 1119, 1126,
	1127, 1126, 1119,
	1119, 1120, 1127,
	1128, 1026, 1018,
	1018, 1121, 1128,
	1129, 1128, 1121,
	1121, 1122, 1129,
	1130, 1129, 1122,
	1122, 1123, 1130,
	1131, 1130, 1123,
	1123, 1124, 1131,
	1132, 1131, 1124,
	1124, 1125, 1132,
	1133, 1132, 1125,
	1125, 1126, 1133,
	1134, 1133, 1126,
	1126, 1127, 1134,
	1135, 1034, 1026,
	1026, 1128, 1135,
	1136, 1135, 1128,
	1128, 1129, 1136,
	1137, 1136, 1129,
	1129, 1130, 1137,
	1138, 1137, 1130,
	1130, 1131, 1138,
	1139, 1138, 1131,
	1131, 1132, 1139,
	1140, 1139, 1132,
	1132, 1133, 1140,
	1141, 1140, 1133,
	1133, 1134, 1141,
	1142, 1042, 1034,
	1034, 1135, 1142,
	1143, 1142, 1135,
	1135, 1136, 1143,
	1144, 1143, 1136,
	1136, 1137, 1144,
	1145, 1144, 1137,
	1137, 1138, 1145,
	1146, 1145, 1138,
	1138, 1139, 1146,
	1147, 1146, 1139,
	1139, 1140, 1147,
	1148, 1147, 1140,
	1140, 1141, 1148,
	1149, 1050, 1042,
	1042, 1142, 1149,
	1150, 1149, 1142,
	1142, 1143, 1150,
	1151, 1150, 1143,
	1143, 1144, 1151,
	1152, 1151, 1144,
	1144, 1145, 1152,
	1153, 1152, 1145,
	1145, 1146, 1153,
	1154, 1153, 1146,
	1146, 1147, 1154,
	1155, 1154, 1147,
	1147, 1148, 1155,
	1156, 1058, 1050,
	1050, 1149, 1156,
	1157, 1156, 1149,
	1149, 1150, 1157,
	1158, 1157, 1150,
	1150, 1151, 1158,
	1159, 1158, 1151,
	1151, 1152, 1159,
	1160, 1159, 1152,
	1152, 1153, 1160,
	1161, 1160, 1153,
	1153, 1154, 1161,
	1162, 1161, 1154,
	1154, 1155, 1162,
	1163, 1066, 1058,
	1058, 1156, 1163,
	1164, 1163, 1156,
	1156, 1157, 1164,
	1165, 1164, 1157,
	1157, 1158, 1165,
	1166, 1165, 1158,
	1158, 1159, 1166,
	1167, 1166, 1159,
	1159, 1160, 1167,
	1168, 1167, 1160,
	1160, 1161, 1168,
	1169, 1168, 1161,
	1161, 1162, 1169,
	1170, 1074, 1066,
	1066, 1163, 1170,
	1171, 1170, 1163,
	1163, 1164, 1171,
	1172, 1171, 1164,
	1164, 1165, 1172,
	1173, 1172, 1165,
	1165, 1166, 1173,
	1174, 1173, 1166,
	1166, 1167, 1174,
	1175, 1174, 1167,
	1167, 1168, 1175,
	1176, 1175, 1168,
	1168, 1169, 1176,
	1177, 1082, 1074,
	1074, 1170, 1177,
	1178, 1177, 1170,
	1170, 1171, 1178,
	1179, 1178, 1171,
	1171, 1172, 1179,
	1180, 1179, 1172,
	1172, 1173, 1180,
	1181, 1180, 1173,
	1173, 1174, 1181,
	1182, 1181, 1174,
	1174, 1175, 1182,
	1183, 1182, 1175,
	1175, 1176, 1183,
	1184, 1090, 1082,
	1082, 1177, 1184,
	1185, 1184, 1177,
	1177, 1178, 1185,
	1186, 1185, 1178,
	1178, 1179, 1186,
	1187, 1186, 1179,
	1179, 1180, 1187,
	1188, 1187, 1180,
	1180, 1181, 1188,
	1189, 1188, 1181,
	1181, 1182, 1189,
	1190, 1189, 1182,
	1182, 1183, 1190,
	1191, 1098, 1090,
	1090, 1184, 1191,
	1192, 1191, 1184,
	1184, 1185, 1192,
	1193, 1192, 1185,
	1185, 1186, 1193,
	1194, 1193, 1186,
	1186, 1187, 1194,
	1195, 1194, 1187,
	1187, 1188, 1195,
	1196, 1195, 1188,
	1188, 1189, 1196,
	1197, 1196, 1189,
	1189, 1190, 1197,
	1198, 1106, 1098,
	1098, 1191, 1198,
	1199, 1198, 1191,
	1191, 1192, 1199,
	1200, 1199, 1192,
	1192, 1193, 1200,
	1201, 1200, 1193,
	1193, 1194, 1201,
	1202, 1201, 1194,
	1194, 1195, 1202,
	1203, 1202, 1195,
	1195, 1196, 1203,
	1204, 1203, 1196,
	1196, 1197, 1204,
	1107, 1002, 1106,
	1106, 1198, 1107,
	1108, 1107, 1198,
	1198, 1199, 1108,
	1109, 1108, 1199,
	1199, 1200, 1109,
	1110, 1109, 1200,
	1200, 1201, 1110,
	1111, 1110, 1201,
	1201, 1202, 1111,
	1112, 1111, 1202,
	1202, 1203, 1112,
	1113, 1112, 1203,
	1203, 1204, 1113,
	1213, 1205, 1205,
	1205, 1206, 1213,
	1214, 1213, 1206,
	1206, 1207, 1214,
	1215, 1214, 1207,
	1207, 1208, 1215,
	1216, 1215, 1208,
	1208, 1209, 1216,
	1217, 1216, 1209,
	1209, 1210, 1217,
	1218, 1217, 1210,
	1210, 1211, 1218,
	1219, 1218, 1211,
	1211, 1212, 1219,
	1220, 1205, 1205,
	1205, 1213, 1220,
	1221, 1220, 1213,
	1213, 1214, 1221,
	1222, 1221, 1214,
	1214, 1215, 1222,
	1223, 1222, 1215,
	1215, 1216, 1223,
	1224, 1223, 1216,
	1216, 1217, 1224,
	1225, 1224, 1217,
	1217, 1218, 1225,
	1226, 1225, 1218,
	1218, 1219, 1226,
	1227, 1205, 1205,
	1205, 1220, 1227,
	1228, 1227, 1220,
	1220, 1221, 1228,
	1229, 1228, 1221,
	1221, 1222, 1229,
	1230, 1229, 1222,
	1222, 1223, 1230,
	1231, 1230, 1223,
	1223, 1224, 1231,
	1232, 1231, 1224,
	1224, 1225, 1232,
	1233, 1232, 1225,
	1225, 1226, 1233,
	1234, 1205, 1205,
	1205, 1227, 1234,
	1235, 1234, 1227,
	1227, 1228, 1235,
	1236, 1235, 1228,
	1228, 1229, 1236,
	1237, 1236, 1229,
	1229, 1230, 1237,
	1238, 1237, 1230,
	1230, 1231, 1238,
	1239, 1238, 1231,
	1231, 1232, 1239,
	1240, 1239, 1232,
	1232, 1233, 1240,
	1241, 1205, 1205,
	1205, 1234, 1241,
	1242, 1241, 1234,
	1234, 1235, 1242,
	1243, 1242, 1235,
	1235, 1236, 1243,
	1244, 1243, 1236,
	1236, 1237, 1244,
	1245, 1244, 1237,
	1237, 1238, 1245,
	1246, 1245, 1238,
	1238, 1239, 1246,
	1247, 1246, 1239,
	1239, 1240, 1247,
	1248, 1205, 1205,
	1205, 1241, 1248,
	1249, 1248, 1241,
	1241, 1242, 1249,
	1250, 1249, 1242,
	1242, 1243, 1250,
	1251, 1250, 1243,
	1243, 1244, 1251,
	1252, 1251, 1244,
	1244, 1245, 1252,
	1253, 1252, 1245,
	1245, 1246, 1253,
	1254, 1253, 1246,
	1246, 1247, 1254,
	1255, 1205, 1205,
	1205, 1248, 1255,
	1256, 1255, 1248,
	1248, 1249, 1256,
	1257, 1256, 1249,
	1249, 1250, 1257,
	1258, 1257, 1250,
	1250, 1251, 1258,
	1259, 1258, 1251,
	1251, 1252, 1259,
	1260, 1259, 1252,
	1252, 1253, 1260,
	1261, 1260, 1253,
	1253, 1254, 1261,
	1262, 1205, 1205,
	1205, 1255, 1262,
	1263, 1262, 1255,
	1255, 1256, 1263,
	1264, 1263, 1256,
	1256, 1257, 1264,
	1265, 1264, 1257,
	1257, 1258, 1265,
	1266, 1265, 1258,
	1258, 1259, 1266,
	1267, 1266, 1259,
	1259, 1260, 1267,
	1268, 1267, 1260,
	1260, 1261, 1268,
	1269, 1205, 1205,
	1205, 1262, 1269,
	1270, 1269, 1262,
	1262, 1263, 1270,
	1271, 1270, 1263,
	1263, 1264, 1271,
	1272, 1271, 1264,
	1264, 1265, 1272,
	1273, 1272, 1265,
	1265, 1266, 1273,
	1274, 1273, 1266,
	1266, 1267, 1274,
	1275, 1274, 1267,
	1267, 1268, 1275,
	1276, 1205, 1205,
	1205, 1269, 1276,
	1277, 1276, 1269,
	1269, 1270, 1277,
	1278, 1277, 1270,
	1270, 1271, 1278,
	1279, 1278, 1271,
	1271, 1272, 1279,
	1280, 1279, 1272,
	1272, 1273, 1280,
	1281, 1280, 1273,
	1273, 1274, 1281,
	1282, 1281, 1274,
	1274, 1275, 1282,
	1283, 1205, 1205,
	1205, 1276, 1283,
	1284, 1283, 1276,
	1276, 1277, 1284,
	1285, 1284, 1277,
	1277, 1278, 1285,
	1286, 1285, 1278,
	1278, 1279, 1286,
	1287, 1286, 1279,
	1279, 1280, 1287,
	1288, 1287, 1280,
	1280, 1281, 1288,
	1289, 1288, 1281,
	1281, 1282, 1289,
	1290, 1205, 1205,
	1205, 1283, 1290,
	1291, 1290, 1283,
	1283, 1284, 1291,
	1292, 1291, 1284,
	1284, 1285, 1292,
	1293, 1292, 1285,
	1285, 1286, 1293,
	1294, 1293, 1286,
	1286, 1287, 1294,
	1295, 1294, 1287,
	1287, 1288, 1295,
	1296, 1295, 1288,
	1288, 1289, 1296,
	1297, 1205, 1205,
	1205, 1290, 1297,
	1298, 1297, 1290,
	1290, 1291, 1298,
	1299, 1298, 1291,
	1291, 1292, 1299,
	1300, 1299, 1292,
	1292, 1293, 1300,
	1301, 1300, 1293,
	1293, 1294, 1301,
	1302, 1301, 1294,
	1294, 1295, 1302,
	1303, 1302, 1295,
	1295, 1296, 1303,
	1304, 1205, 1205,
	1205, 1297, 1304,
	1305, 1304, 1297,
	1297, 1298, 1305,
	1306, 1305, 1298,
	1298, 1299, 1306,
	1307, 1306, 1299,
	1299, 1300, 1307,
	1308, 1307, 1300,
	1300, 1301, 1308,
	1309, 1308, 1301,
	1301, 1302, 1309,
	1310, 1309, 1302,
	1302, 1303, 1310,
	1311, 1205, 1205,
	1205, 1304, 1311,
	1312, 1311, 1304,
	1304, 1305, 1312,
	1313, 1312, 1305,
	1305, 1306, 1313,
	1314, 1313, 1306,
	1306, 1307, 1314,
	1315, 1314, 1307,
	1307, 1308, 1315,
	1316, 1315, 1308,
	1308, 1309, 1316,
	1317, 1316, 1309,
	1309, 1310, 1317,
	1318, 1205, 1205,
	1205, 1311, 1318,
	1319, 1318, 1311,
	1311, 1312, 1319,
	1320, 1319, 1312,
	1312, 1313, 1320,
	1321, 1320, 1313,
	1313, 1314, 1321,
	1322, 1321, 1314,
	1314, 1315, 1322,
	1323, 1322, 1315,
	1315, 1316, 1323,
	1324, 1323, 1316,
	1316, 1317, 1324,
	1325, 1205, 1205,
	1205, 1318, 1325,
	1326, 1325, 1318,
	1318, 1319, 1326,
	1327, 1326, 1319,
	1319, 1320, 1327,
	1328, 1327, 1320,
	1320, 1321, 1328,
	1329, 1328, 1321,
	1321, 1322, 1329,
	1330, 1329, 1322,
	1322, 1323, 1330,
	1331, 1330, 1323,
	1323, 1324, 1331,
	1332, 1205, 1205,
	1205, 1325, 1332,
	1333, 1332, 1325,
	1325, 1326, 1333,
	1334, 1333, 1326,
	1326, 1327, 1334,
	1335, 1334, 1327,
	1327, 1328, 1335,
	1336, 1335, 1328,
	1328, 1329, 1336,
	1337, 1336, 1329,
	1329, 1330, 1337,
	1338, 1337, 1330,
	1330, 1331, 1338,
	1339, 1205, 1205,
	1205, 1332, 1339,
	1340, 1339, 1332,
	1332, 1333, 1340,
	1341, 1340, 1333,
	1333, 1334, 1341,
	1342, 1341, 1334,
	1334, 1335, 1342,
	1343, 1342, 1335,
	1335, 1336, 1343,
	1344, 1343, 1336,
	1336, 1337, 1344,
	1345, 1344, 1337,
	1337, 1338, 1345,
	1346, 1205, 1205,
	1205, 1339, 1346,
	1347, 1346, 1339,
	1339, 1340, 1347,
	1348, 1347, 1340,
	1340, 1341, 1348,
	1349, 1348, 1341,
	1341, 1342, 1349,
	1350, 1349, 1342,
	1342, 1343, 1350,
	1351, 1350, 1343,
	1343, 1344, 1351,
	1352, 1351, 1344,
	1344, 1345, 1352,
	1353, 1205, 1205,
	1205, 1346, 1353,
	1354, 1353, 1346,
	1346, 1347, 1354,
	1355, 1354, 1347,
	1347, 1348, 1355,
	1356, 1355, 1348,
	1348, 1349, 1356,
	1357, 1356, 1349,
	1349, 1350, 1357,
	1358, 1357, 1350,
	1350, 1351, 1358,
	1359, 1358, 1351,
	1351, 1352, 1359,
	1360, 1205, 1205,
	1205, 1353, 1360,
	1361, 1360, 1353,
	1353, 1354, 1361,
	1362, 1361, 1354,
	1354, 1355, 1362,
	1363, 1362, 1355,
	1355, 1356, 1363,
	1364, 1363, 1356,
	1356, 1357, 1364,
	1365, 1364, 1357,
	1357, 1358, 1365,
	1366, 1365, 1358,
	1358, 1359, 1366,
	1367, 1205, 1205,
	1205, 1360, 1367,
	1368, 1367, 1360,
	1360, 1361, 1368,
	1369, 1368, 1361,
	1361, 1362, 1369,
	1370, 1369, 1362,
	1362, 1363, 1370,
	1371, 1370, 1363,
	1363, 1364, 1371,
	1372, 1371, 1364,
	1364, 1365, 1372,
	1373, 1372, 1365,
	1365, 1366, 1373,
	1374, 1205, 1205,
	1205, 1367, 1374,
	1375, 1374, 1367,
	1367, 1368, 1375,
	1376, 1375, 1368,
	1368, 1369, 1376,
	1377, 1376, 1369,
	1369, 1370, 1377,
	1378, 1377, 1370,
	1370, 1371, 1378,
	1379, 1378, 1371,
	1371, 1372, 1379,
	1380, 1379, 1372,
	1372, 1373, 1380,
	1381, 1205, 1205,
	1205, 1374, 1381,
	1382, 1381, 1374,
	1374, 1375, 1382,
	1383, 1382, 1375,
	1375, 1376, 1383,
	1384, 1383, 1376,
	1376, 1377, 1384,
	1385, 1384, 1377,
	1377, 1378, 1385,
	1386, 1385, 1378,
	1378, 1379, 1386,
	1387, 1386, 1379,
	1379, 1380, 1387,
	1388, 1205, 1205,
	1205, 1381, 1388,
	1389, 1388, 1381,
	1381, 1382, 1389,
	1390, 1389, 1382,
	1382, 1383, 1390,
	1391, 1390, 1383,
	1383, 1384, 1391,
	1392, 1391, 1384,
	1384, 1385, 1392,
	1393, 1392, 1385,
	1385, 1386, 1393,
	1394, 1393, 1386,
	1386, 1387, 1394,
	1395, 1205, 1205,
	1205, 1388, 1395,
	1396, 1395, 1388,
	1388, 1389, 1396,
	1397, 1396, 1389,
	1389, 1390, 1397,
	1398, 1397, 1390,
	1390, 1391, 1398,
	1399, 1398, 1391,
	1391, 1392, 1399,
	1400, 1399, 1392,
	1392, 1393, 1400,
	1401, 1400, 1393,
	1393, 1394, 1401,
	1206, 1205, 1205,
	1205, 1395, 1206,
	1207, 1206, 1395,
	1395, 1396, 1207,
	1208, 1207, 1396,
	1396, 1397, 1208,
	1209, 1208, 1397,
	1397, 1398, 1209,
	1210, 1209, 1398,
	1398, 1399, 1210,
	1211, 1210, 1399,
	1399, 1400, 1211,
	1212, 1211, 1400,
	1400, 1401, 1212,
	1409, 1219, 1212,
	1212, 1402, 1409,
	1410, 1409, 1402,
	1402, 1403, 1410,
	1411, 1410, 1403,
	1403, 1404, 1411,
	1412, 1411, 1404,
	1404, 1405, 1412,
	1413, 1412, 1405,
	1405, 1406, 1413,
	1414, 1413, 1406,
	1406, 1407, 1414,
	1415, 1414, 1407,
	1407, 1408, 1415,
	1416, 1226, 1219,
	1219, 1409, 1416,
	1417, 1416, 1409,
	1409, 1410, 1417,
	1418, 1417, 1410,
	1410, 1411, 1418,
	1419, 1418, 1411,
	1411, 1412, 1419,
	1420, 1419, 1412,
	1412, 1413, 1420,
	1421, 1420, 1413,
	1413, 1414, 1421,
	1422, 1421, 1414,
	1414, 1415, 1422,
	1423, 1233, 1226,
	1226, 1416, 1423,
	1424, 1423, 1416,
	1416, 1417, 1424,
	1425, 1424, 1417,
	1417, 1418, 1425,
	1426, 1425, 1418,
	1418, 1419, 1426,
	1427, 1426, 1419,
	1419, 1420, 1427,
	1428, 1427, 1420,
	1420, 1421, 1428,
	1429, 1428, 1421,
	1421, 1422, 1429,
	1430, 1240, 1233,
	1233, 1423, 1430,
	1431, 1430, 1423,
	1423, 1424, 1431,
	1432, 1431, 1424,
	1424, 1425, 1432,
	1433, 1432, 1425,
	1425, 1426, 1433,
	1434, 1433, 1426,
	1426, 1427, 1434,
	1435, 1434, 1427,
	1427, 1428, 1435,
	1436, 1435, 1428,
	1428, 1429, 1436,
	1437, 1247, 1240,
	1240, 1430, 1437,
	1438, 1437, 1430,
	1430, 1431, 1438,
	1439, 1438, 1431,
	1431, 1432, 1439,
	1440, 1439, 1432,
	1432, 1433, 1440,
	1441, 1440, 1433,
	1433, 1434, 1441,
	1442, 1441, 1434,
	1434, 1435, 1442,
	1443, 1442, 1435,
	1435, 1436, 1443,
	1444, 1254, 1247,
	1247, 1437, 1444,
	1445, 1444, 1437,
	1437, 1438, 1445,
	1446, 1445, 1438,
	1438, 1439, 1446,
	1447, 1446, 1439,
	1439, 1440, 1447,
	1448, 1447, 1440,
	1440, 1441, 1448,
	1449, 1448, 1441,
	1441, 1442, 1449,
	1450, 1449, 1442,
	1442, 1443, 1450,
	1451, 1261, 1254,
	1254, 1444, 1451,
	1452, 1451, 1444,
	1444, 1445, 1452,
	1453, 1452, 1445,
	1445, 1446, 1453,
	1454, 1453, 1446,
	1446, 1447, 1454,
	1455, 1454, 1447,
	1447, 1448, 1455,
	1456, 1455, 1448,
	1448, 1449, 1456,
	1457, 1456, 1449,
	1449, 1450, 1457,
	1458, 1268, 1261,
	1261, 1451, 1458,
	1459, 1458, 1451,
	1451, 1452, 1459,
	1460, 1459, 1452,
	1452, 1453, 1460,
	1461, 1460, 1453,
	1453, 1454, 1461,
	1462, 1461, 1454,
	1454, 1455, 1462,
	1463, 1462, 1455,
	1455, 1456, 1463,
	1464, 1463, 1456,
	1456, 1457, 1464,
	1465, 1275, 1268,
	1268, 1458, 1465,
	1466, 1465, 1458,
	1458, 1459, 1466,
	1467, 1466, 1459,
	1459, 1460, 1467,
	1468, 1467, 1460,
	1460, 1461, 1468,
	1469, 1468, 1461,
	1461, 1462, 1469,
	1470, 1469, 1462,
	1462, 1463, 1470,
	1471, 1470, 1463,
	1463, 1464, 1471,
	1472, 1282, 1275,
	1275, 1465, 1472,
	1473, 1472, 1465,
	1465, 1466, 1473,
	1474, 1473, 1466,
	1466, 1467, 1474,
	1475, 1474, 1467,
	1467, 1468, 1475,
	1476, 1475, 1468,
	1468, 1469, 1476,
	1477, 1476, 1469,
	1469, 1470, 1477,
	1478, 1477, 1470,
	1470, 1471, 1478,
	1479, 1289, 1282,
	1282, 1472, 1479,
	1480, 1479, 1472,
	1472, 1473, 1480,
	1481, 1480, 1473,
	1473, 1474, 1481,
	1482, 1481, 1474,
	1474, 1475, 1482,
	1483, 1482, 1475,
	1475, 1476, 1483,
	1484, 1483, 1476,
	1476, 1477, 1484,
	1485, 1484, 1477,
	1477, 1478, 1485,
	1486, 1296, 1289,
	1289, 1479, 1486,
	1487, 1486, 1479,
	1479, 1480, 1487,
	1488, 1487, 1480,
	1480, 1481, 1488,
	1489, 1488, 1481,
	1481, 1482, 1489,
	1490, 1489, 1482,
	1482, 1483, 1490,
	1491, 1490, 1483,
	1483, 1484, 1491,
	1492, 1491, 1484,
	1484, 1485, 1492,
	1493, 1303, 1296,
	1296, 1486, 1493,
	1494, 1493, 1486,
	1486, 1487, 1494,
	1495, 1494, 1487,
	1487, 1488, 1495,
	1496, 1495, 1488,
	1488, 1489, 1496,
	1497, 1496, 1489,
	1489, 1490, 1497,
	1498, 1497, 1490,
	1490, 1491, 1498,
	1499, 1498, 1491,
	1491, 1492, 1499,
	1500, 1310, 1303,
	1303, 1493, 1500,
	1501, 1500, 1493,
	1493, 1494, 1501,
	1502, 1501, 1494,
	1494, 1495, 1502,
	1503, 1502, 1495,
	1495, 1496, 1503,
	1504, 1503, 1496,
	1496, 1497, 1504,
	1505, 1504, 1497,
	1497, 1498, 1505,
	1506, 1505, 1498,
	1498, 1499, 1506,
	1507, 1317, 1310,
	1310, 1500, 1507,
	1508, 1507, 1500,
	1500, 1501, 1508,
	1509, 1508, 1501,
	1501, 1502, 1509,
	1510, 1509, 1502,
	1502, 1503, 1510,
	1511, 1510, 1503,
	1503, 1504, 1511,
	1512, 1511, 1504,
	1504, 1505, 1512,
	1513, 1512, 1505,
	1505, 1506, 1513,
	1514, 1324, 1317,
	1317, 1507, 1514,
	1515, 1514, 1507,
	1507, 1508, 1515,
	1516, 1515, 1508,
	1508, 1509, 1516,
	1517, 1516, 1509,
	1509, 1510, 1517,
	1518, 1517, 1510,
	1510, 1511, 1518,
	1519, 1518, 1511,
	1511, 1512, 1519,
	1520, 1519, 1512,
	1512, 1513, 1520,
	1521, 1331, 1324,
	1324, 1514, 1521,
	1522, 1521, 1514,
	1514, 1515, 1522,
	1523, 1522, 1515,
	1515, 1516, 1523,
	1524, 1523, 1516,
	1516, 1517, 1524,
	1525, 1524, 1517,
	1517, 1518, 1525,
	1526, 1525, 1518,
	1518, 1519, 1526,
	1527, 1526, 1519,
	1519, 1520, 1527,
	1528, 1338, 1331,
	1331, 1521, 1528,
	1529, 1528, 1521,
	1521, 1522, 1529,
	1530, 1529, 1522,
	1522, 1523, 1530,
	1531, 1530, 1523,
	1523, 1524, 1531,
	1532, 1531, 1524,
	1524, 1525, 1532,
	1533, 1532, 1525,
	1525, 1526, 1533,
	1534, 1533, 1526,
	1526, 1527, 1534,
	1535, 1345, 1338,
	1338, 1528, 1535,
	1536, 1535, 1528,
	1528, 1529, 1536,
	1537, 1536, 1529,
	1529, 1530, 1537,
	1538, 1537, 1530,
	1530, 1531, 1538,
	1539, 1538, 1531,
	1531, 1532, 1539,
	1540, 1539, 1532,
	1532, 1533, 1540,
	1541, 1540, 1533,
	1533, 1534, 1541,
	1542, 1352, 1345,
	1345, 1535, 1542,
	1543, 1542, 1535,
	1535, 1536, 1543,
	1544, 1543, 1536,
	1536, 1537, 1544,
	1545, 1544, 1537,
	1537, 1538, 1545,
	1546, 1545, 1538,
	1538, 1539, 1546,
	1547, 1546, 1539,
	1539, 1540, 1547,
	1548, 1547, 1540,
	1540, 1541, 1548,
	1549, 1359, 1352,
	1352, 1542, 1549,
	1550, 1549, 1542,
	1542, 1543, 1550,
	1551, 1550, 1543,
	1543, 1544, 1551,
	1552, 1551, 1544,
	1544, 1545, 1552,
	1553, 1552, 1545,
	1545, 1546, 1553,
	1554, 1553, 1546,
	1546, 1547, 1554,
	1555, 1554, 1547,
	1547, 1548, 1555,
	1556, 1366, 1359,
	1359, 1549, 1556,
	1557, 1556, 1549,
	1549, 1550, 1557,
	1558, 1557, 1550,
	1550, 1551, 1558,
	1559, 1558, 1551,
	1551, 1552, 1559,
	1560, 1559, 1552,
	1552, 1553, 1560,
	1561, 1560, 1553,
	1553, 1554, 1561,
	1562, 1561, 1554,
	1554, 1555, 1562,
	1563, 1373, 1366,
	1366, 1556, 1563,
	1564, 1563, 1556,
	1556, 1557, 1564,
	1565, 1564, 1557,
	1557, 1558, 1565,
	1566, 1565, 1558,
	1558, 1559, 1566,
	1567, 1566, 1559,
	1559, 1560, 1567,
	1568, 1567, 1560,
	1560, 1561, 1568,
	1569, 1568, 1561,
	1561, 1562, 1569,
	1570, 1380, 1373,
	1373, 1563, 1570,
	1571, 1570, 1563,
	1563, 1564, 1571,
	1572, 1571, 1564,
	1564, 1565, 1572,
	1573, 1572, 1565,
	1565, 1566, 1573,
	1574, 1573, 1566,
	1566, 1567, 1574,
	1575, 1574, 1567,
	1567, 1568, 1575,
	1576, 1575, 1568,
	1568, 1569, 1576,
	1577, 1387, 1380,
	1380, 1570, 1577,
	1578, 1577, 1570,
	1570, 1571, 1578,
	1579, 1578, 1571,
	1571, 1572, 1579,
	1580, 1579, 1572,
	1572, 1573, 1580,
	1581, 1580, 1573,
	1573, 1574, 1581,
	1582, 1581, 1574,
	1574, 1575, 1582,
	1583, 1582, 1575,
	1575, 1576, 1583,
	1584, 1394, 1387,
	1387, 1577, 1584,
	1585, 1584, 1577,
	1577, 1578, 1585,
	1586, 1585, 1578,
	1578, 1579, 1586,
	1587, 1586, 1579,
	1579, 1580, 1587,
	1588, 1587, 1580,
	1580, 1581, 1588,
	1589, 1588, 1581,
	1581, 1582, 1589,
	1590, 1589, 1582,
	1582, 1583, 1590,
	1591, 1401, 1394,
	1394, 1584, 1591,
	1592, 1591, 1584,
	1584, 1585, 1592,
	1593, 1592, 1585,
	1585, 1586, 1593,
	1594, 1593, 1586,
	1586, 1587, 1594,
	1595, 1594, 1587,
	1587, 1588, 1595,
	1596, 1595, 1588,
	1588, 1589, 1596,
	1597, 1596, 1589,
	1589, 1590, 1597,
	1402, 1212, 1401,
	1401, 1591, 1402,
	1403, 1402, 1591,
	1591, 1592, 1403,
	1404, 1403, 1592,
	1592, 1593, 1404,
	1405, 1404, 1593,
	1593, 1594, 1405,
	1406, 1405, 1594,
	1594, 1595, 1406,
	1407, 1406, 1595,
	1595, 1596, 1407,
	1408, 1407, 1596,
	1596, 1597, 1408
};

float teapot_normals[3136 * 3] = {
	-0.799826, 0.597761, -0.054409,
	0.861688, 0.504041, 0.058618,
	0.818396, 0.181721, -0.545165,
	0.351371, -0.296064, 0.888192,
	0.183703, -0.551110, 0.813960,
	-0.296793, 0.890381, 0.345160,
	-0.413859, 0.867524, 0.275905,
	0.817730, 0.184735, -0.545152,
	0.892054, 0.353084, -0.282084,
	-0.028938, -0.780956, 0.623916,
	0.358845, -0.286686, 0.888280,
	-0.780964, 0.623923, -0.028556,
	0.000000, 0.875092, 0.483956,
	0.000000, -0.423057, -0.906103,
	0.492047, 0.000000, 0.870569,
	-0.907166, 0.000000, -0.420773,
	0.356819, 0.734121, 0.577708,
	-0.436338, -0.557257, -0.706452,
	0.758215, 0.567570, 0.320896,
	-0.554803, -0.724240, -0.409475,
	0.574247, 0.324671, 0.751552,
	-0.724240, -0.409475, -0.554803,
	0.648509, 0.500521, 0.573511,
	-0.613180, -0.574417, -0.542268,
	-0.856157, -0.325761, 0.401092,
	-0.140966, 0.624151, -0.768482,
	0.230295, 0.928115, -0.292517,
	0.725869, -0.686003, -0.050147,
	-0.386554, 0.145205, 0.910764,
	0.023387, 0.811959, -0.583245,
	0.145078, 0.909010, -0.390709,
	0.811945, -0.583254, 0.023688,
	0.918873, -0.358164, 0.165502,
	-0.391554, 0.152622, 0.907410,
	-0.430420, 0.129589, 0.893278,
	0.172250, 0.917898, -0.357481,
	0.129582, 0.893155, -0.430678,
	0.916599, -0.360126, 0.173652,
	0.904700, -0.398243, 0.151399,
	-0.181274, 0.320762, 0.929651,
	0.090928, 0.433535, 0.896537,
	0.238934, 0.929568, -0.280739,
	-0.852905, 0.518044, -0.064680,
	0.849744, 0.525002, 0.048029,
	0.772175, 0.185171, -0.607830,
	0.392856, -0.289566, 0.872820,
	0.238554, -0.543258, 0.804962,
	-0.254047, 0.878665, 0.404237,
	-0.344814, 0.867742, 0.357948,
	0.826503, 0.267415, -0.495361,
	0.880046, 0.415749, -0.229503,
	0.041430, -0.769931, 0.636781,
	0.452083, -0.162149, 0.877114,
	-0.767305, 0.639788, 0.043762,
	0.192031, 0.807170, 0.558213,
	-0.104546, -0.439437, -0.892169,
	0.563204, 0.191247, 0.803882,
	-0.893339, -0.104009, -0.437181,
	0.520015, 0.620134, 0.587383,
	-0.532783, -0.568442, -0.626910,
	0.655155, 0.582669, 0.480905,
	-0.566736, -0.652957, -0.502451,
	0.587187, 0.493496, 0.641618,
	-0.646335, -0.510179, -0.567423,
	0.729593, 0.415313, 0.543332,
	-0.672608, -0.579113, -0.460681,
	-0.850147, -0.377332, 0.367248,
	0.013170, 0.831461, -0.555427,
	0.076461, 0.879107, -0.470452,
	0.826340, -0.563133, 0.006532,
	-0.552768, 0.006131, 0.833313,
	0.063711, 0.880352, -0.470022,
	-0.005033, 0.821171, -0.570661,
	0.881137, -0.468277, 0.065680,
	0.820469, -0.571661, -0.005830,
	-0.272387, 0.189785, 0.943285,
	-0.625258, -0.040870, 0.779347,
	0.201215, 0.946896, -0.250802,
	-0.049771, 0.770919, -0.634985,
	0.946477, -0.249140, 0.205210,
	0.761664, -0.645280, -0.059010,
	-0.087396, 0.354637, 0.930911,
	0.113379, 0.438881, 0.891364,
	0.289184, 0.938609, -0.188111,
	-0.862044, 0.502265, -0.067903,
	0.850404, 0.524881, 0.036225,
	0.750320, 0.169050, -0.639095,
	0.405416, -0.273791, 0.872168,
	0.272363, -0.504656, 0.819232,
	-0.199511, 0.878776, 0.433528,
	-0.251601, 0.876730, 0.409928,
	0.850408, 0.324325, -0.414270,
	0.878619, 0.447437, -0.166819,
	0.088382, -0.733201, 0.674244,
	0.505851, -0.024471, 0.862273,
	-0.718558, 0.688027, 0.101453,
	0.359121, 0.729111, 0.582605,
	-0.241486, -0.490284, -0.837440,
	0.585528, 0.358192, 0.727225,
	-0.838976, -0.240445, -0.488166,
	0.643093, 0.512205, 0.569278,
	-0.635599, -0.579531, -0.510057,
	0.553851, 0.569368, 0.607511,
	-0.581453, -0.544617, -0.604404,
	0.570558, 0.622433, 0.535762,
	-0.528631, -0.618538, -0.581343,
	0.798222, 0.322635, 0.508672,
	-0.734486, -0.576131, -0.358613,
	-0.854317, -0.387914, 0.345927,
	0.095664, 0.890756, -0.444300,
	-0.044634, 0.775419, -0.629867,
	0.870146, -0.489053, 0.060612,
	-0.688409, -0.096321, 0.718898,
	0.106411, 0.906910, -0.407666,
	-0.122145, 0.685394, -0.717855,
	0.908821, -0.402073, 0.111272,
	0.661527, -0.736489, -0.141300,
	-0.206738, 0.230165, 0.950938,
	-0.766398, -0.166102, 0.620520,
	0.237264, 0.952262, -0.192102,
	-0.185945, 0.594614, -0.782214,
	0.951842, -0.185539, 0.244076,
	0.548337, -0.807314, -0.218108,
	-0.025533, 0.395510, 0.918107,
	0.163320, 0.469737, 0.867568,
	0.367323, 0.926851, -0.077594,
	-0.845347, 0.529913, -0.067677,
	0.859809, 0.510053, 0.023967,
	0.746150, 0.138024, -0.651313,
	0.401491, -0.247258, 0.881855,
	0.293059, -0.442446, 0.847560,
	-0.137781, 0.885110, 0.444519,
	-0.139499, 0.885198, 0.443806,
	0.878971, 0.366556, -0.305034,
	0.881862, 0.461985, -0.094281,
	0.128038, -0.666179, 0.734719,
	0.536799, 0.122574, 0.834759,
	-0.621872, 0.765899, 0.163322,
	0.508597, 0.633224, 0.583401,
	-0.446709, -0.556172, -0.700802,
	0.584681, 0.508023, 0.632504,
	-0.702771, -0.445497, -0.554658,
	0.743061, 0.392915, 0.541736,
	-0.739763, -0.570448, -0.356846,
	0.437421, 0.545016, 0.715276,
	-0.578805, -0.396991, -0.712308,
	0.543464, 0.728093, 0.417764,
	-0.370635, -0.729467, -0.574898,
	0.854760, 0.212574, 0.473495,
	-0.795562, -0.555507, -0.241853,
	-0.866357, -0.373283, 0.331792,
	0.160574, 0.907985, -0.387013,
	-0.133640, 0.632917, -0.762598,
	0.890079, -0.438961, 0.122769,
	-0.796289, -0.167767, 0.581187,
	0.159875, 0.916131, -0.367618,
	-0.207688, 0.519717, -0.828709,
	0.919288, -0.355407, 0.169104,
	0.471887, -0.848906, -0.238077,
	-0.157198, 0.282792, 0.946212,
	-0.860761, -0.251493, 0.442539,
	0.286907, 0.946534, -0.147506,
	-0.280178, 0.396016, -0.874455,
	0.945345, -0.134518, 0.297032,
	0.318879, -0.891308, -0.322315,
	0.033187, 0.446108, 0.894364,
	0.281577, 0.526045, 0.802490,
	0.493216, 0.856965, 0.149492,
	-0.806116, 0.588090, -0.065788,
	0.875711, 0.482693, 0.011748,
	0.755227, 0.095676, -0.648443,
	0.387376, -0.208809, 0.897964,
	0.305375, -0.359245, 0.881867,
	-0.071555, 0.893468, 0.443389,
	-0.012521, 0.884323, 0.466709,
	0.900526, 0.400945, -0.168217,
	0.883639, 0.468085, -0.008872,
	0.177282, -0.549820, 0.816253,
	0.554542, 0.273720, 0.785850,
	-0.428523, 0.865885, 0.258091,
	0.638823, 0.513097, 0.573268,
	-0.709106, -0.569551, -0.415669,
	0.573132, 0.638898, 0.513156,
	-0.416947, -0.708651, -0.569182,
	0.820220, 0.254550, 0.512292,
	-0.831734, -0.524547, -0.181851,
	0.296821, 0.516773, 0.803021,
	-0.539373, -0.220221, -0.812760,
	0.513975, 0.810823, 0.279993,
	-0.187258, -0.827475, -0.529358,
	0.893978, 0.081466, 0.440644,
	-0.851519, -0.510374, -0.120139,
	-0.883949, -0.340117, 0.320864,
	0.222940, 0.906851, -0.357659,
	-0.197505, 0.462243, -0.864478,
	0.897561, -0.393516, 0.198820,
	-0.878790, -0.218127, 0.424440,
	0.228247, 0.915555, -0.331152,
	-0.269349, 0.335438, -0.902736,
	0.919624, -0.308295, 0.243404,
	0.269291, -0.913353, -0.305399,
	-0.103145, 0.349836, 0.931115,
	-0.915777, -0.310357, 0.255013,
	0.351602, 0.931015, -0.097909,
	-0.344501, 0.191401, -0.919067,
	0.927786, -0.076071, 0.365276,
	0.096318, -0.916204, -0.388964,
	0.103379, 0.503896, 0.857556,
	0.645878, 0.572003, 0.505622,
	0.584827, 0.523777, 0.619383,
	-0.741971, 0.667444, -0.063220,
	0.896669, 0.442702, -0.000062,
	0.775603, 0.045528, -0.629577,
	0.366529, -0.157874, 0.916915,
	0.312249, -0.257005, 0.914576,
	-0.003063, 0.901093, 0.433615,
	0.123216, 0.866668, 0.483430,
	0.901990, 0.431690, -0.007547,
	0.876004, 0.472925, 0.094658,
	0.255939, -0.346971, 0.902278,
	0.565888, 0.420116, 0.709417,
	-0.034129, 0.908245, 0.417045,
	0.743056, 0.365990, 0.560285,
	-0.896820, -0.441727, -0.024308,
	0.558968, 0.743853, 0.366384,
	-0.023015, -0.896847, -0.441742,
	0.868988, 0.094580, 0.485710,
	-0.899717, -0.436361, -0.009897,
	0.128351, 0.489198, 0.862677,
	-0.453282, -0.037744, -0.890568,
	0.487273, 0.864988, 0.119840,
	-0.006312, -0.898634, -0.438654,
	0.908122, -0.071265, 0.412597,
	-0.899103, -0.437718, -0.004083,
	-0.905123, -0.290992, 0.309961,
	0.287706, 0.893752, -0.344141,
	-0.244567, 0.271158, -0.930946,
	0.895467, -0.336019, 0.291941,
	-0.933727, -0.257275, 0.248925,
	0.313951, 0.906489, -0.282332,
	-0.315900, 0.140741, -0.938296,
	0.909941, -0.243408, 0.335798,
	0.063841, -0.932816, -0.354652,
	-0.029380, 0.429290, 0.902689,
	-0.932771, -0.355193, 0.061454,
	0.429511, 0.902618, -0.028298,
	-0.391409, -0.010499, -0.920157,
	0.895153, 0.005086, 0.445730,
	-0.111716, -0.893303, -0.435351,
	0.193929, 0.561956, 0.804112,
	0.771730, -0.049752, -0.634001,
	0.394012, -0.001893, 0.919103,
	-0.648739, 0.758603, -0.060490,
	0.921499, 0.388219, -0.011197,
	0.807331, -0.007919, -0.590046,
	0.340929, -0.094831, 0.935294,
	0.315513, -0.139552, 0.938604,
	0.065379, 0.906533, 0.417042,
	0.258154, 0.827999, 0.497769,
	0.871931, 0.461095, 0.164708,
	0.845852, 0.484554, 0.223028,
	0.372708, -0.027023, 0.927555,
	0.576903, 0.549861, 0.604016,
	0.487372, 0.670558, 0.559304,
	0.812067, 0.193194, 0.550657,
	-0.932967, -0.221963, 0.283381,
	0.548522, 0.813434, 0.193519,
	0.286401, -0.932095, -0.221751,
	0.880947, -0.085557, 0.465417,
	-0.940380, -0.311021, 0.137667,
	-0.066079, 0.465847, 0.882394,
	-0.323375, 0.125550, -0.937905,
	0.467170, 0.882037, -0.061341,
	0.147861, -0.939393, -0.309319,
	0.887612, -0.243051, 0.391243,
	-0.936384, -0.336923, 0.098324,
	-0.927943, -0.226461, 0.296036,
	0.356977, 0.870095, -0.339855,
	-0.282694, 0.068955, -0.956728,
	0.881400, -0.249077, 0.401367,
	-0.954052, -0.294174, 0.056984,
	0.416424, 0.886116, -0.203442,
	-0.355524, -0.054825, -0.933058,
	0.885184, -0.142834, 0.442773,
	-0.134926, -0.908259, -0.396055,
	0.076179, 0.512803, 0.855120,
	-0.908561, -0.396215, -0.132407,
	0.512887, 0.855196, 0.074744,
	-0.431372, -0.200990, -0.879500,
	0.840607, 0.120670, 0.528033,
	-0.298917, -0.827900, -0.474585,
	0.308115, 0.608165, 0.731574,
	0.409229, -0.404517, -0.817862,
	0.200568, -0.198257, 0.959409,
	-0.523589, 0.850006, -0.057823,
	0.948685, 0.315493, -0.021462,
	0.851477, -0.057922, -0.521183,
	0.311690, -0.021204, 0.949947,
	0.316197, -0.013885, 0.948592,
	0.131410, 0.909571, 0.394224,
	0.380222, 0.769559, 0.513040,
	0.807856, 0.490392, 0.326931,
	0.769559, 0.513040, 0.380222,
	0.490139, 0.326759, 0.808079,
	0.593542, 0.650274, 0.474186,
	0.740942, 0.317172, 0.591952,
	0.835773, 0.000000, 0.549075,
	-0.891573, 0.000000, 0.452877,
	0.546646, 0.837364, 0.000000,
	0.456199, -0.889878, 0.000000,
	0.846201, -0.280016, 0.453359,
	-0.955193, -0.155537, 0.251821,
	-0.277454, 0.449211, 0.849252,
	-0.157860, 0.255585, -0.953812,
	0.455964, 0.851844, -0.257795,
	0.265390, -0.952394, -0.150047,
	0.821902, -0.426706, 0.377359,
	-0.961006, -0.207144, 0.183189,
	-0.950970, -0.149481, 0.270761,
	0.434843, 0.836372, -0.333758,
	-0.313201, -0.123445, -0.941630,
	0.848198, -0.117180, 0.516556,
	-0.933129, -0.332375, -0.137107,
	0.524898, 0.847417, -0.079792,
	-0.393944, -0.234531, -0.888709,
	0.836629, 0.003469, 0.547759,
	-0.311540, -0.843779, -0.437012,
	0.214916, 0.582082, 0.784214,
	-0.838575, -0.443143, -0.316884,
	0.583858, 0.781302, 0.220630,
	-0.476129, -0.372178, -0.796734,
	0.757360, 0.276368, 0.591630,
	-0.462014, -0.718180, -0.520346,
	0.447854, 0.626850, 0.637563,
	0.382459, -0.496435, -0.779280,
	0.005915, -0.302771, 0.953045,
	-0.295596, 0.955237, 0.012026,
	0.966993, 0.050525, -0.249743,
	0.937250, -0.032703, -0.347121,
	0.169886, -0.094333, 0.980938,
	0.301211, 0.084524, 0.949804,
	-0.007321, 0.972871, 0.231231,
	0.352940, 0.810905, 0.466761,
	0.975087, 0.220744, -0.021827,
	0.733208, 0.515782, 0.443142,
	0.482629, 0.383885, 0.787211,
	0.607256, 0.654784, 0.449997,
	0.767591, 0.174173, 0.616820,
	0.805491, -0.059171, 0.589646,
	-0.920369, 0.067607, 0.385164,
	0.570288, 0.809899, -0.137239,
	0.477961, -0.866036, 0.146748,
	0.773316, -0.427786, 0.467954,
	-0.958134, -0.018819, 0.285700,
	-0.460973, 0.448530, 0.765718,
	0.010699, 0.326728, -0.945058,
	0.461084, 0.775479, -0.431317,
	0.326160, -0.945260, 0.010165,
	0.715360, -0.588446, 0.376816,
	-0.970248, -0.065753, 0.233015,
	-0.964565, -0.045224, 0.259940,
	0.500368, 0.786807, -0.361341,
	-0.354361, -0.290882, -0.888716,
	0.795146, 0.033875, 0.605471,
	-0.864413, -0.390749, -0.316396,
	0.610914, 0.787111, 0.085091,
	-0.448134, -0.391735, -0.803567,
	0.761836, 0.189155, 0.619537,
	-0.466884, -0.731012, -0.497635,
	0.390752, 0.611810, 0.687751,
	-0.710501, -0.512600, -0.482109,
	0.610654, 0.674840, 0.414358,
	-0.539108, -0.518550, -0.663678,
	0.641487, 0.472311, 0.604497,
	-0.597193, -0.548644, -0.585108,
	0.612848, 0.591016, 0.524517,
	0.334295, -0.586126, -0.738040,
	-0.067087, -0.238988, 0.968702,
	0.192444, -0.314696, -0.929479,
	0.804843, -0.316811, -0.501855,
	-0.314052, -0.928991, 0.195825,
	-0.311639, -0.498233, 0.809102,
	-0.930943, 0.191473, -0.310941,
	-0.498663, 0.809801, -0.309123,
	0.199066, -0.323272, -0.925131,
	0.806684, -0.315547, -0.499691,
	-0.322844, -0.925083, 0.199982,
	-0.313329, -0.499811, 0.807474,
	-0.926411, 0.198164, -0.320146,
	-0.500227, 0.808145, -0.310925,
	0.201243, -0.325119, -0.924012,
	0.808660, -0.313106, -0.498029,
	-0.200267, 0.321374, 0.925533,
	-0.753358, -0.050757, -0.655649,
	0.570025, 0.581125, 0.580831,
	-0.573190, -0.586189, -0.572569,
	0.583789, 0.582121, 0.565973,
	-0.589356, -0.573081, -0.569419,
	0.583340, 0.563187, 0.585265,
	-0.573468, -0.566931, -0.591374,
	0.563095, 0.583635, 0.585059,
	-0.568167, -0.591944, -0.571654,
	0.585502, 0.585334, 0.560867,
	-0.594009, -0.571634, -0.566028,
	0.585426, 0.557501, 0.588617,
	-0.571572, -0.562663, -0.597257,
	-0.169060, -0.870697, -0.461851,
	0.375388, -0.602395, -0.704418,
	0.569516, 0.612965, 0.547656,
	-0.593984, -0.571895, -0.565790,
	0.585601, 0.561086, 0.585025,
	-0.574024, -0.566027, -0.591701,
	0.565808, 0.580684, 0.585378,
	-0.569405, -0.585696, -0.576835,
	0.579016, 0.581588, 0.571398,
	-0.581865, -0.576689, -0.573466,
	0.581803, 0.576609, 0.573609,
	-0.580638, -0.577110, -0.574285,
	0.582390, 0.574021, 0.575605,
	-0.578588, -0.568784, -0.584568,
	-0.393456, 0.286040, -0.873713,
	-0.262139, -0.313276, 0.912766,
	0.558088, 0.617130, 0.554697,
	-0.569022, -0.612394, -0.548805,
	0.591857, 0.570343, 0.569574,
	-0.584757, -0.562779, -0.584242,
	0.571365, 0.569162, 0.591267,
	-0.565330, -0.580906, -0.585619,
	0.572500, 0.588958, 0.570414,
	-0.581699, -0.584516, -0.565657,
	0.587768, 0.573081, 0.571058,
	-0.585096, -0.570257, -0.576602,
	0.577047, 0.574120, 0.580864,
	-0.575350, -0.577408, -0.579287,
	-0.105999, 0.334320, 0.936480,
	-0.716168, -0.096539, -0.691219,
	0.577837, 0.578353, 0.575858,
	-0.578907, -0.579498, -0.573628,
	0.581505, 0.578254, 0.572253,
	-0.583474, -0.574462, -0.574066,
	0.579718, 0.570261, 0.582005,
	-0.574600, -0.572707, -0.584673,
	0.569400, 0.582550, 0.580016,
	-0.573535, -0.587039, -0.571352,
	0.584833, 0.581641, 0.565389,
	-0.589833, -0.572066, -0.569946,
	0.581955, 0.563292, 0.586541,
	-0.571646, -0.568179, -0.591940,
	-0.236938, -0.837806, -0.491875,
	0.224694, -0.681891, -0.696087,
	0.587108, 0.584349, 0.560214,
	-0.594539, -0.570213, -0.566904,
	0.583355, 0.557826, 0.590362,
	-0.569073, -0.564529, -0.597882,
	0.546816, 0.606109, 0.577603,
	-0.555362, -0.614846, -0.559944,
	0.564853, 0.610413, 0.555281,
	-0.581887, -0.576649, -0.573484,
	0.578796, 0.573931, 0.579309,
	-0.578181, -0.574251, -0.579606,
	0.577822, 0.575021, 0.579200,
	-0.576550, -0.573805, -0.581668,
	-0.482858, 0.091826, -0.870871,
	-0.154283, -0.103604, 0.982580,
	0.576536, 0.588476, 0.566835,
	-0.586245, -0.583383, -0.562123,
	0.594013, 0.566939, 0.570727,
	-0.583670, -0.557195, -0.590646,
	0.564053, 0.567358, 0.599958,
	-0.544382, -0.607192, -0.578763,
	0.560263, 0.620258, 0.548986,
	-0.570520, -0.614963, -0.544360,
	0.590965, 0.569822, 0.571020,
	-0.582937, -0.563938, -0.584942,
	0.570470, 0.571031, 0.590329,
	-0.565956, -0.581744, -0.584182,
	-0.030217, 0.368064, 0.929309,
	-0.682260, -0.158793, -0.713656,
	0.587140, 0.572838, 0.571947,
	-0.584246, -0.570742, -0.576984,
	0.576810, 0.574995, 0.580234,
	-0.575712, -0.577584, -0.578751,
	0.576864, 0.578474, 0.576711,
	-0.577915, -0.577871, -0.576263,
	0.577462, 0.578907, 0.575677,
	-0.578903, -0.579963, -0.573162,
	0.581853, 0.578278, 0.571876,
	-0.583636, -0.574098, -0.574266,
	0.578946, 0.570495, 0.582543,
	-0.573704, -0.573490, -0.584785,
	-0.299150, -0.809934, -0.504496,
	0.069454, -0.746090, -0.662213,
	0.585702, 0.580939, 0.565210,
	-0.589852, -0.571253, -0.570740,
	0.580306, 0.563988, 0.587505,
	-0.570024, -0.569836, -0.591912,
	0.562375, 0.587577, 0.581797,
	-0.570226, -0.593462, -0.568018,
	0.588439, 0.583041, 0.560180,
	-0.594468, -0.569026, -0.568170,
	0.580985, 0.558893, 0.591688,
	-0.566841, -0.566896, -0.597763,
	0.552828, 0.599335, 0.578946,
	-0.562895, -0.606595, -0.561419,
	-0.531175, -0.092263, -0.842223,
	-0.064474, 0.108669, 0.991985,
	0.574345, 0.586271, 0.571327,
	-0.584507, -0.580044, -0.567363,
	0.589170, 0.567486, 0.575186,
	-0.581050, -0.562177, -0.588505,
	0.565051, 0.572099, 0.594492,
	-0.558026, -0.589960, -0.583571,
	0.573744, 0.599257, 0.558309,
	-0.593684, -0.586927, -0.550504,
	0.604452, 0.558369, 0.568210,
	-0.585906, -0.546463, -0.598409,
	0.554712, 0.565645, 0.610197,
	-0.538473, -0.607001, -0.584463,
	0.051978, 0.419920, 0.906071,
	-0.646877, -0.240496, -0.723680,
	0.589691, 0.569739, 0.572418,
	-0.581178, -0.565557, -0.585130,
	0.570042, 0.572919, 0.588911,
	-0.566944, -0.582358, -0.582609,
	0.575435, 0.586795, 0.569690,
	-0.583471, -0.581410, -0.567032,
	0.586475, 0.572716, 0.572751,
	-0.583519, -0.571290, -0.577177,
	0.576694, 0.575822, 0.579528,
	-0.576046, -0.577779, -0.578224,
	0.577031, 0.578293, 0.576725,
	-0.577922, -0.577699, -0.576429,
	-0.379637, -0.770991, -0.511321,
	-0.095143, -0.768638, -0.632569,
	0.582057, 0.578204, 0.571742,
	-0.583480, -0.573934, -0.574588,
	0.578063, 0.571135, 0.582793,
	-0.573049, -0.574471, -0.584464,
	0.569573, 0.584665, 0.577714,
	-0.575208, -0.587440, -0.569254,
	0.586237, 0.580106, 0.565511,
	-0.589356, -0.570765, -0.571740,
	0.578556, 0.565273, 0.587996,
	-0.568787, -0.571759, -0.591247,
	0.563031, 0.588906, 0.579815,
	-0.571817, -0.593262, -0.566626,
	-0.534408, -0.171576, -0.827629,
	0.007687, 0.189987, 0.981756,
	0.578537, 0.560654, 0.592421,
	-0.565086, -0.569568, -0.596884,
	0.557774, 0.591813, 0.581933,
	-0.569039, -0.597443, -0.565027,
	0.553289, 0.607572, 0.569849,
	-0.558275, -0.591955, -0.581308,
	0.572397, 0.597297, 0.561781,
	-0.592461, -0.583134, -0.555828,
	0.599201, 0.558582, 0.573536,
	-0.582797, -0.551501, -0.596820,
	0.555907, 0.570833, 0.604250,
	-0.547567, -0.598745, -0.584530,
	0.161427, 0.484774, 0.859614,
	-0.645823, -0.321961, -0.692282,
	0.613270, 0.548844, 0.568041,
	-0.585415, -0.537103, -0.607297,
	0.544544, 0.566270, 0.618717,
	-0.532474, -0.607650, -0.589265,
	0.568049, 0.621392, 0.539622,
	-0.576059, -0.615880, -0.537446,
	0.588436, 0.569817, 0.573629,
	-0.579702, -0.567145, -0.585057,
	0.569850, 0.574581, 0.587476,
	-0.567931, -0.582799, -0.581205,
	0.576646, 0.585686, 0.569607,
	-0.584153, -0.579936, -0.567838,
	-0.472186, -0.714097, -0.516823,
	-0.259513, -0.748637, -0.610078,
	0.576496, 0.576514, 0.579037,
	-0.576125, -0.578079, -0.577845,
	0.577094, 0.578400, 0.576554,
	-0.578114, -0.577623, -0.576312,
	0.577684, 0.579343, 0.575015,
	-0.579523, -0.579899, -0.572600,
	0.581937, 0.578071, 0.571999,
	-0.582888, -0.574080, -0.575042,
	0.577172, 0.572215, 0.582617,
	-0.572789, -0.575549, -0.583657,
	0.570462, 0.585012, 0.576484,
	-0.576358, -0.586804, -0.568746,
	-0.559742, -0.356513, -0.748056,
	0.220214, 0.405463, 0.887190,
	0.576878, 0.567097, 0.587888,
	-0.568095, -0.573775, -0.589958,
	0.564314, 0.589595, 0.577865,
	-0.573556, -0.592405, -0.565764,
	0.589589, 0.580156, 0.561964,
	-0.592464, -0.567967, -0.571314,
	0.576225, 0.562999, 0.592450,
	-0.563963, -0.572328, -0.595304,
	0.561301, 0.584178, 0.586240,
	-0.573337, -0.588136, -0.570422,
	0.546899, 0.607596, 0.575959,
	-0.548327, -0.601215, -0.581273,
	0.318635, 0.548679, 0.772932,
	-0.618300, -0.430504, -0.657550,
	0.607356, 0.548944, 0.574265,
	-0.581955, -0.542332, -0.605974,
	0.546185, 0.571698, 0.612245,
	-0.538794, -0.608106, -0.583016,
	0.575186, 0.615577, 0.538726,
	-0.611685, -0.586095, -0.531351,
	0.620154, 0.538927, 0.570058,
	-0.582522, -0.529447, -0.616729,
	0.534127, 0.568904, 0.625345,
	-0.526426, -0.609354, -0.592928,
	0.572617, 0.620266, 0.536078,
	-0.579809, -0.614666, -0.534796,
	-0.566753, -0.634780, -0.525209,
	-0.413969, -0.687591, -0.596530,
	0.569504, 0.575951, 0.586469,
	-0.568508, -0.583279, -0.580159,
	0.577630, 0.584911, 0.569405,
	-0.584970, -0.578568, -0.568392,
	0.586014, 0.571836, 0.574100,
	-0.582685, -0.571334, -0.577975,
	0.575917, 0.577104, 0.579026,
	-0.575698, -0.578664, -0.577685,
	0.577120, 0.578920, 0.576008,
	-0.578681, -0.577578, -0.575789,
	0.578491, 0.578809, 0.574742,
	-0.580025, -0.579046, -0.572954,
	-0.579340, -0.517669, -0.629590,
	0.480833, 0.552104, 0.681161,
	0.576394, 0.573749, 0.581877,
	-0.573076, -0.576596, -0.582340,
	0.571874, 0.584676, 0.575426,
	-0.577528, -0.585552, -0.568851,
	0.585655, 0.578598, 0.567656,
	-0.586677, -0.571262, -0.573994,
	0.575433, 0.569384, 0.587093,
	-0.568063, -0.575712, -0.588098,
	0.566126, 0.589539, 0.576147,
	-0.575257, -0.590936, -0.565574,
	0.589144, 0.578961, 0.563660,
	-0.590584, -0.568333, -0.572895,
	0.409964, 0.575872, 0.707320,
	-0.569188, -0.506962, -0.647313,
	0.563367, 0.576629, 0.591707,
	-0.575674, -0.578994, -0.577378,
	0.542517, 0.606711, 0.581014,
	-0.542218, -0.609054, -0.578837,
	0.575222, 0.611640, 0.543154,
	-0.608785, -0.581629, -0.539526,
	0.611732, 0.541315, 0.576855,
	-0.578721, -0.537123, -0.613662,
	0.538552, 0.574255, 0.616597,
	-0.534064, -0.615769, -0.579313,
	0.578665, 0.618979, 0.531048,
	-0.619329, -0.582297, -0.526651,
	-0.648052, -0.536434, -0.540617,
	-0.455188, -0.630439, -0.628770,
	0.526182, 0.573010, 0.628325,
	-0.522813, -0.610093, -0.595360,
	0.577172, 0.616433, 0.535613,
	-0.583798, -0.610654, -0.535053,
	0.586945, 0.569338, 0.575630,
	-0.577659, -0.568931, -0.585344,
	0.569009, 0.577100, 0.585820,
	-0.568729, -0.583862, -0.579355,
	0.578446, 0.584457, 0.569044,
	-0.585907, -0.577295, -0.568721,
	0.586307, 0.570816, 0.574817,
	-0.582425, -0.570672, -0.578891,
	-0.600897, -0.640942, -0.477614,
	0.700139, 0.576763, 0.420893,
	0.577222, 0.579729, 0.575091,
	-0.579611, -0.577451, -0.574980,
	0.579593, 0.577736, 0.574710,
	-0.580436, -0.577771, -0.573825,
	0.580079, 0.578108, 0.573847,
	-0.580188, -0.575784, -0.576068,
	0.575890, 0.575528, 0.580618,
	-0.573902, -0.577421, -0.580708,
	0.573558, 0.583690, 0.574750,
	-0.578484, -0.583888, -0.569589,
	0.584406, 0.578271, 0.569273,
	-0.584665, -0.572388, -0.574925,
	0.670928, 0.548348, 0.499169,
	-0.566442, -0.614896, -0.548677,
	0.568179, 0.588774, 0.574907,
	-0.576653, -0.589087, -0.566081,
	0.588018, 0.578208, 0.565606,
	-0.588354, -0.569331, -0.574197,
	0.572746, 0.568717, 0.590358,
	-0.563874, -0.577218, -0.590649,
	0.563998, 0.569750, 0.597739,
	-0.576069, -0.570786, -0.585105,
	0.540430, 0.604234, 0.585522,
	-0.540051, -0.614222, -0.575392,
	0.578195, 0.613340, 0.538057,
	-0.613612, -0.578625, -0.537283,
	-0.703278, -0.430024, -0.566108,
	-0.607450, -0.499456, -0.617696,
	0.534365, 0.577219, 0.617473,
	-0.533476, -0.620366, -0.574933,
	0.582482, 0.618877, 0.526978,
	-0.623946, -0.577867, -0.526081,
	0.622990, 0.527566, 0.577545,
	-0.571981, -0.526480, -0.629012,
	0.522069, 0.577243, 0.627881,
	-0.522092, -0.608804, -0.597309,
	0.580743, 0.610229, 0.538849,
	-0.586782, -0.604409, -0.538867,
	0.586386, 0.569114, 0.576420,
	-0.576960, -0.569500, -0.585480,
	-0.632673, -0.714295, -0.299178,
	0.834233, 0.513912, 0.199874,
	0.579135, 0.584169, 0.568638,
	-0.586814, -0.576139, -0.568958,
	0.586674, 0.569670, 0.575579,
	-0.582179, -0.569882, -0.579915,
	0.573902, 0.578380, 0.579752,
	-0.573989, -0.580704, -0.577339,
	0.577446, 0.580625, 0.573961,
	-0.580747, -0.577199, -0.574085,
	0.580728, 0.576350, 0.574956,
	-0.580660, -0.576353, -0.575023,
	0.578579, 0.578487, 0.574977,
	-0.578533, -0.577240, -0.576276,
	0.870515, 0.410047, 0.272148,
	-0.570010, -0.691548, -0.443678,
	0.575228, 0.582264, 0.574527,
	-0.579100, -0.582091, -0.570800,
	0.582726, 0.578352, 0.570911,
	-0.582543, -0.573963, -0.575508,
	0.573840, 0.574263, 0.583892,
	-0.569786, -0.578474, -0.583705,
	0.570167, 0.587500, 0.574242,
	-0.577599, -0.587136, -0.567142,
	0.586394, 0.577967, 0.567535,
	-0.586036, -0.570816, -0.575092,
	0.571818, 0.571480, 0.588587,
	-0.564712, -0.578907, -0.588189,
	-0.708734, -0.371735, -0.599591,
	-0.735825, -0.346603, -0.581745,
	0.538570, 0.601442, 0.590093,
	-0.539220, -0.617976, -0.572143,
	0.580919, 0.613820, 0.534563,
	-0.617025, -0.575768, -0.536444,
	0.612419, 0.534605, 0.582357,
	-0.570671, -0.536520, -0.621676,
	0.531864, 0.579756, 0.617255,
	-0.534246, -0.623226, -0.571114,
	0.585806, 0.617675, 0.524699,
	-0.626835, -0.574016, -0.526861,
	0.621651, 0.525649, 0.580726,
	-0.567123, -0.528417, -0.631781,
	-0.668792, -0.732017, -0.129877,
	0.877171, 0.474424, 0.074121,
	0.582965, 0.603939, 0.543516,
	-0.588451, -0.598195, -0.543955,
	0.585920, 0.568929, 0.577076,
	-0.576426, -0.569911, -0.585606,
	0.568280, 0.578864, 0.584786,
	-0.569001, -0.585051, -0.577887,
	0.579740, 0.584082, 0.568111,
	-0.587779, -0.575080, -0.569033,
	0.587155, 0.568356, 0.576386,
	-0.581960, -0.568924, -0.581075,
	0.572673, 0.579109, 0.580240,
	-0.572992, -0.582036, -0.576988,
	-0.578322, -0.733952, 0.356171,
	0.655874, 0.735883, 0.168243,
	-0.083163, -0.973987, -0.210792,
	0.447081, 0.102908, 0.888554,
	-0.974223, -0.210203, -0.081880,
	0.102874, 0.890671, 0.442857,
	-0.212027, -0.079761, -0.974003,
	0.890905, 0.442688, 0.101566,
	-0.078001, -0.974685, -0.209534,
	0.441453, 0.094638, 0.892280,
	-0.974874, -0.208991, -0.077092,
	0.095393, 0.893648, 0.438513,
	-0.211949, -0.076181, -0.974307,
	0.893863, 0.438434, 0.093726,
	0.682114, 0.325698, 0.654707,
	-0.741648, -0.355317, -0.568953,
	0.584362, 0.578190, 0.569401,
	-0.583669, -0.572694, -0.575632,
	0.571315, 0.574054, 0.586568,
	-0.565905, -0.580114, -0.585849,
	0.562443, 0.557946, 0.610208,
	-0.572995, -0.557427, -0.600793,
	0.537077, 0.597854, 0.595079,
	-0.539345, -0.620302, -0.569501,
	0.583002, 0.613482, 0.532680,
	-0.618969, -0.573480, -0.536654,
	0.611501, 0.533759, 0.584096,
	-0.567485, -0.538018, -0.623296,
	0.212947, 0.659036, 0.721336,
	-0.229099, -0.621103, -0.749496,
	0.588323, 0.615763, 0.524129,
	-0.628048, -0.571111, -0.528571,
	0.619694, 0.525510, 0.582939,
	-0.563254, -0.531223, -0.632888,
	0.519353, 0.583238, 0.624585,
	-0.522237, -0.603361, -0.602681,
	0.583957, 0.597490, 0.549545,
	-0.588959, -0.591895, -0.550261,
	0.585593, 0.568719, 0.577616,
	-0.576031, -0.570119, -0.585792,
	0.567882, 0.579519, 0.584524,
	-0.568963, -0.585765, -0.577200,
	0.676216, 0.695136, 0.243963,
	-0.584784, -0.763831, -0.273112,
	0.587818, 0.566815, 0.577227,
	-0.581804, -0.567728, -0.582399,
	0.571233, 0.579887, 0.580882,
	-0.571842, -0.583596, -0.576553,
	0.578189, 0.582774, 0.571028,
	-0.583668, -0.576391, -0.571932,
	0.583122, 0.572695, 0.576185,
	-0.580645, -0.573061, -0.578319,
	0.574669, 0.579926, 0.577444,
	-0.574861, -0.581098, -0.576073,
	0.576021, 0.580944, 0.575069,
	-0.578386, -0.578195, -0.575465,
	0.683207, 0.246328, 0.687423,
	-0.768448, -0.282013, -0.574419,
	0.578232, 0.579585, 0.574221,
	-0.577997, -0.578218, -0.575833,
	0.573689, 0.578825, 0.579519,
	-0.572970, -0.579664, -0.579392,
	0.573709, 0.583876, 0.574409,
	-0.578249, -0.583172, -0.570560,
	0.581957, 0.578820, 0.571221,
	-0.581232, -0.574903, -0.575896,
	0.571102, 0.576464, 0.584407,
	-0.567318, -0.580922, -0.583677,
	0.561003, 0.552297, 0.616639,
	-0.570487, -0.551482, -0.608615,
	0.143421, 0.706110, 0.693425,
	-0.177741, -0.621699, -0.762823,
	0.584099, 0.612604, 0.532488,
	-0.619335, -0.572106, -0.537697,
	0.610114, 0.534647, 0.584734,
	-0.565281, -0.540342, -0.623288,
	0.531908, 0.582173, 0.614938,
	-0.538337, -0.623861, -0.566560,
	0.589781, 0.613386, 0.525277,
	-0.627554, -0.569442, -0.530954,
	0.617369, 0.527155, 0.583921,
	-0.560591, -0.534617, -0.632394,
	0.520576, 0.584406, 0.622471,
	-0.523157, -0.598249, -0.606964,
	0.711536, 0.680662, 0.174404,
	-0.583382, -0.784037, -0.212017,
	0.585464, 0.568417, 0.578043,
	-0.575769, -0.570060, -0.586107,
	0.567351, 0.580028, 0.584535,
	-0.568716, -0.586648, -0.576546,
	0.580762, 0.584745, 0.566382,
	-0.590263, -0.573238, -0.568321,
	0.588785, 0.564956, 0.578063,
	-0.581793, -0.566176, -0.583920,
	0.569508, 0.580676, 0.581787,
	-0.570435, -0.585426, -0.576091,
	0.578614, 0.584179, 0.569158,
	-0.585516, -0.575922, -0.570513,
	0.667092, 0.184593, 0.721744,
	-0.784269, -0.227453, -0.577223,
	0.572169, 0.580895, 0.578950,
	-0.572679, -0.583507, -0.575812,
	0.576352, 0.582937, 0.572715,
	-0.580424, -0.578080, -0.573525,
	0.580007, 0.575888, 0.576147,
	-0.579249, -0.576017, -0.576780,
	0.575374, 0.580623, 0.576039,
	-0.575422, -0.580856, -0.575757,
	0.573866, 0.581111, 0.577051,
	-0.574907, -0.579864, -0.577269,
	0.575276, 0.581616, 0.575135,
	-0.578055, -0.581123, -0.572843,
	0.163947, 0.743185, 0.648689,
	-0.210775, -0.583573, -0.784230,
	0.571030, 0.578774, 0.582190,
	-0.568846, -0.581420, -0.581692,
	0.559596, 0.546102, 0.623398,
	-0.567919, -0.545262, -0.616569,
	0.536239, 0.585273, 0.608196,
	-0.541599, -0.620247, -0.567418,
	0.583852, 0.611302, 0.534253,
	-0.617837, -0.571986, -0.539546,
	0.608321, 0.537536, 0.583953,
	-0.564332, -0.543440, -0.621452,
	0.534781, 0.581430, 0.613147,
	-0.541347, -0.621381, -0.566418,
	0.731397, 0.677414, 0.078540,
	-0.623006, -0.773092, -0.119133,
	0.614734, 0.530793, 0.583404,
	-0.559347, -0.538495, -0.630202,
	0.523684, 0.583952, 0.620286,
	-0.524982, -0.590396, -0.613048,
	0.582968, 0.582582, 0.566344,
	-0.587332, -0.577344, -0.567200,
	0.585632, 0.567950, 0.578332,
	-0.575679, -0.569623, -0.586621,
	0.566595, 0.580338, 0.584960,
	-0.568121, -0.587792, -0.575968,
	0.581111, 0.585760, 0.564974,
	-0.592012, -0.572564, -0.567180,
	0.639855, 0.138492, 0.755914,
	-0.796114, -0.190397, -0.574415,
	0.567388, 0.581377, 0.583156,
	-0.568577, -0.587584, -0.575730,
	0.578920, 0.586030, 0.566940,
	-0.587697, -0.575559, -0.568633,
	0.586289, 0.567563, 0.578047,
	-0.580545, -0.568516, -0.582887,
	0.569173, 0.581887, 0.580904,
	-0.570008, -0.586272, -0.575653,
	0.576586, 0.585265, 0.570099,
	-0.582728, -0.577969, -0.571297,
	0.581785, 0.572987, 0.577245,
	-0.579019, -0.573449, -0.579563,
	0.127952, 0.771515, 0.623211,
	-0.183101, -0.579487, -0.794147,
	0.573970, 0.583585, 0.574445,
	-0.577009, -0.579958, -0.575073,
	0.576809, 0.579017, 0.576220,
	-0.577632, -0.578874, -0.575540,
	0.575863, 0.581011, 0.575159,
	-0.575693, -0.580228, -0.576119,
	0.570898, 0.581083, 0.580015,
	-0.570389, -0.581698, -0.579900,
	0.558767, 0.538497, 0.630715,
	-0.565909, -0.537868, -0.624856,
	0.538382, 0.573380, 0.617559,
	-0.543847, -0.617049, -0.568754,
	0.779983, 0.623785, 0.050196,
	-0.611286, -0.782759, -0.116695,
	0.605823, 0.543059, 0.581433,
	-0.564919, -0.547585, -0.617266,
	0.540271, 0.578964, 0.610662,
	-0.545276, -0.616338, -0.568156,
	0.588616, 0.607126, 0.533787,
	-0.620150, -0.571070, -0.537859,
	0.611485, 0.536995, 0.581139,
	-0.559775, -0.543030, -0.625915,
	0.529229, 0.581508, 0.617872,
	-0.528616, -0.578046, -0.621633,
	0.581937, 0.572114, 0.577957,
	-0.586244, -0.566996, -0.578647,
	0.605102, 0.105883, 0.789076,
	-0.810128, -0.170292, -0.560975,
	0.565531, 0.580279, 0.586047,
	-0.566917, -0.589298, -0.575615,
	0.581107, 0.587623, 0.563039,
	-0.594254, -0.572366, -0.565030,
	0.592711, 0.559794, 0.579072,
	-0.583110, -0.561085, -0.587509,
	0.564761, 0.581720, 0.585360,
	-0.565922, -0.590100, -0.575772,
	0.578789, 0.588666, 0.564336,
	-0.590241, -0.575609, -0.565941,
	0.588809, 0.564170, 0.578806,
	-0.581049, -0.565146, -0.585656,
	0.104515, 0.803604, 0.585915,
	-0.175200, -0.564671, -0.806505,
	0.576363, 0.588239, 0.567257,
	-0.585271, -0.578144, -0.568514,
	0.584069, 0.569517, 0.578372,
	-0.579042, -0.570150, -0.582785,
	0.567908, 0.582809, 0.581218,
	-0.568532, -0.587130, -0.576238,
	0.573645, 0.586486, 0.571809,
	-0.579207, -0.580159, -0.572655,
	0.578520, 0.575959, 0.577568,
	-0.577187, -0.576137, -0.578723,
	0.571925, 0.582238, 0.577841,
	-0.572092, -0.583272, -0.576631,
	0.805134, 0.585993, 0.091494,
	-0.565053, -0.809432, -0.159797,
	0.560605, 0.531861, 0.634702,
	-0.566425, -0.531635, -0.629704,
	0.548214, 0.554612, 0.625992,
	-0.550952, -0.606878, -0.572845,
	0.578178, 0.602847, 0.549805,
	-0.602535, -0.577520, -0.550837,
	0.598476, 0.555419, 0.577353,
	-0.567860, -0.556786, -0.606238,
	0.552597, 0.574981, 0.603352,
	-0.554221, -0.604004, -0.572729,
	0.586133, 0.598938, 0.545639,
	-0.607935, -0.575740, -0.546753,
	0.560322, 0.097396, 0.822529,
	-0.781676, -0.167170, -0.600864,
	0.541462, 0.577213, 0.611264,
	-0.539530, -0.558913, -0.629701,
	0.582629, 0.557166, 0.591701,
	-0.587177, -0.552049, -0.592000,
	0.587611, 0.566363, 0.577881,
	-0.576783, -0.566788, -0.588279,
	0.564290, 0.579444, 0.588066,
	-0.564795, -0.591046, -0.575909,
	0.580197, 0.590702, 0.560752,
	-0.596715, -0.573339, -0.561440,
	0.596501, 0.556641, 0.578219,
	-0.585479, -0.557096, -0.588948,
	0.098424, 0.830319, 0.548528,
	-0.189600, -0.534727, -0.823480,
	0.577570, 0.592354, 0.561721,
	-0.592770, -0.576714, -0.562161,
	0.592495, 0.560439, 0.578669,
	-0.582731, -0.560720, -0.588233,
	0.561596, 0.582215, 0.587908,
	-0.561825, -0.592294, -0.577529,
	0.575014, 0.592021, 0.564686,
	-0.587653, -0.579136, -0.565036,
	0.587232, 0.565668, 0.578946,
	-0.579962, -0.565863, -0.586041,
	0.563448, 0.582855, 0.585496,
	-0.563655, -0.590149, -0.577942,
	0.842427, 0.532080, 0.084896,
	-0.534048, -0.826125, -0.179751,
	0.580693, 0.572523, 0.578803,
	-0.577227, -0.572639, -0.582146,
	0.567563, 0.582787, 0.581577,
	-0.567700, -0.586020, -0.578184,
	0.569061, 0.585948, 0.576918,
	-0.572903, -0.581992, -0.577120,
	0.566710, 0.528052, 0.632456,
	-0.570982, -0.528340, -0.628360,
	0.569280, 0.530309, 0.628246,
	-0.565568, -0.588261, -0.577998,
	0.575982, 0.588033, 0.567857,
	-0.582662, -0.581977, -0.567281,
	0.546307, 0.048539, 0.836177,
	-0.830279, -0.170437, -0.530649,
	0.571576, 0.572965, 0.587378,
	-0.570865, -0.583031, -0.578090,
	0.585810, 0.582576, 0.563411,
	-0.587163, -0.581312, -0.563309,
	0.586968, 0.569083, 0.575858,
	-0.567020, -0.567618, -0.596907,
	0.560493, 0.574224, 0.596753,
	-0.561178, -0.534529, -0.631948,
	0.588554, 0.537525, 0.603880,
	-0.593797, -0.532253, -0.603416,
	0.589671, 0.565728, 0.576403,
	-0.578852, -0.564215, -0.588721,
	0.108742, 0.849604, 0.516090,
	-0.224925, -0.484700, -0.845266,
	0.577497, 0.595028, 0.558963,
	-0.598286, -0.576575, -0.556431,
	0.601617, 0.554329, 0.575132,
	-0.589881, -0.552681, -0.588714,
	0.559784, 0.578388, 0.593388,
	-0.557641, -0.594013, -0.579815,
	0.574400, 0.596857, 0.560202,
	-0.594187, -0.579741, -0.557532,
	0.597392, 0.557434, 0.576533,
	-0.586406, -0.555681, -0.589360,
	0.558564, 0.579542, 0.593412,
	-0.556529, -0.593712, -0.581190,
	0.859665, 0.501100, 0.099376,
	-0.485776, -0.845482, -0.221770,
	0.591426, 0.562304, 0.577953,
	-0.582650, -0.560840, -0.588199,
	0.559849, 0.580613, 0.591149,
	-0.558296, -0.591521, -0.581729,
	0.569303, 0.593147, 0.569272,
	-0.582120, -0.582279, -0.567527,
	0.583571, 0.569220, 0.579167,
	-0.578555, -0.568396, -0.584978,
	0.563824, 0.581419, 0.586562,
	-0.563044, -0.587221, -0.581509,
	0.566658, 0.587676, 0.577525,
	-0.573322, -0.582012, -0.576684,
	0.484074, 0.035374, 0.874312,
	-0.835939, -0.173772, -0.520586,
	0.600565, 0.506947, 0.618326,
	-0.587795, -0.565573, -0.578468,
	0.581661, 0.564130, 0.586027,
	-0.559253, -0.580521, -0.591804,
	0.556966, 0.592859, 0.581642,
	-0.568853, -0.596848, -0.565843,
	0.590165, 0.579540, 0.561996,
	-0.595248, -0.558465, -0.577752,
	0.593425, 0.558037, 0.580036,
	-0.562891, -0.581303, -0.587572,
	0.561603, 0.587655, 0.582463,
	-0.566529, -0.589192, -0.576106,
	0.131441, 0.861066, 0.491211,
	-0.286124, -0.357966, -0.888816,
	0.602986, 0.515691, 0.608663,
	-0.609432, -0.510309, -0.606777,
	0.591930, 0.566219, 0.573598,
	-0.582456, -0.561621, -0.587645,
	0.564420, 0.573182, 0.594047,
	-0.558253, -0.592039, -0.581244,
	0.572374, 0.599601, 0.559344,
	-0.596921, -0.582825, -0.551363,
	0.606727, 0.554922, 0.569160,
	-0.596109, -0.549709, -0.585212,
	0.560301, 0.573112, 0.598001,
	-0.553654, -0.592659, -0.584998,
	0.866960, 0.481945, 0.126923,
	-0.419333, -0.863218, -0.281094,
	0.602388, 0.557111, 0.571626,
	-0.592140, -0.551755, -0.587312,
	0.558661, 0.573758, 0.598915,
	-0.552382, -0.591790, -0.587077,
	0.566620, 0.599162, 0.565638,
	-0.587740, -0.585504, -0.558343,
	0.595871, 0.561111, 0.574537,
	-0.587420, -0.556517, -0.587561,
	0.559449, 0.575088, 0.596900,
	-0.554409, -0.589397, -0.587573,
	0.565032, 0.594713, 0.571887,
	-0.581370, -0.584331, -0.566185,
	0.445825, 0.103787, 0.889083,
	-0.867482, -0.273681, -0.415420,
	0.562841, 0.577039, 0.591807,
	-0.559950, -0.585234, -0.586479,
	0.563998, 0.587169, 0.580637,
	-0.573147, -0.581429, -0.577445,
	0.586108, 0.535372, 0.608156,
	-0.587007, -0.535995, -0.606739,
	0.634582, 0.500704, 0.588728,
	-0.616402, -0.546590, -0.566823,
	0.600500, 0.534049, 0.595141,
	-0.541397, -0.565517, -0.622157,
	0.525663, 0.601248, 0.601813,
	-0.554182, -0.624649, -0.550177,
	0.123636, 0.856059, 0.501873,
	-0.326330, -0.291512, -0.899183,
	0.612642, 0.529222, 0.587021,
	-0.544395, -0.567663, -0.617570,
	0.531083, 0.596623, 0.601657,
	-0.554674, -0.614852, -0.560619,
	0.591629, 0.595283, 0.543703,
	-0.627136, -0.505784, -0.592354,
	0.622760, 0.501043, 0.600937,
	-0.630322, -0.496271, -0.597000,
	0.593024, 0.568500, 0.570202,
	-0.586553, -0.560674, -0.584466,
	0.567835, 0.568541, 0.595252,
	-0.556602, -0.588981, -0.585915,
	0.862978, 0.475693, 0.170248,
	-0.341679, -0.870574, -0.354058,
	0.608722, 0.559766, 0.562246,
	-0.601441, -0.550838, -0.578659,
	0.564628, 0.566922, 0.599828,
	-0.552921, -0.587916, -0.590452,
	0.563416, 0.601488, 0.566370,
	-0.587960, -0.590575, -0.552742,
	0.604585, 0.560886, 0.565582,
	-0.597508, -0.551773, -0.581834,
	0.563356, 0.566818, 0.601121,
	-0.552399, -0.586166, -0.592676,
	0.562056, 0.598412, 0.570960,
	-0.583957, -0.589066, -0.558566,
	-0.544883, -0.819985, -0.175291,
	-0.003660, -0.601188, 0.799099,
	0.005597, 0.919415, 0.393249,
	-0.996772, -0.080059, -0.005965,
	0.919414, 0.393252, 0.005502,
	-0.083626, -0.007617, -0.996468,
	0.386953, 0.007048, 0.922073,
	-0.007015, -0.996459, -0.083788,
	0.004288, 0.922923, 0.384961,
	-0.996478, -0.083571, -0.006880,
	0.922924, 0.384960, 0.004204,
	-0.086804, -0.007515, -0.996197,
	0.374397, 0.004926, 0.927255,
	-0.006884, -0.996088, -0.088096,
	0.186261, 0.851302, 0.490502,
	-0.426023, -0.181688, -0.886281,
	0.625762, 0.510580, 0.589687,
	-0.541222, -0.538699, -0.645663,
	0.502347, 0.594343, 0.628016,
	-0.529294, -0.645155, -0.551020,
	0.592185, 0.625992, 0.507396,
	-0.648477, -0.539090, -0.537457,
	0.635767, 0.508624, 0.580605,
	-0.544129, -0.541713, -0.640680,
	0.508637, 0.591187, 0.625928,
	-0.532806, -0.633856, -0.560664,
	0.056267, 0.647138, 0.760294,
	0.000000, 0.000000, 0.000000,
	0.705090, -0.145989, -0.693927,
	0.357584, 0.860839, 0.362064,
	0.357058, 0.800774, 0.480906,
	-0.402963, 0.915163, 0.009869,
	0.881419, 0.415704, 0.224254,
	0.960175, 0.127851, -0.248431,
	0.220630, -0.095590, 0.970662,
	-0.004630, -0.410953, 0.911645,
	-0.129917, 0.971998, 0.195808,
	-0.414004, 0.910249, -0.006958,
	0.967752, 0.139114, -0.210008,
	0.953990, 0.083065, -0.288101,
	-0.117698, -0.514052, 0.849646,
	0.000000, 0.000000, 0.000000,
	0.453441, 0.184786, 0.871921,
	0.395184, -0.196613, -0.897314,
	-0.503381, 0.859077, -0.092702,
	-0.428462, 0.903051, -0.030319,
	0.570312, -0.340317, -0.747615,
	0.941889, 0.042790, -0.333188,
	-0.414149, -0.799885, 0.434356,
	-0.035010, -0.431172, 0.901590,
	-0.782603, 0.482289, -0.393611,
	-0.458222, 0.886946, -0.057962,
	-0.026836, -0.563377, -0.825764,
	0.922136, 0.002771, -0.386857,
	-0.286164, -0.941878, -0.175998,
	0.000000, 0.000000, 0.000000,
	0.227081, 0.844544, 0.484954,
	0.448176, -0.277553, 0.849766,
	-0.825872, 0.017765, -0.563578,
	-0.518264, 0.848594, -0.106259,
	-0.379739, -0.591693, -0.711124,
	0.873231, -0.067448, -0.482617,
	-0.593570, -0.713440, -0.372395,
	-0.068957, -0.491275, 0.868270,
	-0.710270, -0.356172, -0.607172,
	-0.145658, -0.910721, -0.386486,
	-0.591928, -0.571702, -0.568135,
	-0.945929, -0.322072, -0.038574,
	-0.303512, -0.498190, -0.812211,
	0.000000, 0.000000, 0.000000,
	0.391355, -0.380086, -0.838079,
	0.717751, 0.690133, -0.092464,
	0.394496, -0.895699, -0.205174,
	0.820816, 0.015355, 0.570986,
	-0.882026, -0.171361, 0.438936,
	-0.085127, 0.527042, 0.845565,
	-0.074822, 0.558844, -0.825891,
	0.527736, 0.845527, -0.081104,
	0.560760, -0.824887, -0.071477,
	0.812433, 0.074044, 0.578333,
	-0.668285, 0.108740, 0.735915,
	0.033279, 0.548834, 0.835268,
	0.115931, 0.943564, 0.310237,
	0.000000, 0.000000, 0.000000,
	0.526615, 0.362459, 0.768960,
	-0.232450, -0.414692, -0.879771,
	0.337312, 0.780139, 0.526881,
	-0.788842, 0.597355, -0.144551,
	0.844526, 0.490527, 0.214845,
	0.848364, 0.113187, -0.517172,
	0.355015, -0.103423, 0.929122,
	0.051864, -0.586334, 0.808407,
	-0.133228, 0.931831, 0.337551,
	-0.552667, 0.829750, 0.077940,
	0.931426, 0.290292, -0.219491,
	0.907120, 0.200548, -0.370018,
	0.071084, -0.557736, 0.826969,
	0.000000, 0.000000, 0.000000,
	0.326659, 0.807118, 0.491788,
	0.388416, -0.717369, 0.578372,
	-0.562261, 0.824689, 0.061247,
	-0.329519, 0.917269, 0.223686,
	0.477602, -0.239959, -0.845172,
	0.930312, 0.273382, -0.244502,
	-0.313323, -0.882260, 0.351350,
	0.241900, -0.301746, 0.922190,
	-0.877165, 0.371110, -0.304727,
	-0.100424, 0.929590, 0.354650,
	-0.163766, -0.515333, -0.841197,
	0.927794, 0.363811, -0.082701,
	-0.193328, -0.951981, -0.237394,
	0.000000, 0.000000, 0.000000,
	0.344107, -0.478912, -0.807610,
	-0.807711, -0.347962, 0.475947,
	-0.849536, -0.124954, -0.512518,
	0.244858, 0.826768, 0.506457,
	-0.460846, -0.566481, -0.683169,
	0.834436, 0.499910, 0.231964,
	-0.568919, -0.686549, -0.452749,
	0.507015, 0.234743, 0.829356,
	-0.683494, -0.444729, -0.578836,
	0.720864, 0.390051, 0.572901,
	-0.636696, -0.550041, -0.540438,
	0.862192, 0.481788, 0.156539,
	-0.229985, -0.889491, -0.394857,
	0.000000, 0.000000, 0.000000,
	0.569332, 0.470949, 0.673846,
	-0.913370, 0.233227, 0.333707,
	0.266088, -0.832686, -0.485625,
	0.871093, 0.051379, 0.488423,
	-0.822880, -0.545441, 0.159255,
	0.194491, 0.514252, 0.835295,
	-0.599968, 0.024509, -0.799649,
	0.514720, 0.835852, 0.190824,
	0.098535, -0.772470, -0.627360,
	0.883029, 0.024242, 0.468692,
	-0.764888, -0.639907, 0.073925,
	0.011150, 0.392186, 0.919818,
	-0.503255, 0.013711, -0.864029,
	0.000000, 0.000000, 0.000000,
	0.487899, 0.671017, 0.558294,
	0.355984, 0.816902, 0.453814,
	-0.285885, -0.656041, -0.698484,
	0.913990, 0.056241, 0.401820,
	-0.679402, -0.670223, -0.298687,
	0.190851, 0.356999, 0.914400,
	-0.694820, -0.261556, -0.669936,
	0.357799, 0.916449, 0.179167,
	-0.243533, -0.623773, -0.742697,
	0.920960, 0.136550, 0.364949,
	-0.645570, -0.719580, -0.255820,
	0.241904, 0.301024, 0.922425,
	-0.370652, -0.288140, -0.882945,
	0.000000, 0.000000, 0.000000,
	0.067276, 0.602957, 0.794932,
	-0.196200, -0.601212, -0.774629,
	-0.188580, -0.577863, -0.794048,
	0.929971, 0.197638, 0.309990,
	-0.596151, -0.777892, -0.198717,
	0.278637, 0.225452, 0.933559,
	-0.802546, -0.140049, -0.579919,
	0.226224, 0.936758, 0.267033,
	-0.123953, -0.513270, -0.849228,
	0.940735, 0.245842, 0.233621,
	-0.525842, -0.840498, -0.130587,
	0.305109, 0.125472, 0.944016,
	-0.512142, -0.113164, -0.851413,
	0.000000, 0.000000, 0.000000,
	0.592426, 0.683929, 0.425760,
	0.080585, 0.606294, 0.791147,
	-0.057017, -0.428978, -0.901514,
	0.949541, 0.285096, 0.130734,
	-0.423399, -0.904066, -0.058294,
	-0.245393, -0.700677, 0.669950,
	-0.895866, 0.321147, -0.307064,
	-0.634309, 0.754706, -0.167543,
	0.808164, -0.361201, -0.465193,
	-0.733163, -0.501470, -0.459347,
	0.661540, 0.590573, 0.462157,
	-0.556462, -0.416265, -0.719078,
	0.217938, 0.358204, 0.907851,
	0.000000, 0.000000, 0.000000,
	0.715236, 0.378827, 0.587305,
	-0.397054, -0.841050, -0.367401,
	0.433056, 0.630738, 0.643920,
	-0.734918, -0.523049, -0.431643,
	0.659187, 0.609751, 0.440086,
	-0.279656, 0.067977, -0.957691,
	0.672227, 0.120903, 0.730406,
	0.162365, -0.955844, -0.244949,
	-0.099220, 0.713509, 0.693585,
	0.752073, 0.605476, 0.260356,
	-0.695363, -0.681419, -0.228336,
	0.172747, 0.334668, 0.926367,
	-0.226524, -0.325950, -0.917847,
	0.000000, 0.000000, 0.000000,
	-0.934511, -0.355811, 0.009369,
	0.262705, 0.493607, -0.829059,
	-0.180574, -0.650395, -0.737821,
	0.927999, 0.139874, 0.345331,
	-0.662991, -0.724277, -0.189386,
	0.209284, 0.278706, 0.937296,
	-0.745891, -0.138588, -0.651491,
	0.282723, 0.937615, 0.202353,
	-0.135331, -0.611275, -0.779762,
	0.939106, 0.183156, 0.290748,
	-0.621247, -0.770609, -0.142175,
	0.232595, 0.205628, 0.950588,
	-0.296984, -0.196841, -0.934374,
	0.000000, 0.000000, 0.000000,
	0.613585, 0.711242, 0.342998,
	0.138508, 0.744288, 0.653338,
	-0.086207, -0.562182, -0.822508,
	0.952134, 0.214310, 0.217970,
	-0.568671, -0.817571, -0.090503,
	0.243960, 0.110686, 0.963448,
	-0.833746, -0.023437, -0.551650,
	0.117278, 0.963334, 0.241317,
	-0.038629, -0.503708, -0.863010,
	0.964413, 0.234651, 0.121842,
	-0.499992, -0.865131, -0.039448,
	-0.383066, -0.708048, 0.593236,
	-0.676152, 0.577321, -0.457733,
	0.000000, 0.000000, 0.000000,
	0.833081, -0.527056, 0.167892,
	-0.725343, 0.672713, -0.146065,
	0.790291, -0.421796, -0.444441,
	-0.747271, -0.486802, -0.452339,
	0.671228, 0.594313, 0.442996,
	-0.527897, -0.407365, -0.745237,
	0.619317, 0.401779, 0.674551,
	-0.424237, -0.733626, -0.530862,
	0.419675, 0.643693, 0.639946,
	-0.746948, -0.504779, -0.432743,
	0.664955, 0.610518, 0.430235,
	-0.139763, 0.125108, -0.982250,
	0.231412, -0.097137, 0.967994,
	0.000000, 0.000000, 0.000000,
	-0.885286, -0.461764, -0.055170,
	0.172770, -0.685656, 0.707125,
	-0.066889, 0.733469, 0.676424,
	0.827712, 0.512990, 0.227452,
	-0.718873, -0.678275, -0.152200,
	0.112178, 0.309795, 0.944163,
	-0.697302, -0.112791, -0.707847,
	0.312602, 0.943612, 0.108980,
	-0.113902, -0.680698, -0.723655,
	0.942908, 0.092675, 0.319899,
	-0.687233, -0.716594, -0.119179,
	0.144209, 0.255376, 0.956027,
	-0.135635, -0.257113, -0.956816,
	0.000000, 0.000000, 0.000000,
	0.547678, 0.795945, 0.257916,
	0.189860, 0.870393, 0.454279,
	-0.079567, -0.645244, -0.759822,
	0.954884, 0.129547, 0.267236,
	-0.649998, -0.755350, -0.083359,
	0.163145, 0.186405, 0.968833,
	-0.769638, -0.024933, -0.637993,
	0.192856, 0.967670, 0.162544,
	-0.046297, -0.604705, -0.795102,
	0.967740, 0.154201, 0.199254,
	-0.607496, -0.792842, -0.048481,
	0.168877, 0.098689, 0.980684,
	-0.221969, -0.091656, -0.970736,
	0.000000, 0.000000, 0.000000,
	0.905358, -0.414971, 0.090141,
	0.108529, 0.982409, 0.151966,
	-0.018250, -0.560440, -0.827994,
	0.979755, 0.166471, 0.111208,
	-0.558795, -0.829097, -0.018633,
	-0.482935, -0.685983, 0.544243,
	-0.655805, 0.615201, -0.437546,
	-0.659439, 0.609783, -0.439665,
	0.771241, -0.485390, -0.411806,
	-0.774851, -0.458228, -0.435468,
	0.691256, 0.594419, 0.410890,
	-0.492299, -0.392481, -0.776917,
	-0.008674, 0.393124, 0.919444,
	0.000000, 0.000000, 0.000000,
	-0.832812, -0.551335, -0.049544,
	-0.276565, -0.836988, 0.472190,
	0.395335, 0.664455, 0.634201,
	-0.769243, -0.478499, -0.423442,
	0.680058, 0.609396, 0.407625,
	-0.024447, 0.139543, -0.989914,
	0.634920, 0.001051, 0.772577,
	0.140567, -0.989667, -0.028305,
	-0.034125, 0.757950, 0.651419,
	0.892163, 0.407790, 0.194300,
	-0.748617, -0.658684, -0.075558,
	0.022738, 0.286426, 0.957832,
	-0.004550, -0.291101, -0.956681,
	0.000000, 0.000000, 0.000000,
	0.460736, 0.874451, 0.151850,
	0.227361, 0.942067, 0.246612,
	-0.047617, -0.715572, -0.696915,
	0.955859, 0.011588, 0.293598,
	-0.717879, -0.694394, -0.049675,
	0.054174, 0.234438, 0.970620,
	-0.707004, -0.004509, -0.707195,
	0.236690, 0.970076, 0.054130,
	-0.024551, -0.682683, -0.730302,
	0.968896, 0.046132, 0.243130,
	-0.683913, -0.729112, -0.025647,
	0.073082, 0.170435, 0.982655,
	-0.044191, -0.176609, -0.983289,
	0.000000, 0.000000, 0.000000,
	0.962608, -0.204766, 0.177362,
	0.176142, 0.982352, 0.062908,
	-0.006703, -0.647510, -0.762027,
	0.981257, 0.068824, 0.179996,
	-0.647848, -0.761738, -0.007004,
	0.078538, 0.091020, 0.992747,
	-0.767652, 0.053410, -0.638638,
	0.096411, 0.992014, 0.081317,
	0.002470, -0.611668, -0.791110,
	0.991876, 0.078496, 0.100098,
	-0.611871, -0.790954, 0.002524,
	-0.564695, -0.645914, 0.513726,
	-0.442271, 0.734459, -0.514749,
	0.000000, 0.000000, 0.000000,
	-0.721528, -0.516817, 0.460758,
	-0.756905, 0.565195, -0.328099,
	0.751256, -0.549292, -0.365912,
	-0.812012, -0.411998, -0.413394,
	0.720496, 0.587325, 0.368694,
	-0.444402, -0.374709, -0.813695,
	0.606758, 0.333984, 0.721318,
	-0.396770, -0.796109, -0.456929,
	0.362108, 0.693001, 0.623400,
	-0.800947, -0.438631, -0.407539,
	0.704074, 0.603175, 0.374779,
	0.071319, 0.121023, -0.990084,
	-0.031954, -0.118310, 0.992463,
	0.000000, 0.000000, 0.000000,
	0.246868, 0.968216, 0.040165,
	0.098413, -0.988267, -0.116802,
	-0.001461, 0.784536, 0.620082,
	0.945142, 0.282497, 0.164017,
	-0.784042, -0.620700, -0.003108,
	-0.091764, 0.267041, 0.959306,
	-0.634001, 0.028484, -0.772807,
	0.263657, 0.960269, -0.091481,
	0.014721, -0.755502, -0.654981,
	0.957975, -0.098793, 0.269304,
	-0.754894, -0.655668, 0.015306,
	-0.059472, 0.218293, 0.974069,
	0.098952, -0.229355, -0.968300,
	0.000000, 0.000000, 0.000000,
	0.976960, 0.143680, 0.157816,
	0.194764, 0.980529, 0.025086,
	0.027044, -0.725017, -0.688199,
	0.973054, -0.065630, 0.221040,
	-0.723863, -0.689368, 0.028171,
	-0.038424, 0.159787, 0.986403,
	-0.694710, 0.072610, -0.715615,
	0.157331, 0.986745, -0.039772,
	0.030737, -0.693604, -0.719700,
	0.985787, -0.043294, 0.162324,
	-0.692283, -0.720914, 0.032050,
	-0.030023, 0.089028, 0.995577,
	0.049229, -0.092669, -0.994479,
	0.000000, 0.000000, 0.000000,
	-0.616301, -0.728372, 0.299413,
	0.118469, 0.902669, -0.413707,
	0.022490, -0.662760, -0.748494,
	0.995392, -0.033666, 0.089788,
	-0.664385, -0.747036, 0.022999,
	-0.638061, -0.590296, 0.494397,
	-0.381909, 0.753861, -0.534639,
	-0.647579, 0.348347, -0.677713,
	0.731616, -0.610391, -0.303579,
	-0.854281, -0.343205, -0.390402,
	0.758008, 0.568479, 0.319774,
	-0.379382, -0.357720, -0.853291,
	-0.224467, 0.428762, 0.875087,
	0.000000, 0.000000, 0.000000,
	-0.184808, 0.976416, -0.111615,
	-0.418193, -0.891130, 0.176073,
	0.322597, 0.729340, 0.603319,
	-0.839468, -0.379365, -0.389070,
	0.736819, 0.587469, 0.334630,
	0.153731, 0.076931, -0.985113,
	0.537338, -0.083564, 0.839217,
	0.121194, -0.959991, 0.252446,
	0.030426, 0.812439, 0.582252,
	0.981620, 0.129811, 0.139896,
	-0.824722, -0.562164, 0.061685,
	-0.229614, 0.253405, 0.939715,
	0.221853, -0.250779, -0.942280,
	0.000000, 0.000000, 0.000000,
	-0.005000, 0.669874, 0.742458,
	0.926851, 0.290606, 0.237686,
	-0.647477, -0.735860, 0.198201,
	-0.823536, 0.567155, 0.011142,
	-0.719059, 0.246297, -0.649840,
	0.566112, 0.074776, -0.820930,
	0.240156, -0.636373, -0.733045,
	0.083009, -0.829549, 0.552230,
	-0.637892, -0.734621, 0.231141,
	-0.833327, 0.546778, 0.081242,
	-0.714139, 0.272848, -0.644639,
	0.536226, 0.138703, -0.832600,
	0.281294, -0.603217, -0.746326,
	0.000000, 0.000000, 0.000000,
	0.083664, 0.996460, -0.008241,
	0.035225, -0.797152, -0.602750,
	0.996166, -0.010960, 0.086791,
	-0.795961, -0.604233, 0.036712,
	-0.595823, -0.435096, 0.675046,
	-0.405107, 0.611772, -0.679429,
	-0.494379, 0.313845, -0.810611,
	0.432905, -0.738364, -0.517120,
	-0.811568, -0.462380, -0.357158,
	0.718076, 0.642930, 0.266475,
	-0.073312, -0.471660, -0.878728,
	0.651082, 0.257843, 0.713869,
	-0.475133, -0.876511, -0.077313,
	0.286492, 0.685777, 0.669054,
	-0.132690, -0.462067, -0.876862,
	0.660206, 0.286438, 0.694320,
	-0.470178, -0.870725, -0.144120,
	0.326822, 0.659977, 0.676475,
	-0.865889, -0.136639, -0.481213,
	0.640637, 0.696069, 0.324149,
	0.579386, 0.031530, -0.814443,
	0.553932, -0.093248, 0.827323,
	0.297678, -0.526234, 0.796534,
	0.078615, 0.887266, 0.454509,
	0.940115, -0.274446, 0.202147,
	-0.907359, -0.412697, 0.079880,
	-0.228949, 0.172203, 0.958086,
	-0.428319, 0.099498, -0.898133,
	0.956301, -0.240787, 0.165861,
	-0.888394, -0.451157, 0.084933,
	-0.197679, 0.133658, 0.971112,
	-0.464177, 0.106777, -0.879283,
	0.121397, 0.970162, -0.209878,
	0.077718, -0.868903, -0.488843,
	0.968959, -0.213143, 0.125254,
	-0.866905, -0.491906, 0.080648,
	-0.172942, 0.091169, 0.980703,
	-0.501736, 0.104573, -0.858677,
	0.076697, 0.978837, -0.189727,
	0.060043, -0.844735, -0.531806,
	0.978142, -0.192111, 0.079571,
	-0.843097, -0.534121, 0.062474,
	-0.458350, 0.058473, -0.886846,
	0.387431, -0.809498, -0.441146,
	-0.881673, -0.314958, -0.351360,
	0.770371, 0.606081, 0.197976,
	0.081385, -0.485569, -0.870402,
	0.613835, 0.200212, 0.763624,
	-0.481934, -0.871905, 0.086729,
	0.225014, 0.738332, 0.635795,
	-0.865980, 0.094316, -0.491104,
	0.743352, 0.626233, 0.235075,
	0.014197, -0.479583, -0.877381,
	0.628869, 0.235957, 0.740843,
	-0.478754, -0.877809, 0.015678,
	0.272411, 0.707917, 0.651649,
	0.605464, -0.106562, -0.788706,
	0.452060, -0.079563, 0.888432,
	0.298350, -0.318387, 0.899787,
	0.132741, 0.906705, 0.400331,
	0.855123, -0.478546, 0.199394,
	-0.943587, -0.305654, 0.127356,
	-0.433768, 0.180737, 0.882711,
	-0.323845, 0.134936, -0.936438,
	0.161002, 0.877982, -0.450806,
	0.121763, -0.932167, -0.340938,
	0.875387, -0.454317, 0.165206,
	-0.930017, -0.345390, 0.125596,
	-0.404906, 0.147238, 0.902426,
	-0.362481, 0.131811, -0.922623,
	0.891509, -0.435094, 0.126114,
	-0.914052, -0.389562, 0.112917,
	-0.380140, 0.110186, 0.918342,
	-0.404692, 0.117302, -0.906899,
	0.078212, 0.904271, -0.419736,
	0.081068, -0.896741, -0.435067,
	0.902943, -0.422027, 0.081159,
	-0.895048, -0.437946, 0.084220,
	-0.759577, -0.224103, 0.610591,
	-0.202308, 0.535861, -0.819710,
	-0.428690, -0.193666, -0.882450,
	0.313958, -0.880476, -0.355237,
	-0.929479, -0.087166, -0.358429,
	0.835787, 0.534975, 0.123543,
	-0.484416, -0.819497, 0.306211,
	0.155403, 0.806158, 0.570928,
	-0.811706, 0.312351, -0.493529,
	0.809056, 0.564858, 0.162369,
	0.225395, -0.494819, -0.839256,
	0.570126, 0.177485, 0.802157,
	-0.482031, -0.839633, 0.250325,
	0.206139, 0.773655, 0.599137,
	-0.831800, 0.254829, -0.493123,
	0.767102, 0.607467, 0.206248,
	0.634763, -0.243590, -0.733307,
	0.346255, -0.030098, 0.937657,
	0.299580, -0.092359, 0.949590,
	0.180295, 0.923512, 0.338555,
	-0.641712, 0.199133, 0.740642,
	-0.194297, 0.158958, -0.967978,
	0.171477, 0.726223, -0.665730,
	0.151662, -0.966727, -0.206002,
	0.722407, -0.668703, 0.175970,
	-0.964944, -0.211025, 0.156049,
	-0.624512, 0.171250, 0.762009,
	-0.235025, 0.146217, -0.960929,
	0.134204, 0.736942, -0.662500,
	0.132428, -0.958436, -0.252712,
	0.733764, -0.665139, 0.138497,
	-0.956668, -0.257035, 0.136820,
	-0.610687, 0.138745, 0.779622,
	-0.282106, 0.120738, -0.951755,
	0.737571, -0.668914, 0.092425,
	-0.945041, -0.311157, 0.100395,
	-0.827860, -0.081124, 0.555038,
	-0.120605, 0.407741, -0.905098,
	-0.422843, -0.401554, -0.812378,
	0.199446, -0.940598, -0.274768,
	-0.899680, 0.215104, -0.379876,
	0.906055, 0.419498, 0.055549,
	0.503563, -0.498897, -0.705355,
	0.437609, 0.079807, 0.895617,
	-0.479517, -0.696832, 0.533375,
	0.090252, 0.882254, 0.462043,
	-0.687064, 0.537865, -0.488513,
	0.883588, 0.458690, 0.094212,
	-0.475940, -0.717074, 0.509202,
	0.141400, 0.852239, 0.503682,
	-0.707459, 0.512348, -0.486828,
	0.850213, 0.506505, 0.143491,
	0.675681, -0.360361, -0.643114,
	0.246820, 0.045121, 0.968010,
	0.297867, 0.099626, 0.949395,
	0.217970, 0.938297, 0.268491,
	0.507460, -0.833449, 0.218742,
	-0.981990, -0.018012, 0.188072,
	-0.812265, 0.218935, 0.540641,
	-0.044325, 0.172357, -0.984037,
	0.186887, 0.516166, -0.835850,
	0.170946, -0.984146, -0.047265,
	-0.810779, 0.196136, 0.551515,
	-0.084745, 0.151164, -0.984869,
	0.151319, 0.510823, -0.846264,
	0.146832, -0.984880, -0.091929,
	0.506232, -0.848142, 0.156158,
	-0.983765, -0.096256, 0.151465,
	-0.814097, 0.166360, 0.556391,
	-0.134920, 0.116437, -0.983991,
	0.104203, 0.492419, -0.864098,
	0.106212, -0.983065, -0.149340,
	0.488747, -0.865699, 0.108132,
	-0.982152, -0.152506, 0.110087,
	-0.896689, 0.048968, 0.439943,
	-0.055678, 0.243337, -0.968342,
	-0.755245, 0.507737, -0.414498,
	0.964238, 0.264641, 0.014475,
	0.673015, -0.500634, -0.544441,
	0.292674, 0.047505, 0.955032,
	-0.478614, -0.526545, 0.702624,
	0.051112, 0.948980, 0.311167,
	-0.514652, 0.705262, -0.487584,
	0.949470, 0.309298, 0.053305,
	0.657009, -0.506323, -0.558548,
	0.336631, 0.093338, 0.936999,
	-0.473262, -0.530053, 0.703610,
	0.101751, 0.926168, 0.363125,
	-0.518336, 0.705083, -0.483928,
	0.926040, 0.362564, 0.104868,
	0.290889, 0.243420, 0.925273,
	0.246302, 0.951404, 0.184838,
	0.286978, -0.930347, 0.228250,
	-0.966918, 0.154678, 0.202840,
	-0.922631, 0.231675, 0.308347,
	0.125675, 0.178142, -0.975946,
	0.199867, 0.279191, -0.939205,
	0.181169, -0.974223, 0.134412,
	0.272848, -0.939928, 0.205157,
	-0.974002, 0.129654, 0.185766,
	-0.930560, 0.211757, 0.298693,
	0.089723, 0.149548, -0.984675,
	0.165853, 0.253715, -0.952954,
	0.153200, -0.983345, 0.097789,
	0.301969, 0.709759, 0.636441,
	-0.667692, -0.676842, -0.309954,
	0.716126, 0.621449, 0.317751,
	-0.671447, -0.259871, -0.693993,
	0.635927, 0.262452, 0.725752,
	-0.261715, -0.672915, -0.691874,
	0.264661, 0.708150, 0.654582,
	-0.671131, -0.688046, -0.276001,
	0.714139, 0.641705, 0.279680,
	-0.679827, -0.223807, -0.698388,
	0.653565, 0.223914, 0.722991,
	-0.226113, -0.674505, -0.702791,
	0.226402, 0.702750, 0.674452,
	-0.673008, -0.699502, -0.240327,
	-0.407842, -0.615057, -0.674811,
	-0.060503, -0.554421, -0.830035,
	0.448110, 0.471536, 0.759508,
	-0.383249, -0.642978, -0.663098,
	0.468705, 0.747872, 0.470110,
	-0.638069, -0.655621, -0.403769,
	0.735137, 0.470329, 0.488226,
	-0.632190, -0.400114, -0.663509,
	0.451081, 0.476652, 0.754539,
	-0.391956, -0.638622, -0.662218,
	0.473764, 0.741360, 0.475324,
	-0.569544, -0.693052, -0.441926,
	0.659291, 0.529279, 0.534040,
	-0.712059, -0.546549, -0.440745,
	-0.476954, -0.110951, -0.871897,
	0.188379, 0.202482, 0.960997,
	0.589322, 0.473660, 0.654481,
	-0.508187, -0.517281, -0.688598,
	0.462855, 0.643001, 0.610176,
	-0.509348, -0.680412, -0.526882,
	0.633235, 0.605680, 0.481835,
	-0.667930, -0.524269, -0.528216,
	0.586934, 0.476110, 0.654849,
	-0.507857, -0.518869, -0.687645,
	0.465264, 0.641258, 0.610178,
	-0.510941, -0.677696, -0.528836,
	0.630251, 0.604794, 0.486834,
	-0.663376, -0.526295, -0.531927,
	0.181431, 0.255422, 0.949654,
	-0.555905, -0.241043, -0.795530,
	0.470093, 0.637750, 0.610153,
	-0.513803, -0.672487, -0.532699,
	0.625102, 0.603542, 0.494959,
	-0.655842, -0.530111, -0.537451,
	0.579879, 0.489106, 0.651549,
	-0.511312, -0.526381, -0.679326,
	0.484188, 0.614265, 0.623089,
	-0.524946, -0.649703, -0.549835,
	0.522340, 0.651167, 0.550584,
	-0.544038, -0.600412, -0.586112,
	0.674774, 0.629767, 0.384803,
	-0.675480, -0.629165, -0.384549,
	-0.504727, -0.542973, -0.671142,
	-0.268751, -0.552443, -0.789037,
	0.415832, 0.492197, 0.764739,
	-0.371455, -0.680450, -0.631671,
	0.485307, 0.757727, 0.436265,
	-0.672624, -0.628265, -0.390974,
	0.742570, 0.441173, 0.503941,
	-0.600274, -0.393930, -0.696053,
	0.416217, 0.498824, 0.760222,
	-0.384161, -0.659561, -0.646065,
	0.496960, 0.741685, 0.450482,
	-0.597530, -0.676007, -0.431245,
	0.658266, 0.519106, 0.545175,
	-0.721347, -0.521481, -0.455758,
	-0.483558, -0.385968, -0.785621,
	0.404027, 0.413638, 0.815883,
	0.571772, 0.484721, 0.661908,
	-0.497049, -0.534556, -0.683514,
	0.469371, 0.656063, 0.590993,
	-0.522499, -0.679921, -0.514493,
	0.642622, 0.591101, 0.487480,
	-0.663836, -0.516895, -0.540501,
	0.568268, 0.488097, 0.662444,
	-0.495980, -0.537543, -0.681947,
	0.472401, 0.654858, 0.589914,
	-0.525202, -0.676900, -0.515722,
	0.640100, 0.589329, 0.492913,
	-0.659033, -0.518427, -0.544894,
	-0.411492, 0.421976, 0.807843,
	-0.505283, -0.419475, -0.754142,
	0.477553, 0.651587, 0.589388,
	-0.528706, -0.671489, -0.519204,
	0.635122, 0.587775, 0.501140,
	-0.651440, -0.522062, -0.550525,
	0.560708, 0.502073, 0.658429,
	-0.498894, -0.546397, -0.672722,
	0.490861, 0.611766, 0.620321,
	-0.541019, -0.635045, -0.551376,
	0.525086, 0.646845, 0.553062,
	-0.532797, -0.614595, -0.581722,
	0.694605, 0.607710, 0.384983,
	-0.704519, -0.597563, -0.382846,
	-0.687947, -0.517663, -0.508679,
	0.168526, -0.716911, -0.676489,
	0.388397, 0.517406, 0.762522,
	-0.370845, -0.713982, -0.593888,
	0.507247, 0.759885, 0.406541,
	-0.703780, -0.594334, -0.389179,
	0.742898, 0.415769, 0.524633,
	-0.563611, -0.398201, -0.723726,
	0.386662, 0.526073, 0.757456,
	-0.381732, -0.681328, -0.624558,
	0.521928, 0.736551, 0.430214,
	-0.625551, -0.652218, -0.428133,
	0.657008, 0.508561, 0.556512,
	-0.726685, -0.496788, -0.474479,
	-0.487364, -0.617445, -0.617445,
	0.684963, 0.515182, 0.515182,
	0.556189, 0.498801, 0.664719,
	-0.489898, -0.551868, -0.674864,
	0.479889, 0.664422, 0.572931,
	-0.536292, -0.675897, -0.505524,
	0.647851, 0.577697, 0.496543,
	-0.656821, -0.512854, -0.552782,
	0.552081, 0.503128, 0.664882,
	-0.488637, -0.556009, -0.672375,
	0.483738, 0.663259, 0.571039,
	-0.539983, -0.672303, -0.506387,
	0.645451, 0.575230, 0.502496,
	-0.651729, -0.514095, -0.557633,
	0.160620, 0.715174, 0.680240,
	-0.490351, -0.620829, -0.611659,
	0.489252, 0.659901, 0.570231,
	-0.543966, -0.666561, -0.509703,
	0.640508, 0.573478, 0.510757,
	-0.644219, -0.517537, -0.563148,
	0.544960, 0.517554, 0.659664,
	-0.491692, -0.565568, -0.662096,
	0.494913, 0.609018, 0.619805,
	-0.551490, -0.620139, -0.557931,
	0.528109, 0.642214, 0.555573,
	-0.528731, -0.626250, -0.572935,
	0.713173, 0.582162, 0.390476,
	-0.728078, -0.564076, -0.389513,
	-0.756941, -0.369285, -0.539138,
	-0.117466, -0.692816, -0.711483,
	0.366525, 0.544603, 0.754366,
	-0.379530, -0.740939, -0.554045,
	0.531898, 0.755756, 0.381992,
	-0.728794, -0.558178, -0.396606,
	0.737377, 0.395047, 0.547917,
	-0.526341, -0.410500, -0.744618,
	0.363239, 0.555415, 0.748045,
	-0.384495, -0.702094, -0.599357,
	0.546765, 0.727368, 0.414710,
	-0.650998, -0.623606, -0.432801,
	0.655596, 0.497097, 0.568409,
	-0.728205, -0.473012, -0.495960,
	-0.504189, -0.771771, -0.387509,
	0.893655, 0.426956, 0.138163,
	0.543527, 0.514410, 0.663295,
	-0.486684, -0.567976, -0.663733,
	0.492794, 0.668256, 0.557305,
	-0.549402, -0.669363, -0.500111,
	0.649037, 0.566681, 0.507566,
	-0.647808, -0.512117, -0.563987,
	0.539330, 0.519397, 0.662835,
	-0.485605, -0.572859, -0.660318,
	0.497350, 0.666831, 0.554959,
	-0.553815, -0.665170, -0.500838,
	0.646619, 0.563758, 0.513869,
	-0.642596, -0.513109, -0.569025,
	0.734542, 0.592623, 0.330525,
	-0.510981, -0.774450, -0.372996,
	0.503001, 0.663291, 0.554108,
	-0.558039, -0.659195, -0.504038,
	0.641792, 0.561879, 0.521915,
	-0.635460, -0.516162, -0.574253,
	0.533263, 0.533308, 0.656668,
	-0.488972, -0.582375, -0.649419,
	0.496389, 0.607566, 0.620049,
	-0.556333, -0.607460, -0.566997,
	0.534402, 0.633711, 0.559307,
	-0.533810, -0.630412, -0.563584,
	0.729096, 0.552746, 0.403597,
	-0.743485, -0.532191, -0.404972,
	-0.769648, -0.230911, -0.595250,
	-0.368785, -0.557245, -0.743959,
	0.353727, 0.570752, 0.741026,
	-0.397484, -0.758732, -0.516075,
	0.555994, 0.746014, 0.366515,
	-0.744782, -0.523856, -0.413370,
	0.726334, 0.382819, 0.570867,
	-0.491879, -0.430600, -0.756729,
	0.349611, 0.583384, 0.733100,
	-0.394728, -0.718412, -0.572777,
	0.568951, 0.714159, 0.407764,
	-0.671249, -0.590270, -0.448338,
	0.654160, 0.484663, 0.580669,
	-0.726724, -0.451021, -0.518124,
	0.106615, 0.976966, 0.184852,
	0.682403, 0.214923, 0.698666,
	-0.212432, 0.653623, 0.726395,
	0.070242, -0.459514, -0.885389,
	0.654205, 0.726910, -0.208849,
	-0.462693, -0.883664, 0.071092,
	0.701495, -0.218342, 0.678403,
	-0.877162, 0.072669, -0.474664,
	-0.165663, 0.656874, 0.735576,
	0.050044, -0.501789, -0.863541,
	0.671701, 0.696594, -0.252141,
	-0.499448, -0.863752, 0.066965,
	0.763714, 0.419026, 0.491078,
	-0.693061, -0.423196, -0.583585,
	0.078557, -0.618797, -0.781613,
	0.091492, -0.276522, 0.956643,
	-0.621497, -0.779146, 0.081690,
	-0.277702, 0.955996, 0.094616,
	-0.552348, -0.507781, -0.661113,
	-0.077558, -0.590527, -0.803283,
	-0.638889, -0.579752, -0.505677,
	0.557538, 0.823578, -0.104257,
	0.449194, 0.765824, -0.460150,
	0.395990, -0.915546, 0.070475,
	0.841795, -0.410999, 0.349943,
	-0.909006, 0.086839, 0.407636,
	-0.442189, 0.356852, 0.822877,
	0.132679, 0.390275, -0.911089,
	0.809084, -0.433193, 0.397150,
	-0.900272, 0.157697, 0.405760,
	-0.463416, 0.402083, 0.789668,
	0.203149, 0.389520, -0.898334,
	0.420224, 0.772144, -0.476660,
	0.355692, -0.906941, 0.225703,
	0.818107, -0.165549, 0.550722,
	-0.671965, 0.447060, 0.590425,
	0.066921, 0.612332, 0.787764,
	0.306009, 0.717618, -0.625606,
	-0.592632, -0.802061, 0.074068,
	-0.187007, 0.951129, 0.245726,
	-0.787540, 0.051901, -0.614074,
	0.952584, 0.237148, -0.190643,
	-0.625361, -0.776275, 0.079505,
	-0.152093, 0.966756, 0.205548,
	-0.758897, 0.043055, -0.649786,
	0.967687, 0.196232, -0.158350,
	0.072090, -0.654152, -0.752920,
	0.154089, -0.122645, 0.980416,
	-0.656829, -0.750309, 0.074915,
	-0.125324, 0.979376, 0.158482,
	-0.729719, 0.020843, -0.683429,
	0.979847, 0.148295, -0.133821,
	0.054627, -0.684097, -0.727343,
	0.099528, -0.106380, 0.989332,
	-0.686022, -0.725354, 0.056881,
	-0.108014, 0.988804, 0.102955,
	-0.598844, -0.581162, -0.551032,
	0.636918, 0.764389, 0.100226,
	0.549103, 0.715957, -0.431152,
	0.201979, -0.978001, 0.052143,
	0.824675, -0.363089, 0.433679,
	-0.976663, 0.059665, 0.206324,
	-0.404237, 0.438602, 0.802633,
	0.117518, 0.192964, -0.974143,
	0.456004, 0.785602, -0.418199,
	0.189484, -0.974287, 0.121905,
	0.782954, -0.396322, 0.479491,
	-0.971521, 0.130906, 0.197510,
	-0.438103, 0.481227, 0.759267,
	0.191101, 0.190212, -0.962964,
	0.780269, -0.070119, 0.621501,
	-0.887790, 0.342971, 0.306919,
	0.008964, 0.646952, 0.762478,
	0.311620, 0.627691, -0.713370,
	-0.645530, -0.763663, 0.010490,
	-0.050195, 0.962007, 0.268371,
	-0.745605, -0.009043, -0.666326,
	0.962571, 0.266276, -0.050538,
	0.023695, -0.676187, -0.736349,
	0.221261, -0.008238, 0.975180,
	-0.677211, -0.735376, 0.024639,
	-0.013844, 0.973893, 0.226586,
	-0.714770, -0.008593, -0.699306,
	0.974064, 0.225826, -0.014189,
	-0.707158, -0.706344, 0.031725,
	0.012539, 0.984219, 0.176508,
	-0.683572, -0.018144, -0.729657,
	0.984050, 0.177408, 0.013121,
	0.027414, -0.733463, -0.679177,
	0.111928, 0.030365, 0.993252,
	-0.734484, -0.678024, 0.028585,
	0.027955, 0.992875, 0.115832,
	-0.505358, -0.587967, -0.631592,
	0.885372, 0.356220, 0.298704,
	-0.536190, -0.602538, -0.591141,
	0.405740, 0.431559, 0.805687,
	0.630123, 0.667690, -0.396402,
	0.032369, -0.999257, 0.020907,
	-0.357338, 0.503928, 0.786363,
	0.089799, 0.021461, -0.995729,
	0.520501, 0.768349, -0.372450,
	0.018706, -0.995467, 0.093249,
	0.760043, -0.354912, 0.544401,
	-0.995387, 0.093990, 0.019268,
	-0.406992, 0.544492, 0.733407,
	0.167189, 0.018214, -0.985757,
	0.561972, 0.710544, -0.423455,
	-0.011329, -0.982352, 0.186700,
	0.746513, 0.001308, 0.665369,
	-0.984062, 0.176349, -0.022858,
	-0.049239, 0.677497, 0.733876,
	0.315284, 0.546430, -0.775893,
	-0.709314, -0.073939, -0.701004,
	0.952898, 0.297197, 0.060493,
	-0.035316, -0.713695, -0.699565,
	0.245668, 0.105569, 0.963588,
	-0.711963, -0.701251, -0.036834,
	0.097280, 0.962871, 0.251825,
	-0.676786, -0.062203, -0.733547,
	0.961621, 0.256286, 0.097989,
	-0.015677, -0.743797, -0.668222,
	0.192569, 0.128257, 0.972866,
	-0.743090, -0.668991, -0.016369,
	0.122343, 0.972468, 0.198341,
	-0.642943, -0.056391, -0.763836,
	0.970601, 0.205539, 0.125245,
	-0.771834, -0.635820, -0.002392,
	0.135639, 0.982003, 0.131419,
	-0.501991, -0.596461, -0.626290,
	0.653324, 0.539826, 0.530808,
	-0.456226, -0.641330, -0.616891,
	-0.241736, -0.146351, 0.959242,
	0.698696, 0.621667, -0.354053,
	-0.113181, -0.993348, -0.021204,
	0.799977, -0.239206, 0.550288,
	-0.993290, -0.023730, -0.113190,
	-0.297271, 0.555185, 0.776788,
	0.051784, -0.126249, -0.990646,
	0.570642, 0.759253, -0.312893,
	-0.127951, -0.990316, 0.053882,
	-0.366193, 0.595235, 0.715261,
	0.133775, -0.130131, -0.982431,
	0.612545, 0.690607, -0.384513,
	-0.153479, -0.976687, 0.150088,
	0.718657, 0.053808, 0.693280,
	-0.957614, 0.021837, -0.287227,
	-0.106349, 0.703299, 0.702894,
	0.314822, 0.479499, -0.819126,
	-0.696086, -0.705005, -0.135762,
	0.145359, 0.934944, 0.323651,
	-0.678408, -0.140180, -0.721188,
	0.934034, 0.327635, 0.142255,
	-0.098033, -0.737406, -0.668298,
	0.271988, 0.194758, 0.942386,
	-0.644025, -0.115929, -0.756169,
	0.941293, 0.285349, 0.180397,
	-0.064684, -0.770759, -0.633835,
	0.215242, 0.216162, 0.952337,
	-0.767398, -0.637587, -0.067702,
	0.207181, 0.952812, 0.221867,
	-0.605844, -0.092749, -0.790159,
	0.950608, 0.230763, 0.207591,
	-0.033481, -0.802898, -0.595175,
	0.142892, 0.225916, 0.963610,
	-0.801383, -0.597125, -0.035021,
	0.220406, 0.964110, 0.148028,
	-0.501303, -0.590769, -0.632208,
	0.645652, 0.541212, 0.538723,
	0.758703, 0.576506, -0.303332,
	-0.236956, -0.968820, -0.072383,
	0.796151, -0.155530, 0.584768,
	-0.969167, -0.075252, -0.234634,
	-0.219370, 0.591672, 0.775759,
	0.005287, -0.253913, -0.967212,
	0.605194, 0.760953, -0.233862,
	-0.254109, -0.967160, 0.005519,
	0.735842, -0.241722, 0.632540,
	-0.966909, 0.001600, -0.255117,
	-0.310886, 0.633938, 0.708148,
	0.092493, -0.259887, -0.961199,
	0.651118, 0.683115, -0.330756,
	-0.275903, -0.955492, 0.104464,
	-0.161452, 0.722840, 0.671889,
	0.306941, 0.433468, -0.847285,
	-0.696990, -0.685380, -0.210854,
	0.200347, 0.914403, 0.351750,
	-0.652550, -0.204442, -0.729645,
	0.914633, 0.355209, 0.193061,
	-0.161359, -0.749400, -0.642155,
	0.297976, 0.260031, 0.918474,
	-0.738243, -0.653002, -0.169074,
	0.240787, 0.921097, 0.305945,
	-0.615246, -0.167421, -0.770352,
	0.921150, 0.310908, 0.234134,
	-0.114451, -0.789298, -0.603249,
	0.237483, 0.282695, 0.929347,
	0.402872, 0.229617, 0.885986,
	-0.124604, -0.722357, -0.680201,
	0.198841, 0.893672, 0.402260,
	-0.714118, -0.688401, -0.127040,
	0.888665, 0.409886, 0.205593,
	-0.654131, -0.144848, -0.742382,
	0.366715, 0.221928, 0.903476,
	-0.100641, -0.760163, -0.641890,
	0.194676, 0.909566, 0.367138,
	-0.754095, -0.648679, -0.102741,
	0.904950, 0.374501, 0.202027,
	-0.611768, -0.122750, -0.781455,
	0.331040, 0.219656, 0.917695,
	-0.084635, -0.795460, -0.600066,
	-0.188777, -0.242248, -0.951672,
	-0.449099, 0.449099, -0.772412,
	0.313479, 0.641675, 0.699988,
	-0.507606, -0.785690, -0.353591,
	0.614997, 0.723683, 0.313148,
	-0.758387, -0.393217, -0.519836,
	0.676704, 0.354444, 0.645322,
	-0.331722, -0.548353, -0.767639,
	0.296704, 0.672505, 0.678014,
	-0.489716, -0.775310, -0.398839,
	0.631394, 0.687672, 0.358398,
	-0.702215, -0.325444, -0.633230,
	0.661708, 0.303107, 0.685761,
	-0.738796, -0.216437, -0.638228,
	-0.941474, -0.328249, 0.076674,
	0.973587, 0.169445, 0.153023,
	0.501088, 0.584802, 0.637900,
	-0.481631, -0.617553, -0.621820,
	0.541442, 0.670664, 0.507001,
	-0.575122, -0.655559, -0.489363,
	0.624774, 0.553318, 0.550905,
	-0.601165, -0.533887, -0.594613,
	0.491398, 0.592164, 0.638647,
	-0.474488, -0.629897, -0.614891,
	0.552726, 0.673316, 0.491059,
	-0.590172, -0.651668, -0.476473,
	0.631209, 0.538194, 0.558500,
	-0.599949, -0.523388, -0.605084,
	0.472905, -0.828184, 0.300786,
	-0.732072, -0.671482, -0.114814,
	0.565107, 0.673517, 0.476476,
	-0.607067, -0.643865, -0.465733,
	0.635665, 0.521712, 0.568987,
	-0.596053, -0.512130, -0.618420,
	0.470734, 0.607570, 0.639741,
	-0.462952, -0.655797, -0.596327,
	0.513319, 0.665155, 0.542284,
	-0.573207, -0.619972, -0.535788,
	0.559027, 0.447578, 0.697970,
	-0.592945, -0.459957, -0.660951,
	0.802343, 0.102251, 0.588039,
	-0.758105, -0.402869, -0.512810,
	-0.072726, -0.229489, -0.970590,
	-0.347559, 0.684074, -0.641284,
	0.275039, 0.628175, 0.727839,
	-0.523118, -0.798457, -0.298015,
	0.598362, 0.753363, 0.272777,
	-0.765604, -0.355456, -0.536191,
	0.689865, 0.329172, 0.644773,
	-0.265646, -0.570696, -0.777006,
	0.247626, 0.670896, 0.698985,
	-0.521169, -0.791891, -0.318264,
	0.633557, 0.714127, 0.297706,
	-0.688825, -0.191555, -0.699161,
	0.676607, 0.186114, 0.712435,
	-0.766675, -0.064175, -0.638819,
	-0.984290, -0.151877, 0.090036,
	0.993796, 0.102501, 0.043156,
	0.478043, 0.589933, 0.650733,
	-0.459364, -0.617866, -0.638143,
	0.534982, 0.687030, 0.491716,
	-0.565927, -0.674708, -0.473810,
	0.626196, 0.553615, 0.548989,
	-0.600605, -0.530487, -0.598211,
	0.460010, 0.602932, 0.651816,
	-0.443915, -0.637646, -0.629561,
	0.554501, 0.692861, 0.460948,
	-0.589275, -0.672825, -0.447282,
	0.637843, 0.528849, 0.559888,
	-0.600303, -0.513322, -0.613299,
	0.598172, -0.711085, 0.369524,
	-0.750377, -0.656918, -0.073438,
	0.577918, 0.691368, 0.433614,
	-0.618292, -0.660685, -0.425688,
	0.644433, 0.501694, 0.577069,
	-0.593693, -0.495807, -0.633801,
	0.427403, 0.632340, 0.646122,
	-0.426155, -0.682229, -0.594100,
	0.548318, 0.688422, 0.474786,
	-0.606893, -0.636095, -0.476512,
	0.526241, 0.322604, 0.786763,
	-0.572102, -0.334766, -0.748753,
	0.767643, -0.261400, 0.585145,
	-0.772507, -0.363238, -0.520856,
	-0.421662, -0.238126, -0.874927,
	-0.073460, -0.796613, -0.600009,
	0.187105, 0.566739, 0.802371,
	-0.541763, -0.820355, -0.183058,
	0.537928, 0.822570, 0.184425,
	-0.780925, -0.280578, -0.558061,
	0.737348, 0.268474, 0.619871,
	-0.124181, -0.600975, -0.789562,
	0.122619, 0.626400, 0.769797,
	-0.578306, -0.802750, -0.145442,
	0.606007, 0.782384, 0.143636,
	-0.613821, 0.093175, -0.783928,
	0.717511, -0.056578, 0.694246,
	-0.807697, 0.219023, -0.547408,
	-0.999124, -0.004540, 0.041606,
	0.980334, 0.021409, -0.196183,
	0.425990, 0.585435, 0.689781,
	-0.411399, -0.600937, -0.685293,
	0.512619, 0.721826, 0.464961,
	-0.538912, -0.715859, -0.443982,
	0.633834, 0.553438, 0.540334,
	-0.609542, -0.515845, -0.601966,
	0.373880, 0.615349, 0.693944,
	-0.362764, -0.633485, -0.683447,
	0.555128, 0.737640, 0.384344,
	-0.575607, -0.727302, -0.373777,
	0.657609, 0.502901, 0.560928,
	-0.612498, -0.481363, -0.627006,
	0.326250, -0.869226, 0.371494,
	-0.747560, -0.663198, -0.036367,
	0.613445, 0.722176, 0.319603,
	-0.635994, -0.704053, -0.315945,
	0.662343, 0.449796, 0.599153,
	-0.594951, -0.450993, -0.665311,
	0.317593, 0.690964, 0.649387,
	-0.327828, -0.732314, -0.596862,
	0.636186, 0.699089, 0.326408,
	-0.677948, -0.652660, -0.338262,
	0.343556, 0.039640, 0.938295,
	-0.451524, -0.050632, -0.890821,
	0.471654, -0.830672, 0.295849,
	-0.827371, -0.225287, -0.514493,
	-0.329482, -0.250161, -0.910418,
	0.461292, -0.487002, 0.741646,
	-0.046036, 0.266807, 0.962650,
	-0.537725, -0.811737, 0.227893,
	0.277543, 0.959566, -0.046941,
	-0.808319, 0.005849, -0.588715,
	0.912650, 0.042307, 0.406546,
	0.355542, -0.589402, -0.725393,
	-0.343321, 0.174495, 0.922866,
	-0.645105, -0.662807, 0.380165,
	0.262375, 0.885765, -0.382857,
	-0.211479, 0.679016, -0.703003,
	0.807098, -0.526780, 0.266638,
	-0.815228, 0.555829, -0.162657,
	-0.992851, 0.095664, -0.071374,
	0.544993, -0.168864, -0.821260,
	0.321632, 0.511239, 0.796986,
	-0.338429, -0.504735, -0.794172,
	0.426535, 0.773832, 0.468244,
	-0.452723, -0.787325, -0.418523,
	0.643510, 0.575035, 0.505202,
	-0.660217, -0.473485, -0.583031,
	-0.062783, 0.521753, 0.850783,
	-0.103146, -0.450250, -0.886925,
	0.541328, 0.835673, -0.092818,
	-0.426429, -0.895612, -0.126641,
	0.741702, 0.337157, 0.579830,
	-0.731446, -0.316573, -0.603960,
	0.059385, -0.896001, 0.440063,
	-0.681246, -0.730531, 0.047206,
	0.763240, 0.525391, -0.376071,
	-0.520497, -0.796056, 0.308831,
	0.638819, 0.163955, 0.751684,
	-0.653431, -0.158535, -0.740199,
	-0.135905, 0.870649, 0.472759,
	0.307776, -0.725279, -0.615828,
	0.883516, 0.457140, -0.102093,
	-0.806499, -0.556785, 0.198872,
	-0.332728, -0.340781, 0.879295,
	0.112379, 0.495563, -0.861271,
	0.172512, -0.968748, -0.178231,
	-0.015553, 0.971364, 0.237087,
	-0.209198, -0.262437, -0.942000,
	0.322084, -0.227894, 0.918872,
	-0.134840, -0.344070, 0.929211,
	0.058426, 0.259532, 0.963965,
	-0.335807, 0.925822, -0.173459,
	0.279838, 0.956227, 0.085564,
	0.827863, -0.205454, -0.521949,
	0.985646, 0.023123, 0.167235,
	-0.438865, -0.762362, 0.475606,
	-0.267029, 0.251022, 0.930421,
	-0.676475, 0.460357, -0.574851,
	0.428979, 0.865387, -0.259003,
	0.822603, -0.045688, -0.566778,
	-0.912482, 0.287449, 0.291117,
	-0.197552, 0.197552, 0.960180,
	-0.146387, 0.235008, 0.960907,
	-0.039859, 0.848179, -0.528207,
	0.598485, 0.784467, -0.162564,
	0.849034, -0.526936, -0.038462,
	0.687905, -0.060811, 0.723249,
	-0.473694, 0.079519, 0.877092,
	-0.215703, 0.537023, 0.815523,
	0.169678, 0.837497, -0.519431,
	0.513663, 0.809210, -0.285183,
	0.815611, -0.531277, 0.229181,
	0.697680, -0.302128, 0.649586,
	0.249871, -0.152908, 0.956129,
	0.412777, 0.543117, 0.731190,
	0.000000, 0.000000, 0.000000,
	0.693174, -0.652412, 0.306380,
	-0.303786, 0.308880, 0.901281,
	-0.715919, -0.408110, 0.566485,
	0.275284, 0.939486, -0.203921,
	-0.163814, 0.898421, -0.407436,
	0.939720, -0.308693, 0.147088,
	-0.367660, -0.069886, -0.927330,
	-0.566455, 0.434352, 0.700334,
	-0.184451, -0.982298, -0.032678,
	0.812397, -0.369202, -0.451333,
	0.339317, -0.699046, -0.629443,
	0.772975, 0.522404, 0.360005,
	0.789024, 0.583153, 0.193323,
	0.000000, 0.000000, 0.000000,
	0.678385, -0.325867, 0.658486,
	0.060501, 0.841110, 0.537470,
	0.920449, 0.351840, -0.170240,
	0.847174, 0.527055, 0.067148,
	0.704383, 0.037727, 0.708816,
	0.370796, -0.442080, 0.816747,
	-0.423296, 0.781388, 0.458534,
	-0.421498, 0.781231, 0.460454,
	0.751532, -0.193799, -0.630589,
	0.768341, 0.612610, -0.185365,
	-0.889790, -0.442001, 0.113619,
	0.546932, 0.618370, 0.564345,
	-0.637618, -0.549046, -0.540362,
	0.000000, 0.000000, 0.000000,
	-0.646047, -0.269249, -0.714233,
	0.524295, 0.412029, 0.745216,
	-0.456185, -0.617187, -0.641073,
	0.517330, 0.642636, 0.565144,
	-0.662969, -0.559606, -0.497306,
	0.690411, 0.445794, 0.569737,
	-0.579221, -0.301516, -0.757358,
	0.571342, 0.307072, 0.761101,
	-0.422035, -0.603550, -0.676472,
	0.464284, 0.666403, 0.583393,
	-0.674826, -0.596337, -0.434733,
	0.732464, 0.492186, 0.470373,
	-0.648847, -0.224230, -0.727130,
	0.000000, 0.000000, 0.000000,
	-0.390576, -0.375627, -0.840449,
	0.406326, 0.670292, 0.620973,
	-0.643036, -0.662076, -0.384916,
	0.729726, 0.568277, 0.380211,
	-0.736487, -0.219142, -0.639971,
	0.588935, 0.171516, 0.789771,
	-0.230518, -0.475456, -0.849001,
	0.123311, 0.866601, 0.483526,
	-0.498832, -0.856106, -0.135088,
	0.799318, 0.594098, 0.090216,
	-0.502610, -0.635978, -0.585590,
	0.662287, 0.568998, 0.487461,
	-0.674962, -0.480026, -0.560358,
	0.000000, 0.000000, 0.000000,
	0.367494, -0.929849, -0.018148,
	0.415487, 0.637070, 0.649240,
	-0.441269, -0.653630, -0.614857,
	0.681751, 0.598228, 0.421117,
	-0.704645, -0.545027, -0.454336,
	0.639603, 0.243602, 0.729085,
	-0.569726, -0.285227, -0.770751,
	0.187575, 0.682885, 0.706034,
	-0.337813, -0.824892, -0.453250,
	0.711541, 0.692779, 0.117334,
	-0.752338, -0.644166, -0.137974,
	0.553904, 0.612242, 0.564225,
	-0.629624, -0.563662, -0.534657,
	0.000000, 0.000000, 0.000000,
	-0.830847, -0.202996, -0.518156,
	0.547274, 0.467490, 0.694222,
	-0.495286, -0.587259, -0.640171,
	0.538130, 0.625130, 0.565358,
	-0.635710, -0.577319, -0.512422,
	0.671723, 0.494475, 0.551618,
	-0.618413, -0.410011, -0.670415,
	0.563851, 0.436198, 0.701287,
	-0.483471, -0.576287, -0.658900,
	0.515507, 0.638717, 0.571220,
	-0.633486, -0.600543, -0.487897,
	0.689060, 0.511352, 0.513533,
	-0.651530, -0.390989, -0.650104,
	0.000000, 0.000000, 0.000000,
	-0.399856, -0.284509, -0.871304,
	0.488061, 0.649771, 0.582747,
	-0.619112, -0.631405, -0.466934,
	0.697461, 0.536220, 0.475412,
	-0.685415, -0.384089, -0.618613,
	0.615281, 0.384695, 0.688069,
	-0.483724, -0.533169, -0.694076,
	0.454035, 0.739159, 0.497490,
	-0.588983, -0.717749, -0.371396,
	0.763635, 0.542426, 0.350195,
	-0.752545, -0.321026, -0.574994,
	0.567728, 0.281433, 0.773616,
	-0.378704, -0.494793, -0.782153,
	0.000000, 0.000000, 0.000000,
	0.044379, -0.926767, -0.373005,
	0.758961, 0.567843, 0.318642,
	-0.782139, -0.334897, -0.525455,
	0.581938, 0.260590, 0.770351,
	-0.385775, -0.448527, -0.806226,
	0.301383, 0.737272, 0.604647,
	-0.494524, -0.785257, -0.372582,
	0.752633, 0.590424, 0.291449,
	-0.795188, -0.447691, -0.408960,
	0.587615, 0.412903, 0.695858,
	-0.464197, -0.512394, -0.722477,
	0.438588, 0.662208, 0.607554,
	-0.536624, -0.697455, -0.474964,
	0.000000, 0.000000, 0.000000,
	-0.775347, -0.311449, 0.549397,
	0.596291, 0.415693, 0.686758,
	-0.473498, -0.504780, -0.721801,
	0.438911, 0.655693, 0.614349,
	-0.523023, -0.697586, -0.489715,
	0.661027, 0.596806, 0.454825,
	-0.700465, -0.468056, -0.538769,
	0.605550, 0.429108, 0.670205,
	-0.492410, -0.505031, -0.708855,
	0.452857, 0.643937, 0.616656,
	-0.519202, -0.684805, -0.511343,
	0.645214, 0.600339, 0.472538,
	-0.683406, -0.492337, -0.539036,
	0.000000, 0.000000, 0.000000,
	-0.463942, 0.503019, -0.729198,
	0.482996, 0.626427, 0.611804,
	-0.527060, -0.657256, -0.538723,
	0.624465, 0.597213, 0.503368,
	-0.653407, -0.522673, -0.547606,
	0.670919, 0.533488, 0.515032,
	-0.618291, -0.573458, -0.537458,
	0.586752, 0.633444, 0.504451,
	-0.544225, -0.622003, -0.562967,
	0.480283, 0.652730, 0.585894,
	-0.481702, -0.630171, -0.608973,
	0.562821, 0.660085, 0.497515,
	-0.634324, -0.593309, -0.495597,
	0.000000, 0.000000, 0.000000,
	0.480464, -0.739610, -0.471308,
	0.514778, 0.543644, 0.662914,
	-0.508348, -0.614304, -0.603501,
	0.555965, 0.636512, 0.534560,
	-0.612240, -0.588055, -0.528538,
	0.632661, 0.517359, 0.576263,
	-0.675844, -0.522652, -0.519682,
	0.635781, 0.557046, 0.534305,
	-0.616909, -0.614629, -0.491584,
	0.555749, 0.602760, 0.572559,
	-0.478421, -0.652309, -0.587883,
	0.478435, 0.652000, 0.588214,
	-0.562613, -0.662409, -0.494652,
	0.000000, 0.000000, 0.000000,
	-0.853112, -0.185940, 0.487469,
	0.570215, 0.464029, 0.677888,
	-0.484476, -0.539835, -0.688375,
	0.479004, 0.647353, 0.592866,
	-0.552350, -0.663009, -0.505300,
	0.650047, 0.573351, 0.498707,
	-0.663454, -0.482861, -0.571553,
	0.578995, 0.472225, 0.664656,
	-0.499388, -0.538427, -0.678755,
	0.488668, 0.638144, 0.594958,
	-0.547247, -0.654886, -0.521195,
	0.636627, 0.578277, 0.510198,
	-0.651363, -0.501068, -0.569786,
	0.000000, 0.000000, 0.000000,
	-0.342441, 0.444139, -0.827934,
	0.509597, 0.623593, 0.592827,
	-0.548565, -0.636598, -0.542051,
	0.618842, 0.580346, 0.529370,
	-0.631075, -0.525132, -0.570947,
	0.645865, 0.528680, 0.550777,
	-0.605016, -0.564267, -0.561745,
	0.576412, 0.645983, 0.500455,
	-0.547956, -0.644421, -0.533353,
	0.532826, 0.654311, 0.536633,
	-0.528508, -0.610854, -0.589522,
	0.572899, 0.617053, 0.539474,
	-0.612945, -0.572084, -0.544994,
	0.000000, 0.000000, 0.000000,
	0.467503, -0.820397, -0.329225,
	0.547235, 0.556391, 0.625270,
	-0.549868, -0.596194, -0.584977,
	0.564755, 0.599223, 0.567436,
	-0.591572, -0.571273, -0.568937,
	0.594539, 0.550150, 0.586394,
	-0.639938, -0.547789, -0.538895,
	0.635955, 0.551618, 0.539703,
	-0.626787, -0.608778, -0.486341,
	0.561925, 0.609002, 0.559783,
	-0.505989, -0.653607, -0.562826,
	0.504228, 0.640013, 0.579774,
	-0.575356, -0.637832, -0.511992,
	0.000000, 0.000000, 0.000000,
	-0.895770, -0.077989, 0.437624,
	0.557952, 0.496171, 0.665209,
	-0.497452, -0.558617, -0.663693,
	0.506099, 0.635325, 0.583290,
	-0.567149, -0.637595, -0.521358,
	0.635856, 0.562532, 0.528436,
	-0.636328, -0.498657, -0.588581,
	0.565861, 0.502088, 0.653994,
	-0.510164, -0.556564, -0.655720,
	0.513930, 0.627682, 0.584715,
	-0.562002, -0.631985, -0.533618,
	0.624372, 0.567882, 0.536349,
	-0.627043, -0.512935, -0.586272,
	0.000000, 0.000000, 0.000000,
	-0.235607, 0.401010, -0.885257,
	0.529253, 0.616384, 0.583063,
	-0.560624, -0.620203, -0.548680,
	0.610420, 0.572149, 0.547754,
	-0.613568, -0.531293, -0.584176,
	0.618507, 0.531487, 0.578767,
	-0.587519, -0.561720, -0.582488,
	0.570283, 0.657804, 0.492007,
	-0.557609, -0.659033, -0.504724,
	0.574054, 0.645748, 0.503459,
	-0.558574, -0.588800, -0.584217,
	0.571758, 0.588539, 0.571590,
	-0.589160, -0.565617, -0.577034,
	0.000000, 0.000000, 0.000000,
	0.449596, -0.871370, -0.196413,
	0.573569, 0.557350, 0.600316,
	-0.577551, -0.574821, -0.579669,
	0.563410, 0.574441, 0.593790,
	-0.569277, -0.567260, -0.595096,
	0.569301, 0.576659, 0.585970,
	-0.612399, -0.569271, -0.548542,
	0.633364, 0.546798, 0.547597,
	-0.634347, -0.603511, -0.483092,
	0.569735, 0.613456, 0.546877,
	-0.529517, -0.651658, -0.543097,
	0.522172, 0.628655, 0.576306,
	-0.581837, -0.619457, -0.527009,
	0.000000, 0.000000, 0.000000,
	-0.920024, 0.026446, 0.390969,
	0.552160, 0.518950, 0.652541,
	-0.509936, -0.569238, -0.644929,
	0.525408, 0.622951, 0.579550,
	-0.574688, -0.618097, -0.536368,
	0.621689, 0.557806, 0.549867,
	-0.615584, -0.513456, -0.597846,
	0.559372, 0.523785, 0.642458,
	-0.521467, -0.566694, -0.637911,
	0.532442, 0.616140, 0.580411,
	-0.569830, -0.614002, -0.546165,
	0.611722, 0.563341, 0.555377,
	-0.608078, -0.525071, -0.595434,
	0.000000, 0.000000, 0.000000,
	-0.132350, 0.363007, -0.922339,
	0.544159, 0.607839, 0.578293,
	-0.567620, -0.607047, -0.556149,
	0.601664, 0.568372, 0.561207,
	-0.599641, -0.538404, -0.592073,
	0.589094, 0.539466, 0.601619,
	-0.566468, -0.564105, -0.600749,
	0.567689, 0.668852, 0.479965,
	-0.571469, -0.668005, -0.476646,
	0.609177, 0.631635, 0.479521,
	-0.578194, -0.567533, -0.586172,
	0.564493, 0.569607, 0.597407,
	-0.566371, -0.566695, -0.598399,
	0.000000, 0.000000, 0.000000,
	0.424049, -0.903204, -0.066373,
	0.595343, 0.551523, 0.584286,
	-0.596169, -0.553492, -0.581575,
	0.556352, 0.558086, 0.615639,
	-0.548086, -0.569951, -0.612174,
	0.552670, 0.598777, 0.579675,
	-0.589523, -0.588776, -0.552997,
	0.627217, 0.543397, 0.557960,
	-0.640340, -0.598793, -0.481052,
	0.579692, 0.616123, 0.533245,
	-0.551672, -0.647483, -0.525761,
	0.535294, 0.618199, 0.575578,
	-0.584678, -0.605024, -0.540460,
	0.000000, 0.000000, 0.000000,
	-0.931506, 0.133269, 0.338431,
	0.549932, 0.535931, 0.640588,
	-0.521657, -0.575049, -0.630233,
	0.539686, 0.610877, 0.579282,
	-0.577820, -0.602537, -0.550520,
	0.608007, 0.556630, 0.566119,
	-0.599020, -0.527335, -0.602572,
	0.556716, 0.540238, 0.631039,
	-0.532616, -0.571928, -0.623874,
	0.546419, 0.604364, 0.579803,
	-0.573361, -0.599398, -0.558551,
	0.599288, 0.562299, 0.569801,
	-0.592730, -0.537063, -0.600195,
	0.000000, 0.000000, 0.000000,
	-0.026593, 0.320928, -0.946730,
	0.555788, 0.598758, 0.576705,
	-0.571494, -0.596143, -0.563923,
	0.593099, 0.567207, 0.571411,
	-0.588152, -0.545827, -0.596783,
	0.556446, 0.552669, 0.620422,
	-0.541078, -0.571575, -0.616877,
	0.568848, 0.678956, 0.464146,
	-0.589642, -0.671803, -0.448334,
	0.641202, 0.612742, 0.461960,
	-0.590292, -0.547906, -0.592751,
	0.552949, 0.558138, 0.618651,
	-0.545258, -0.572592, -0.612235,
	0.000000, 0.000000, 0.000000,
	0.386432, -0.919981, 0.065614,
	0.613305, 0.540857, 0.575613,
	-0.607984, -0.533024, -0.588423,
	0.545153, 0.548442, 0.634050,
	-0.528528, -0.577069, -0.622615,
	0.542798, 0.617559, 0.569202,
	-0.569037, -0.607923, -0.553739,
	0.616161, 0.542816, 0.570698,
	-0.645071, -0.594608, -0.479921,
	0.592710, 0.616321, 0.518501,
	-0.574512, -0.640623, -0.509449,
	0.545079, 0.608548, 0.576679,
	-0.584889, -0.593410, -0.552965,
	0.000000, 0.000000, 0.000000,
	-0.930612, 0.245582, 0.271386,
	0.550041, 0.548943, 0.629378,
	-0.532723, -0.577504, -0.618623,
	0.550246, 0.599158, 0.581584,
	-0.577614, -0.589927, -0.564223,
	0.594661, 0.558062, 0.578745,
	-0.585450, -0.540703, -0.604060,
	0.556709, 0.552982, 0.619908,
	-0.543516, -0.573601, -0.612840,
	0.556916, 0.592489, 0.582066,
	-0.573585, -0.587362, -0.570970,
	0.587036, 0.563875, 0.580890,
	-0.580022, -0.549082, -0.601734,
	0.000000, 0.000000, 0.000000,
	0.085934, 0.266163, -0.960090,
	0.564876, 0.589291, 0.577626,
	-0.573031, -0.586894, -0.572006,
	0.584745, 0.567921, 0.579258,
	-0.578376, -0.553543, -0.599225,
	0.519019, 0.572736, 0.634502,
	-0.510002, -0.585688, -0.629974,
	0.575047, 0.687344, 0.443711,
	-0.613239, -0.669391, -0.419349,
	0.671791, 0.588119, 0.450347,
	-0.595885, -0.530494, -0.602908,
	0.537874, 0.553707, 0.635688,
	-0.526193, -0.582314, -0.619702,
	0.000000, 0.000000, 0.000000,
	0.330738, -0.921527, 0.203469,
	0.627182, 0.526236, 0.574212,
	-0.613595, -0.513917, -0.599492,
	0.530345, 0.545343, 0.649103,
	-0.510953, -0.587849, -0.627184,
	0.539399, 0.633129, 0.555154,
	-0.549728, -0.628274, -0.550519,
	0.598231, 0.547638, 0.584989,
	-0.648542, -0.591115, -0.479558,
	0.610140, 0.612401, 0.502686,
	-0.599903, -0.629451, -0.493870,
	0.552180, 0.599651, 0.579237,
	-0.582741, -0.584200, -0.564909,
	0.000000, 0.000000, 0.000000,
	-0.913515, 0.364833, 0.179963,
	0.551970, 0.558858, 0.618875,
	-0.543180, -0.577200, -0.609750,
	0.557476, 0.587845, 0.586224,
	-0.574249, -0.580025, -0.577762,
	0.581451, 0.561954, 0.588321,
	-0.574406, -0.553940, -0.602667,
	0.558926, 0.562675, 0.609096,
	-0.554083, -0.572179, -0.604651,
	0.564114, 0.580590, 0.587103,
	-0.570605, -0.577733, -0.583639,
	0.574856, 0.567949, 0.589045,
	-0.569610, -0.561410, -0.600303,
	0.000000, 0.000000, 0.000000,
	-0.738733, 0.478611, -0.474558,
	0.456682, 0.187644, -0.869616,
	0.267360, 0.262508, -0.927151,
	0.188011, -0.879407, 0.437373,
	0.254448, -0.926861, 0.276017,
	-0.926487, 0.230608, 0.297392,
	-0.919774, 0.275459, 0.279532,
	-0.580079, 0.486834, -0.653071,
	0.274544, 0.312551, -0.909361,
	0.493721, -0.659029, -0.567380,
	0.306020, -0.909456, 0.281497,
	-0.347952, -0.802140, 0.485284,
	-0.899099, 0.284109, 0.333020,
	0.564011, 0.684315, -0.462175,
	0.245460, -0.905008, 0.347433,
	0.635275, 0.508681, 0.581093,
	-0.612213, -0.497396, -0.614649,
	0.512418, 0.549887, 0.659584,
	-0.496431, -0.602073, -0.625352,
	0.543407, 0.644363, 0.538057,
	-0.531761, -0.651273, -0.541363,
	0.571407, 0.562336, 0.597722,
	-0.650552, -0.588777, -0.479713,
	0.633608, 0.601031, 0.487137,
	-0.629441, -0.610778, -0.480369,
	0.556710, 0.591696, 0.583069,
	-0.578125, -0.577601, -0.576323,
	0.358437, -0.338349, 0.870082,
	-0.956183, 0.248832, -0.154262,
	0.555510, 0.565898, 0.609236,
	-0.552748, -0.574352, -0.603812,
	0.560966, 0.577386, 0.593247,
	-0.567381, -0.573397, -0.591012,
	0.568467, 0.568655, 0.594539,
	-0.566133, -0.567182, -0.598162,
	0.563319, 0.569255, 0.598849,
	-0.563861, -0.567724, -0.599792,
	0.567273, 0.569159, 0.595197,
	-0.563929, -0.571163, -0.596454,
	0.562944, 0.574887, 0.593801,
	-0.561881, -0.574157, -0.595511,
	-0.376561, 0.262702, 0.888363,
	0.273442, 0.012154, -0.961812,
	0.575027, 0.569569, 0.587312,
	-0.569105, -0.573080, -0.589660,
	0.568633, 0.574613, 0.588623,
	-0.563074, -0.570649, -0.597752,
	0.439061, 0.641940, 0.628600,
	-0.438940, -0.642289, -0.628328,
	0.615365, 0.683070, 0.393372,
	-0.681202, -0.629765, -0.373309,
	0.721724, 0.520179, 0.456649,
	-0.584054, -0.514451, -0.627870,
	0.507431, 0.569248, 0.646893,
	-0.505332, -0.608317, -0.612038,
	0.473562, 0.714364, -0.515192,
	0.056842, -0.904717, 0.422204,
	0.629792, 0.497072, 0.596892,
	-0.600055, -0.492624, -0.630283,
	0.499185, 0.563715, 0.658057,
	-0.493530, -0.616190, -0.613789,
	0.556098, 0.643685, 0.525761,
	-0.519582, -0.671456, -0.528376,
	0.538191, 0.591034, 0.600857,
	-0.650470, -0.588917, -0.479651,
	0.661686, 0.578359, 0.477150,
	-0.660944, -0.581924, -0.473832,
	0.558397, 0.585728, 0.587465,
	-0.571478, -0.574563, -0.585911,
	0.537754, -0.378046, 0.753593,
	-0.893331, 0.313087, -0.322392,
	0.559992, 0.569636, 0.601601,
	-0.559981, -0.569718, -0.601534,
	0.559767, 0.569704, 0.601746,
	-0.557454, -0.571602, -0.602093,
	0.557426, 0.577882, 0.596094,
	-0.561924, -0.579025, -0.590739,
	0.569287, 0.571883, 0.590646,
	-0.570990, -0.561058, -0.599320,
	0.564900, 0.560505, 0.605576,
	-0.553745, -0.569456, -0.607525,
	0.553143, 0.584367, 0.593758,
	-0.558386, -0.585851, -0.587353,
	-0.411095, 0.353684, 0.840183,
	0.346940, -0.181817, -0.920095,
	0.573822, 0.561547, 0.596149,
	-0.563074, -0.570166, -0.598213,
	0.562200, 0.580483, 0.589042,
	-0.558794, -0.579495, -0.593241,
	0.422391, 0.683452, 0.595382,
	-0.422611, -0.678626, -0.600722,
	0.648415, 0.658680, 0.381704,
	-0.716985, -0.584729, -0.379505,
	0.728819, 0.488411, 0.479872,
	-0.568052, -0.527126, -0.632025,
	0.507208, 0.584012, 0.633774,
	-0.514657, -0.614785, -0.597635,
	-0.071468, 0.901397, -0.427055,
	-0.140234, -0.857880, 0.494344,
	0.610454, 0.500095, 0.614207,
	-0.580234, -0.504316, -0.639526,
	0.498263, 0.580891, 0.643661,
	-0.505932, -0.623613, -0.595936,
	0.571321, 0.630113, 0.525880,
	-0.514802, -0.686953, -0.512908,
	0.510021, 0.629920, 0.585730,
	-0.650767, -0.590433, -0.477380,
	0.687844, 0.544043, 0.480508,
	-0.687864, -0.544242, -0.480254,
	0.557826, 0.582835, 0.590875,
	-0.564590, -0.575258, -0.591875,
	0.929881, -0.367649, 0.012503,
	-0.732262, 0.555042, -0.394615,
	0.563759, 0.570483, 0.597264,
	-0.563149, -0.565446, -0.602607,
	0.554333, 0.567175, 0.609121,
	-0.546959, -0.575235, -0.608228,
	0.550925, 0.587603, 0.592626,
	-0.562381, -0.587085, -0.582287,
	0.574830, 0.570667, 0.586438,
	-0.573137, -0.554868, -0.603022,
	0.557015, 0.557869, 0.615237,
	-0.542448, -0.573649, -0.613740,
	0.548273, 0.594147, 0.588545,
	-0.560111, -0.593745, -0.577705,
	-0.310260, -0.117287, 0.943389,
	0.574603, -0.297746, -0.762351,
	0.567380, 0.558485, 0.605124,
	-0.555524, -0.571465, -0.604004,
	0.559080, 0.586392, 0.586151,
	-0.558192, -0.586427, -0.586962,
	0.431701, 0.717193, 0.547055,
	-0.428544, -0.710214, -0.558521,
	0.676298, 0.626508, 0.387439,
	-0.738852, -0.531413, -0.414365,
	0.721826, 0.465624, 0.512017,
	-0.555239, -0.549109, -0.624651,
	0.519834, 0.593314, 0.614615,
	-0.533029, -0.610930, -0.585359,
	-0.185737, 0.810406, -0.555648,
	-0.251767, -0.698172, 0.670201,
	0.585028, 0.519000, 0.623202,
	-0.561657, -0.528393, -0.636666,
	0.510878, 0.592704, 0.622660,
	-0.528675, -0.619277, -0.580515,
	0.580451, 0.610057, 0.539358,
	-0.517869, -0.698865, -0.493354,
	0.498763, 0.668827, 0.551277,
	-0.653470, -0.591224, -0.472686,
	0.702125, 0.504160, 0.502836,
	-0.702128, -0.504164, -0.502828,
	0.556802, 0.582939, 0.591738,
	-0.559814, -0.578290, -0.593456,
	0.883629, -0.436482, 0.169361,
	-0.556689, 0.610285, -0.563604,
	0.565001, 0.570359, 0.596209,
	-0.561856, -0.564242, -0.604937,
	0.547412, 0.570579, 0.612193,
	-0.539586, -0.582574, -0.607828,
	0.550395, 0.594684, 0.586017,
	-0.566122, -0.589853, -0.575829,
	0.577348, 0.567738, 0.586807,
	-0.569529, -0.552778, -0.608336,
	0.546680, 0.562874, 0.619930,
	-0.534139, -0.582264, -0.612914,
	0.549956, 0.600627, 0.580341,
	-0.565836, -0.595703, -0.570060,
	-0.408898, 0.041865, 0.911619,
	0.666160, -0.491381, -0.561049,
	0.558038, 0.562352, 0.610207,
	-0.549221, -0.576389, -0.605088,
	0.560215, 0.589854, 0.581576,
	-0.560921, -0.589624, -0.581128,
	0.468642, 0.729577, 0.498088,
	-0.460256, -0.724330, -0.513332,
	0.687472, 0.598090, 0.411911,
	-0.730459, -0.488515, -0.477266,
	0.701903, 0.461080, 0.542898,
	-0.554200, -0.573361, -0.603423,
	0.545232, 0.589367, 0.596128,
	-0.554008, -0.593845, -0.583458,
	-0.080605, 0.754773, -0.651015,
	-0.434306, -0.534543, 0.725012,
	0.562433, 0.552859, 0.614830,
	-0.554790, -0.557881, -0.617233,
	0.537314, 0.590296, 0.602366,
	-0.554400, -0.599564, -0.577203,
	0.575034, 0.590773, 0.565971,
	-0.527407, -0.704358, -0.475102,
	0.509528, 0.695487, 0.506635,
	-0.658851, -0.589800, -0.466960,
	0.696248, 0.472505, 0.540350,
	-0.698241, -0.473409, -0.536977,
	0.557692, 0.584487, 0.589368,
	-0.559167, -0.580946, -0.591468,
	0.827386, -0.482079, 0.288154,
	-0.333966, 0.636366, -0.695346,
	0.562819, 0.571984, 0.596716,
	-0.557756, -0.567979, -0.605234,
	0.543528, 0.577682, 0.608983,
	-0.538741, -0.589447, -0.601922,
	0.554588, 0.596236, 0.580461,
	-0.569726, -0.587832, -0.574339,
	0.575176, 0.566772, 0.589866,
	-0.562360, -0.557315, -0.610861,
	0.539804, 0.573050, 0.616624,
	-0.532946, -0.590682, -0.605857,
	0.556494, 0.600622, 0.574079,
	-0.571608, -0.591986, -0.568170,
	-0.484184, 0.161580, 0.859917,
	0.716137, -0.638380, -0.282167,
	0.551240, 0.571228, 0.608139,
	-0.547416, -0.581840, -0.601497,
	0.564117, 0.588893, 0.578772,
	-0.564499, -0.588664, -0.578632,
	0.517936, 0.717396, 0.465923,
	-0.506002, -0.716131, -0.480747,
	0.679236, 0.584192, 0.444250,
	-0.689818, -0.477596, -0.544107,
	0.678912, 0.474967, 0.559898,
	-0.570012, -0.587975, -0.573908,
	0.576224, 0.569022, 0.586668,
	-0.566284, -0.568088, -0.597159,
	-0.016546, 0.662088, -0.749243,
	-0.587692, -0.329523, 0.738941,
	0.552588, 0.591641, 0.587033,
	-0.566904, -0.578440, -0.586539,
	0.569347, 0.570198, 0.592214,
	-0.569553, -0.570217, -0.591998,
	0.552544, 0.581538, 0.597083,
	-0.538201, -0.704576, -0.462506,
	0.536587, 0.704474, 0.464534,
	-0.665905, -0.585976, -0.461739,
	0.673169, 0.459830, 0.579136,
	-0.680017, -0.459875, -0.571044,
	0.561932, 0.584990, 0.584824,
	-0.562743, -0.580564, -0.588443,
	0.765522, -0.510219, 0.391985,
	-0.093189, 0.625705, -0.774473,
	0.558328, 0.577138, 0.595971,
	-0.554149, -0.575958, -0.600991,
	0.545802, 0.583834, 0.601031,
	-0.544563, -0.591586, -0.594540,
	0.559361, 0.592446, 0.579761,
	-0.569631, -0.584156, -0.578172,
	0.569537, 0.570392, 0.591845,
	-0.556129, -0.567269, -0.607392,
	0.540627, 0.582359, 0.607108,
	-0.539136, -0.593940, -0.597132,
	0.562857, 0.594886, 0.573849,
	-0.573110, -0.586284, -0.572552,
	-0.548615, 0.261243, 0.794213,
	0.709329, -0.704480, 0.023651,
	0.551104, 0.579567, 0.600322,
	-0.550710, -0.584089, -0.596287,
	0.567170, 0.584515, 0.580224,
	-0.565717, -0.585796, -0.580351,
	0.560635, 0.690834, 0.456548,
	-0.548803, -0.692875, -0.467696,
	0.659007, 0.585734, 0.471833,
	-0.636905, -0.500048, -0.586774,
	0.663812, 0.497087, 0.558801,
	-0.598966, -0.585370, -0.546426,
	0.603229, 0.536250, 0.590382,
	-0.562542, -0.543208, -0.623275,
	0.002371, 0.507765, -0.861492,
	-0.700846, -0.113327, 0.704252,
	0.558947, 0.623521, 0.546626,
	-0.594315, -0.580405, -0.556704,
	0.596147, 0.537560, 0.596354,
	-0.566056, -0.543057, -0.620217,
	0.518874, 0.586293, 0.622118,
	-0.545660, -0.703367, -0.455555,
	0.570112, 0.698851, 0.431948,
	-0.673621, -0.580458, -0.457496,
	0.644068, 0.464843, 0.607534,
	-0.657294, -0.461430, -0.595859,
	0.569197, 0.582663, 0.580101,
	-0.568752, -0.576125, -0.587028,
	0.700646, -0.513091, 0.495815,
	0.134632, 0.583862, -0.800612,
	0.553738, 0.585565, 0.592020,
	-0.553731, -0.585565, -0.592027,
	0.553756, 0.585534, 0.592034,
	-0.553978, -0.587590, -0.589786,
	0.560925, 0.586296, 0.584483,
	-0.564616, -0.582293, -0.584930,
	0.563366, 0.578304, 0.590070,
	-0.554188, -0.578902, -0.598121,
	0.548449, 0.586345, 0.596157,
	-0.549108, -0.590570, -0.591361,
	0.565018, 0.587247, 0.579565,
	-0.569027, -0.582689, -0.580243,
	-0.608722, 0.350686, 0.711672,
	0.664140, -0.683019, 0.303977,
	0.557036, 0.583371, 0.591092,
	-0.556719, -0.581913, -0.592825,
	0.566956, 0.579592, 0.585349,
	-0.563352, -0.583925, -0.584523,
	0.588011, 0.661963, 0.464810,
	-0.579443, -0.665018, -0.471166,
	0.635135, 0.597727, 0.489209,
	-0.591980, -0.539999, -0.598298,
	0.660995, 0.516631, 0.544223,
	-0.633033, -0.566409, -0.527683,
	0.620475, 0.497599, 0.606140,
	-0.543493, -0.525756, -0.654367,
	-0.085316, 0.194599, -0.977165,
	-0.781593, 0.075936, 0.619149,
	0.578561, 0.642741, 0.502147,
	-0.627803, -0.564663, -0.535741,
	0.611975, 0.500113, 0.612677,
	-0.546096, -0.525357, -0.652517,
	0.481725, 0.602883, 0.635983,
	-0.548299, -0.704015, -0.451366,
	0.603595, 0.684307, 0.409142,
	-0.681765, -0.573726, -0.453912,
	0.617698, 0.479783, 0.623103,
	-0.637624, -0.470840, -0.609709,
	0.578280, 0.577022, 0.576748,
	-0.575052, -0.568119, -0.588690,
	0.647553, -0.442236, 0.620566,
	0.334247, 0.524815, -0.782846,
	0.550729, 0.595969, 0.584396,
	-0.557446, -0.594394, -0.579612,
	0.565028, 0.581981, 0.584844,
	-0.563591, -0.578601, -0.589564,
	0.557892, 0.580719, 0.592892,
	-0.555551, -0.584042, -0.591826,
	0.558855, 0.588732, 0.584017,
	-0.557297, -0.589162, -0.585071,
	0.560158, 0.584241, 0.587269,
	-0.559081, -0.582085, -0.590428,
	0.562069, 0.580900, 0.588756,
	-0.560618, -0.583084, -0.587980,
	-0.673057, 0.431508, 0.600662,
	0.624063, -0.552652, 0.552377,
	0.566275, 0.582100, 0.583517,
	-0.562838, -0.576318, -0.592512,
	0.563308, 0.576123, 0.592255,
	-0.558100, -0.584504, -0.588964,
	0.600244, 0.637656, 0.482806,
	-0.596886, -0.639320, -0.484765,
	0.615699, 0.610810, 0.497822,
	-0.565462, -0.577465, -0.588886,
	0.667065, 0.525319, 0.528265,
	-0.661795, -0.537842, -0.522258,
	0.624665, 0.464041, 0.628060,
	-0.515642, -0.520556, -0.680540,
	-0.443820, -0.467475, -0.764520,
	-0.942911, 0.020767, 0.332396,
	0.603955, 0.647222, 0.465126,
	-0.656094, -0.538863, -0.528363,
	0.614471, 0.469148, 0.634291,
	-0.517946, -0.521231, -0.678270,
	0.451875, 0.624663, 0.636872,
	-0.550098, -0.704616, -0.448228,
	0.631798, 0.664688, 0.398774,
	-0.688292, -0.564708, -0.455366,
	0.598237, 0.497130, 0.628470,
	-0.624536, -0.481357, -0.615021,
	0.587631, 0.568740, 0.575521,
	-0.579923, -0.558049, -0.593523,
	0.655144, 0.257234, 0.710364,
	0.482754, 0.449884, -0.751368,
	0.550071, 0.606662, 0.573919,
	-0.564702, -0.600848, -0.565768,
	0.576934, 0.574270, 0.580828,
	-0.570969, -0.566991, -0.593731,
	0.550853, 0.577464, 0.602575,
	-0.544384, -0.589418, -0.596851,
	0.556980, 0.599600, 0.574676,
	-0.564428, -0.596347, -0.570782,
	0.572564, 0.577470, 0.581978,
	-0.566593, -0.571115, -0.593970,
	0.555126, 0.577396, 0.598706,
	-0.550140, -0.587331, -0.593623,
	-0.833154, 0.422159, 0.357261,
	0.651675, 0.278644, 0.705462,
	0.576158, 0.577021, 0.578869,
	-0.567426, -0.569060, -0.595145,
	0.557265, 0.574828, 0.599190,
	-0.551396, -0.587565, -0.592225,
	0.600952, 0.621220, 0.502934,
	-0.603515, -0.619663, -0.501785,
	0.602573, 0.621505, 0.500637,
	-0.554518, -0.605002, -0.571387,
	0.676814, 0.524358, 0.516693,
	-0.681907, -0.507470, -0.526761,
	0.619755, 0.438988, 0.650533,
	-0.486264, -0.524733, -0.698715,
	0.167903, 0.242626, -0.955480,
	0.573284, 0.574842, 0.583869,
	0.565669, 0.590009, 0.576114,
	-0.565461, -0.583204, -0.583204,
	0.700041, 0.504947, 0.504947,
	-0.609614, -0.609614, -0.506697,
	-0.624517, -0.624518, 0.468994,
	-0.670871, 0.287076, -0.683753,
	-0.441663, 0.776083, -0.450143,
	0.669459, -0.630378, -0.392999,
	0.695803, -0.609488, -0.379976,
	-0.571163, -0.356083, 0.739579,
	-0.823256, -0.513246, 0.242543,
	-0.140600, 0.962017, -0.233999,
};

void 
computeNormal (float x0, float y0, float z0, 
	       float x1, float y1, float z1, 
	       float x2, float y2, float z2, 
	       float *nx, float *ny, float *nz)
{
  // Vecteur 12
  float ax, ay, az;
  // Vecteur 10
  float bx, by, bz, norm, nnx, nny, nnz;
  
  ax = x2 - x1;
  ay = y2 - y1;
  az = z2 - z1;
  bx = x0 - x1;
  by = y0 - y1;
  bz = z0 - z1;
  
  // Normale = resultat du produit vectoriel entre les vecteurs a et b
  nnx = ay*bz - az*by;
  nny = az*bx - ax*bz;
  nnz = ax*by - ay*bx;

  norm = (float)sqrt(nnx * nnx + nny * nny + nnz * nnz);
  if(norm < 0.000000001f) *nx = *ny = *nz = 0.0f;
  else{
  *nx = nnx / norm;
  *ny = nny / norm;
  *nz = nnz / norm;
  }

  // Si on ne la normalise pas, il faut activer GL_NORMALIZE
}

static int done = 0;

void FGAPIENTRY glutSolidTeapot(GLdouble size)
{
	float s = size / 16.0f;

	glPushMatrix();

	glScalef(s, s, s);
	
	if(!done)
	{
		int i;
		FILE *f = fopen("normals.txt","wt");
		fprintf(f, "float teapot_normals[3136 * 3] = {\n");
		for(i = 0; i < 3136; i++)
		{
			float nx, ny, nz;
			unsigned short v1, v2, v3;
			v3 = teapot_faces[i * 3 + 0];
			v2 = teapot_faces[i * 3 + 1];
			v1 = teapot_faces[i * 3 + 2];

			computeNormal(teapot_vertex[v1 + 0], teapot_vertex[v1 + 1], teapot_vertex[v1 + 2], 
				teapot_vertex[v2 + 0], teapot_vertex[v2 + 1], teapot_vertex[v2 + 2], 
				teapot_vertex[v3 + 0], teapot_vertex[v3 + 1], teapot_vertex[v3 + 2],
				&nx, &ny, &nz);
			teapot_normals[i * 3 + 0] = nx;
			teapot_normals[i * 3 + 1] = ny;
			teapot_normals[i * 3 + 2] = nz;

			fprintf(f, "\t%f, %f, %f,\n", nx, ny, nz);
		}
		fprintf(f, "};"); 
		fclose(f);
		done = 1;
	}
	
	glVertexPointer(3, GL_FLOAT, 0, teapot_vertex);
	glEnableClientState(GL_VERTEX_ARRAY);
	glNormalPointer(GL_FLOAT, 0, teapot_normals);
	glEnableClientState(GL_NORMAL_ARRAY);

	glDrawElements(GL_TRIANGLES, 3136*3, GL_UNSIGNED_SHORT, teapot_faces);
	
	glDisableClientState (GL_VERTEX_ARRAY);

	glPopMatrix();
}

#endif

